/* 
 * 
 * eulerGlyphs v0.9.0
 * 
 * 2013-12-18
 *
 * 
 * 
 * eulerGlyphs -- Drawing Area-Proportional Euler Diagrams With Glyphs	    
 * 		http://www.eulerdiagrams.org/eulerGlyphs
 * 
 * 
 * 		Copyright (C) 2012-2013, Luana Micallef. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerGlyphs.
 * 			
 * 		eulerGlyphs is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerGlyphs is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerGlyphs (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */

 


package elliptic.areaproptool;


import java.awt.Polygon;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;


/**
 * Geometric Operations
 * 
 */


public class GeometricOps {


	// Change coordinate systems 

	public static double getPolarCoordAngForPnt (Point2D.Double pnt, Point2D.Double centre){
		// outputs angle in radians form
		
		// check if null to avoid exceptions
		if (centre == null){
			centre = new Point2D.Double (0, 0);
		}

		double ang = 0.0;

		double dx = Math.abs (pnt.x - centre.x);
		double dy = Math.abs (pnt.y - centre.y);

		double unverifiedAng = Math.abs(Math.atan(dy/dx));
		if ((pnt.y > centre.y) && (pnt.x > centre.x)){  // right, top -> +ve x, +ve y
			ang = unverifiedAng;
		} else if ((pnt.y > centre.y) && (pnt.x < centre.x)){    // left, top -> -ve x, +ve y
			ang = Math.PI - unverifiedAng;
		} else if ((pnt.y < centre.y) && (pnt.x < centre.x)){    // left, bottom -> -ve x, -ve y
			ang = Math.PI + unverifiedAng;
		} else if ((pnt.y < centre.y) && (pnt.x > centre.x)){    // right, bottom -> +ve x, -ve y
			ang = (2*Math.PI) - unverifiedAng;
		} else if ((dx == 0) && (pnt.y > centre.y)){   // same x, pnt ABOVE centre => 90 degrees
			ang = (Math.PI /2);  //unverifiedAng;   // or (Math.PI /2);
		} else if ((dx == 0) && (pnt.y < centre.y)){   // same x, pnt BELOW centre => 270 degrees
			ang = (3 * Math.PI /2);  //Math.PI + unverifiedAng;   // or (3 * Math.PI /2);
		} else if ((dy == 0) && (pnt.x > centre.x)){   // same y, pnt RIGHT to centre => 0 degrees
			ang = 0;  //unverifiedAng;   // or 0;
		} else if ((dy == 0) && (pnt.x < centre.x)){   // same y, pnt LEFT to centre => 180 degrees
			ang = Math.PI; //Math.PI + unverifiedAng;  // or Math.PI;
		}

		return  ang;
	}

	public static double getPolarCoordRForPnt (Point2D.Double pnt, Point2D.Double centre){

		// check if null to avoid exceptions
		if (centre == null){
			centre = new Point2D.Double (0, 0);
		}

		double dx = Math.abs (pnt.x - centre.x);
		double dy = Math.abs (pnt.y - centre.y);

		return (Math.sqrt(Math.pow(dx, 2) + Math.pow(dy, 2)));
	}

	
	// Points
	
	public static double distanceBetween2Points (Point2D.Double pnt1, Point2D.Double pnt2){
		
		double diff_x = Math.abs(pnt2.x - pnt1.x);
		double diff_y = Math.abs(pnt2.y - pnt1.y);
	
		return Math.sqrt(Math.pow(diff_x,2) + Math.pow(diff_y,2));	
	}
	
	
	public static Point2D.Double getPerpPntOnParallelLine (Line2D.Double line, double perpDistBetLines, Point2D.Double pnt, boolean newPntAboveLine){
		// this method could have been placed in MyLine as a method operating on the initial line 
		// ... but since we do not need the equation of the line (and the gradient and y-intercept)
		// ... then I decided to put it here
		
		double ydDIVxd = (line.y1-line.y2)/(line.x1-line.x2);
		
		double lnSlopeAsAng_rad = Math.atan(Math.abs(ydDIVxd));

		// the slope of the parallel line that is perpDistBetLines away from the given line as an angle
		double parLnSlopeAsAng_rad = Math.toRadians(90) - lnSlopeAsAng_rad;
		
		double diffInX = perpDistBetLines * Math.cos(parLnSlopeAsAng_rad);
		double diffInY = perpDistBetLines * Math.sin(parLnSlopeAsAng_rad);
		if (ydDIVxd > 0){ // => +ve gradient
			if (newPntAboveLine){
				return (new Point2D.Double(pnt.x - diffInX, pnt.y + diffInY));
			} else {
				return (new Point2D.Double(pnt.x + diffInX, pnt.y - diffInY));
			}

		} else { // => -ve gradient
			if (newPntAboveLine){
				return (new Point2D.Double(pnt.x + diffInX, pnt.y + diffInY));
			} else {
				return (new Point2D.Double(pnt.x - diffInX, pnt.y - diffInY));
			}
		}

	}
	
	
	

	// Lines 
	
	public static Point2D.Double getIntPntsOfLines (Line2D.Double line1, Line2D.Double line2) {
	    Point2D.Double intpnt = new Point2D.Double(0.0,0.0);
	    
		if (! line1.intersectsLine(line2) ){
			return null;

		} else {

			double l1x1 = line1.x1;
			double l1y1 = line1.y1;
			double l1xd = line1.x2-l1x1;
			double l1yd = line1.y2-l1y1;

			double l2x1 = line2.x1;
			double l2y1 = line2.y1;
			double l2xd = line2.x2-l2x1;
			double l2yd = line2.y2-l2y1;

			double det = - (l1yd * l2xd) + (l2yd * l1xd);

			if (det == 0) {
				return null;
			} else {
				double A1 = l1yd;
				double A2 = l2yd;
				double B1 = - l1xd;
				double B2 = - l2xd;
				double C1 = (l1yd * l1x1) - (l1xd * l1y1);
				double C2 = (l2yd * l2x1) - (l2xd * l2y1);

				intpnt.x = ((B2*C1) - (B1*C2))/det;
				intpnt.y = ((A1*C2) - (A2*C1))/det;

				return intpnt;
			}
		}
	}
	
	
	// Triangles 
	public static double areaOfTriangle (Point2D.Double pnt1, Point2D.Double pnt2, Point2D.Double base_pnt){
		Point2D.Double pnt1FromBasePnt = new Point2D.Double (pnt1.x - base_pnt.x, pnt1.y - base_pnt.y);
		Point2D.Double pnt2FromBasePnt = new Point2D.Double (pnt2.x - base_pnt.x, pnt2.y - base_pnt.y);
		double area = 0.5 * Math.abs((pnt1FromBasePnt.x * pnt2FromBasePnt.y) - (pnt1FromBasePnt.y * pnt2FromBasePnt.x));
		return area;
	}
	
	
	// Polygons 
	public static boolean isPointInPolygon(Polygon poly, Point2D.Double pnt){
		//if ((poly==null)||(pnt==null)){return null;}
		
		double pntPosVar = 0.5;
		double sqPntPosVarWidth = pntPosVar*2;
		
		return ( poly.contains(pnt.x, pnt.y) || 
			     poly.intersects(pnt.x-pntPosVar, pnt.y-pntPosVar, sqPntPosVarWidth, sqPntPosVarWidth) );
	}
	
}
