/* 
 * 
 * eulerGlyphs v0.9.0
 * 
 * 2013-12-18
 *
 * 
 * 
 * eulerGlyphs -- Drawing Area-Proportional Euler Diagrams With Glyphs	    
 * 		http://www.eulerdiagrams.org/eulerGlyphs
 * 
 * 
 * 		Copyright (C) 2012-2013, Luana Micallef. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerGlyphs.
 * 			
 * 		eulerGlyphs is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerGlyphs is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerGlyphs (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */

 


package elliptic.areaproptool;

import java.util.ArrayList;


/**
 * 
 * To find the roots of equations
 *
 */


public class RootFinder {
	
	
	public static ArrayList<Double> solveLinear (ArrayList<Double> cs){
		return solveLinear(Utilities.convertDblArrayListToArray(cs));
	}
	public static ArrayList<Double> solveLinear (double[] c){
		ArrayList<Double> s = new ArrayList<Double>();
		
		if (isZero(c[1])){return null;}
		
		s.add(-c[0]/c[1]);
		
		return s;
	}
	
	
	public static ArrayList<Double> solveCubic (ArrayList<Double> cs){
		return solveCubic(Utilities.convertDblArrayListToArray(cs));
	}
	public static ArrayList<Double> solveCubic (double[] c){
		
		// normalize the equation:x ^ 3 + Ax ^ 2 + Bx  + C = 0
		double A = c[2] / c[3];
		double B = c[1] / c[3];
		double C = c[0] / c[3];

		// substitute x = y - A / 3 to eliminate the quadric term: x^3 + px + q = 0
		double sq_A = A * A;
		double p = (1.0/3.0) * (((-1.0/3.0) * sq_A) + B);
		double q = (1.0/2.0) * (((2.0/27.0) * A *sq_A) - ((1.0/3.0) * A * B) + C);
		
		
		// use Cardano's formula
		
		double cb_p = p * p * p;
		double D = (q * q) + cb_p;
		
		ArrayList<Double> s = new ArrayList<Double>(3); //3 is the initial capacity which will grow automatically if necessary
		
		if (isZero(D))
	    {
			if (isZero(q)) {
				// one triple solution
				s.add(0.0);
				//num = 1;
			} else {
				// one single and one double solution
				double u = Math.cbrt(-q);
				s.add(2.0 * u);
				s.add(- u);
				//num = 2;
			}
			
	    } else if (D < 0.0) {
	    	// three real solutions
	    	double phi = (1.0/3.0) * Math.acos(-q / Math.sqrt(-cb_p)); //output of Math.acos is in radians
	    	double t = 2.0 * Math.sqrt(-p);
	    	s.add (t * Math.cos(phi));
	    	s.add (-t * Math.cos(phi + (Math.PI / 3.0)));
	    	s.add(-t * Math.cos(phi - (Math.PI / 3.0)));
	    	//num = 3;
	    	
		} else {
			// one real solution
			double sqrt_D = Math.sqrt(D);
			double u = Math.cbrt(sqrt_D + Math.abs(q)); //instead of Math.cbrt
			if (q > 0.0){
				s.add(- u + (p / u));
			} else {
				s.add(u - (p / u));
			}
			//num = 1;
		}
		
	    // resubstitute
	    double sub = 1.0 / 3.0 * A;
	    int i = 0;
	   	for (Double s_1 : s){
	   	    s.set(i, s_1-sub);
	   	    i++;
	   	}
	   
	   	return s;
	}
	
	
	public static ArrayList<Double> solveQuadric (ArrayList<Double> cs){
		return solveQuadric(Utilities.convertDblArrayListToArray(cs));
	}
	public static ArrayList<Double> solveQuadric (double[] coeff){
		
		if (isZero(coeff[2])){
			return solveLinear(coeff);
		}
		
		ArrayList<Double> s = new ArrayList<Double>(2); //2 is the initial capacity which will grow automatically if necessary

		// normal for: x^2 + px + q
		double p = coeff[1] / (2.0 * coeff[2]);
		double q = coeff[0] / coeff[2]; 
		double D = (p * p) - q;

		if (isZero(D)){
			// one double root
			s.add(-p);
			s.add(-p);
			return s;
	    }

		if (D < 0.0){
			// no real root
			return null;
		} else {
			// two real roots
			double sqrt_D = Math.sqrt(D);
			s.add(sqrt_D - p);
			s.add(-sqrt_D - p);			
			return s;
	    }
	   
		
	}

		
	private static boolean isZero(double x){
		double EQN_EPS = 1e-11;
		return ((x > -EQN_EPS) && (x < EQN_EPS));
	}
	
	
	
	
}
	

