/* 
 * 
 * eulerGlyphs v0.9.0
 * 
 * 2013-12-18
 *
 * 
 * 
 * eulerGlyphs -- Drawing Area-Proportional Euler Diagrams With Glyphs	    
 * 		http://www.eulerdiagrams.org/eulerGlyphs
 * 
 * 
 * 		Copyright (C) 2012-2013, Luana Micallef. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerGlyphs.
 * 			
 * 		eulerGlyphs is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerGlyphs is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerGlyphs (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */

 


package eulerellipse;

import java.awt.Rectangle;
import java.awt.geom.Area;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

import uicomponent.vispanel.VisUtils.Placement;
import utils.GeometryUtils;
import utils.ReflectionUtils;

import elliptic.areaproptool.Ellipse;
import elliptic.areaproptool.EllipseDiagram;
import elliptic.areaproptool.EllipseDiagramOps;


/**
 * Contains operations and utilities related to the actual visualization of {@link #EulerEllipseDiagram(???)}
 * 
 */


public class EulerEllipseVisUtils {

	
	// ideally these methods should be in their respective object class such as EulerEllipseDiagram and Ellipse
	
	public static Rectangle getEllipseBoundingBox (Ellipse ellipse, Point2D.Double centreOfBox, boolean coordSys_originMid_NOT_originTopLeft){ 
		//since we are using the getBoundingBox of Java then the coordinate system is as follows: x incr to right, y incr down
		Area setAsArea = EllipseDiagramOps.getSetAsArea(ellipse, centreOfBox);
		Rectangle setAsAreaBounds_yIncrDown = setAsArea.getBounds();
		
		if (!coordSys_originMid_NOT_originTopLeft){
			return setAsAreaBounds_yIncrDown;
		} else {
			Point2D.Double centreOfSystem = new Point2D.Double(720/2, 650/2);
			Point2D.Double convertedPnt = GeometryUtils.changeCoorSys_originTopLeft_to_originMid(new Point2D.Double(new Double(setAsAreaBounds_yIncrDown.x),new Double(setAsAreaBounds_yIncrDown.y)), centreOfSystem);
			return (new Rectangle((int)convertedPnt.x, (int)convertedPnt.y, setAsAreaBounds_yIncrDown.width, setAsAreaBounds_yIncrDown.height));
		}
	}
	
	public static Rectangle2D.Double getDiagramBoundingBox (EllipseDiagram diag, Point2D.Double centreOfBox, boolean coordSys_originMid_NOT_originTopLeft){ 
		// work out the bounding box using the default coordinate system that is y incr down and then if the opposite required, change it before returning
		ArrayList<Ellipse> ellipses = diag.getEllipses();
	    Rectangle e0Box = getEllipseBoundingBox(ellipses.get(0), centreOfBox, false);
	    
	    double leftmost_x = e0Box.x;
	    double topmost_y   = e0Box.y;
	    double rightmost_x   = e0Box.x + e0Box.width;
	    double bottommost_y = e0Box.y + e0Box.height;

	    Rectangle eIBox;
	    double currEll_left_x;
	    double currEll_top_y;
	    double currEll_right_x;
	    double currEll_bottom_y;
	    
	    for (int i = 1; i < ellipses.size(); i++) {
	    	eIBox = getEllipseBoundingBox(ellipses.get(i), centreOfBox, false);
	    	
	    	currEll_left_x = eIBox.x;
	    	currEll_top_y   = eIBox.y;
	    	currEll_right_x   = eIBox.x + eIBox.width;
	    	currEll_bottom_y = eIBox.y + eIBox.height;
	    	
		    leftmost_x = (currEll_left_x < leftmost_x) ? currEll_left_x : leftmost_x;
		    topmost_y   = (currEll_top_y < topmost_y)   ? currEll_top_y : topmost_y;
		    rightmost_x   = (currEll_right_x > rightmost_x)   ? currEll_right_x : rightmost_x;
		    bottommost_y = (currEll_bottom_y > bottommost_y) ? currEll_bottom_y : bottommost_y;
	    }
		
	    Point2D.Double leftBottomPnt_originTopLeft = new Point2D.Double (leftmost_x, bottommost_y);
	    Point2D.Double rightTopPnt_originTopLeft = new Point2D.Double (rightmost_x, topmost_y);
	    if (!coordSys_originMid_NOT_originTopLeft){
	    	return new Rectangle2D.Double(leftBottomPnt_originTopLeft.x, rightTopPnt_originTopLeft.y, Math.abs(leftBottomPnt_originTopLeft.x-rightTopPnt_originTopLeft.x), Math.abs(leftBottomPnt_originTopLeft.y-rightTopPnt_originTopLeft.y));
	    } else {
	    	Point2D.Double leftBottomPnt_originMid = GeometryUtils.changeCoorSys_originTopLeft_to_originMid(leftBottomPnt_originTopLeft, centreOfBox);
	    	Point2D.Double rightTopPnt_originMid = GeometryUtils.changeCoorSys_originTopLeft_to_originMid(rightTopPnt_originTopLeft, centreOfBox);
	    	return new Rectangle2D.Double(leftBottomPnt_originMid.x, rightTopPnt_originMid.y, Math.abs(leftBottomPnt_originMid.x-rightTopPnt_originMid.x), Math.abs(leftBottomPnt_originMid.y-rightTopPnt_originMid.y)); 	
	    }
	    	
	}
	
	public static Ellipse getEllipseCircumscribingRectangles2(Rectangle2D.Double[] rectangles, String ellipseRefLabel, boolean useCartesianCoordSystem){
		// the flag useCartesianCoordSystem is ignored as the cartesian coordinate system is assumed throughout
		
		Rectangle2D.Double bboxForRects = GeometryUtils.getBoundingBoxForRectangles(rectangles, useCartesianCoordSystem);
		
		Point2D.Double bboxCentre = new Point2D.Double(bboxForRects.x+(bboxForRects.getWidth()/2), bboxForRects.y - (bboxForRects.getHeight()/2));
		double rot = 0.0;
		
		double circleRadius = 0;
		double xToComputeCircleRadius = 0;
		double yToComputeCircleRadius = 0;
		double centreForOtherRect_x = 0;
		double xToComputeCentreForOtherRect = 0;
		double yToComputeCentreForOtherRect = 0;
		
		//assuming that that rectangles passed on as input are sorted out in order from left to right according to their positioning
		if (rectangles[0].y <= rectangles[1].y){
			xToComputeCircleRadius = rectangles[0].x;
			yToComputeCircleRadius = rectangles[0].y;
			xToComputeCentreForOtherRect = rectangles[1].x + rectangles[1].getWidth();
			yToComputeCentreForOtherRect = rectangles[1].y;
		} else {
			xToComputeCircleRadius = rectangles[1].x + rectangles[1].getWidth();
			yToComputeCircleRadius = rectangles[1].y;
			xToComputeCentreForOtherRect = rectangles[0].x;
			yToComputeCentreForOtherRect = rectangles[0].y;
		}
		
		circleRadius = Math.sqrt(Math.pow(xToComputeCircleRadius-bboxCentre.x,2) + Math.pow(yToComputeCircleRadius-bboxCentre.y,2));
		centreForOtherRect_x = xToComputeCentreForOtherRect - Math.sqrt(Math.pow(circleRadius,2) - Math.pow(yToComputeCentreForOtherRect - bboxCentre.y, 2)); 
		 
		double diffBetCentresOfBBoxAndCircleForOtherRect =  bboxCentre.x - centreForOtherRect_x;
		Point2D.Double centre = new Point2D.Double(bboxCentre.x - (diffBetCentresOfBBoxAndCircleForOtherRect/2), bboxCentre.y);
		double semiHorizAxis = circleRadius + (Math.abs(diffBetCentresOfBBoxAndCircleForOtherRect)/2);
		double semiVertAxis = Ellipse.computeSemiVerticalAxisGivenRotationZero(xToComputeCentreForOtherRect, yToComputeCentreForOtherRect, centre.x, centre.y, semiHorizAxis); 
			
		return new Ellipse(ellipseRefLabel, semiHorizAxis, semiVertAxis, centre.x, centre.y, rot);
	}
	
	public static Ellipse getEllipseCircumscribingRectangles (Rectangle2D.Double[] rectangles, String ellipseRefLabel, boolean useCartesianCoordSystem){
		  // the coordinate system used depends on the input flag useCartesianCoordSystem
		  // -> if useCartesianCoordSystem = true then the origin is at the centre (x increases to the right, y increases upwards) 
		  //    else the origin is at the top left corner (x increases to the right, y increases downwards) 
		  // this will affect the returned value
		Rectangle2D.Double bboxForRects = GeometryUtils.getBoundingBoxForRectangles(rectangles, useCartesianCoordSystem);
		return getEllipseCircumscribingRectangle(bboxForRects, ellipseRefLabel, useCartesianCoordSystem);
	}
	public static Ellipse getEllipseCircumscribingRectangle (Rectangle2D.Double rectangle, String ellipseRefLabel, boolean useCartesianCoordSystem){
		// this uses the concept that the 
		// width of ellipse : height of ellipse =  width of maxInscribedRectangle : height of maxInscribedRectangle
		// => 2*semiHorizAxis/2*semiVertAxis = rectWidth/rectHeight (or just  semiHorizAxis/semiVertAxis = rectWidth/rectHeight)
		
		// the coordinate system used depends on the in put flag useCartesianCoordSystem
		// -> if useCartesianCoordSystem = true then the origin is at the centre (x increases to the right, y increases upwards) 
		//    else the origin is at the top left corner (x increases to the right, y increases downwards) 
		// this will affect the returned value
		
		double rectWidth = rectangle.getWidth();
		double rectHeight = rectangle.getHeight();
		
		Point2D.Double centre = new Point2D.Double(rectangle.x+(rectWidth/2), rectangle.y + ((useCartesianCoordSystem ? -1 : 1) * (rectHeight/2)));
		double rot = 0.0;
		
		// using the principle that semiHorizAxis/semiVertAxis = rectWidth/rectHeight
		// and the equation of the ellipse when rotation is 0:  ((x-xc)^2)/a^2  +  ((y-yc)^2)/b^2 = 1
		
		// Set semiHorizAxis first 
		double semiHorizAxis = Math.sqrt((Math.pow(rectHeight,2)*Math.pow(rectangle.x-centre.x,2)) + (Math.pow(rectWidth,2)*Math.pow(rectangle.y-centre.y,2)))/rectHeight;
		double semiVertAxis = (rectHeight * semiHorizAxis)/rectWidth;
		
		return new Ellipse(ellipseRefLabel, semiHorizAxis, semiVertAxis, centre.x, centre.y, rot);
	}
	
	public static Ellipse getCircleCircumscribingRectangles (Rectangle2D.Double[] rectangles, String ellipseRefLabel, boolean useCartesianCoordSystem){
		  // the coordinate system used depends on the input flag useCartesianCoordSystem
		  // -> if useCartesianCoordSystem = true then the origin is at the centre (x increases to the right, y increases upwards) 
		  //    else the origin is at the top left corner (x increases to the right, y increases downwards) 
		  // this will affect the returned value
		Rectangle2D.Double bboxForRects = GeometryUtils.getBoundingBoxForRectangles(rectangles, useCartesianCoordSystem);
		return getCircleCircumscribingRectangle(bboxForRects, ellipseRefLabel, useCartesianCoordSystem);
	}
	public static Ellipse getCircleCircumscribingRectangle (Rectangle2D.Double rectangle, String ellipseRefLabel, boolean useCartesianCoordSystem){
		// the coordinate system used depends on the in put flag useCartesianCoordSystem
		// -> if useCartesianCoordSystem = true then the origin is at the centre (x increases to the right, y increases upwards) 
		//    else the origin is at the top left corner (x increases to the right, y increases downwards) 
		// this will affect the returned value
		
		double rectWidth = rectangle.getWidth();
		double rectHeight = rectangle.getHeight();
		
		Point2D.Double centre = new Point2D.Double(rectangle.x+(rectWidth/2), rectangle.y + ((useCartesianCoordSystem ? -1 : 1) * (rectHeight/2)));
		double rot = 0.0;
		
		/*
		// since a straight line y=x passing through two corners of the square (which corners are also points on the circumscribed circle) and the centre of the square 
		// is 45 degrees to a horizontal line y=0 parallel to the width of the square and passing through its centre 
		// then  (width/2) = (sqrt(2)/2) * radius of circle   and   (height/2) = (sqrt(2)/2) * radius of circle
		// =>  radius of circle = (2/sqrt(2)) * (side of square/2) = side of square / sqrt(2)
		double semiVertAxis = rectWidth / Math.sqrt(2);
		*/
		double semiVertAxis = Math.sqrt(Math.pow(rectangle.x-centre.x,2) + Math.pow(rectangle.y-centre.y,2));
		double semiHorizAxis = semiVertAxis;
		
		return new Ellipse(ellipseRefLabel, semiHorizAxis, semiVertAxis, centre.x, centre.y, rot);
	}
	
	
	public static EllipseDiagram translateDiagramInCircle(EllipseDiagram containedDiagram, Ellipse containingCircle, Placement placementInCircle){
		// the contained diagram can be anywhere inside the containing circle
		// using a Cartesian coordinate system such that the origin is at the centre and thus x increments to the right and y increments upwards
		
		// check if containing circle passed on as an input is really a circle
		// .. if not ideally throw an exception or output an error message
		if (containingCircle.getA() != containingCircle.getB()){return null;}
		
		Rectangle2D.Double diagBBox = getDiagramBoundingBox(containedDiagram, new Point2D.Double(containingCircle.getXc(),containingCircle.getYc()), true);
		
		double perpDistChordFromCircleCentre;
		Point2D.Double topleftCoordOfTranslatedDiagBBox = new Point2D.Double(0,0);
		Point2D.Double translateBy = new Point2D.Double(0,0);
		switch (placementInCircle) {
			case CENTRE: { 
				topleftCoordOfTranslatedDiagBBox.x = containingCircle.getXc()-(diagBBox.getWidth()/2);
				topleftCoordOfTranslatedDiagBBox.y = containingCircle.getYc()+(diagBBox.getHeight()/2);	
			}
			case NORTH: {
				perpDistChordFromCircleCentre = GeometryUtils.computePerpendicularDistanceOfCircleChordFromCentre(containingCircle.getA(), diagBBox.getWidth());
				topleftCoordOfTranslatedDiagBBox.x = containingCircle.getXc() - (diagBBox.getWidth()/2);
				topleftCoordOfTranslatedDiagBBox.y = containingCircle.getYc() + perpDistChordFromCircleCentre;	
				break;
			}
			case SOUTH: {
				perpDistChordFromCircleCentre = GeometryUtils.computePerpendicularDistanceOfCircleChordFromCentre(containingCircle.getA(), diagBBox.getWidth());
				topleftCoordOfTranslatedDiagBBox.x = containingCircle.getXc() - (diagBBox.getWidth()/2);
				topleftCoordOfTranslatedDiagBBox.y = containingCircle.getYc() - perpDistChordFromCircleCentre + diagBBox.getHeight();	
				break;
			}
			case WEST: {
				perpDistChordFromCircleCentre = GeometryUtils.computePerpendicularDistanceOfCircleChordFromCentre(containingCircle.getA(), diagBBox.getHeight());
				topleftCoordOfTranslatedDiagBBox.x = containingCircle.getXc() - perpDistChordFromCircleCentre;	
				topleftCoordOfTranslatedDiagBBox.y = containingCircle.getYc() + (diagBBox.getHeight()/2);
				break;	
			}
			case EAST: {
				perpDistChordFromCircleCentre = GeometryUtils.computePerpendicularDistanceOfCircleChordFromCentre(containingCircle.getA(), diagBBox.getHeight());
				topleftCoordOfTranslatedDiagBBox.x = containingCircle.getXc() + perpDistChordFromCircleCentre - diagBBox.getWidth();	
				topleftCoordOfTranslatedDiagBBox.y = containingCircle.getYc() + (diagBBox.getHeight()/2);
				break;	
			}
			default:
				break;
		}
		
		translateBy.x += (topleftCoordOfTranslatedDiagBBox.x-diagBBox.x);
		translateBy.y += (topleftCoordOfTranslatedDiagBBox.y-diagBBox.y);
		
		ArrayList<Ellipse> containedEllipses = containedDiagram.getEllipses();
		for (Ellipse containedEllipse : containedEllipses){
			containedEllipse.setXc(containedEllipse.getXc()+translateBy.x);
			containedEllipse.setYc(containedEllipse.getYc()+translateBy.y);
		}

		return containedDiagram;
	}
	
	public static EllipseDiagram translateDiagramInRectangle (EllipseDiagram containedDiagram, Rectangle2D.Double containingRect, Placement placementInRect){
		// using a Cartesian coordinate system such that the origin is at the centre and thus x increments to the right and y increments upwards
		
		Point2D.Double containingRectCentre = new Point2D.Double((containingRect.x + (containingRect.width/2)), (containingRect.y - (containingRect.height/2))); 
		Rectangle2D.Double diagBBox = getDiagramBoundingBox(containedDiagram, new Point2D.Double(containingRectCentre.x,containingRectCentre.y), true);
		
		Point2D.Double topleftCoordOfTranslatedDiagBBox = new Point2D.Double(0,0);
		Point2D.Double translateBy = new Point2D.Double(0,0);
		
		switch (placementInRect) {
			case CENTRE: { 
				topleftCoordOfTranslatedDiagBBox.x = containingRectCentre.x -(diagBBox.getWidth()/2);
				topleftCoordOfTranslatedDiagBBox.y = containingRectCentre.y +(diagBBox.getHeight()/2);
			}
			case NORTH: {
				topleftCoordOfTranslatedDiagBBox.x = containingRectCentre.x -(diagBBox.getWidth()/2);
				topleftCoordOfTranslatedDiagBBox.y = containingRectCentre.y + (containingRect.height/2);	
				break;
			}
			case SOUTH: {
				topleftCoordOfTranslatedDiagBBox.x = containingRectCentre.x - (diagBBox.getWidth()/2);
				topleftCoordOfTranslatedDiagBBox.y = containingRectCentre.y - (containingRect.height/2) + diagBBox.getHeight();	
				break;
			}
			case WEST: {
				topleftCoordOfTranslatedDiagBBox.x = containingRectCentre.x - (containingRect.width/2);	
				topleftCoordOfTranslatedDiagBBox.y = containingRectCentre.y + (diagBBox.getHeight()/2);
				break;	
			}
			case EAST: {
				topleftCoordOfTranslatedDiagBBox.x = containingRectCentre.x + (containingRect.width/2) - diagBBox.getWidth();	
				topleftCoordOfTranslatedDiagBBox.y = containingRectCentre.y + (diagBBox.getHeight()/2);
				break;	
			}
			default:
				break;
		}
		
		translateBy.x += (topleftCoordOfTranslatedDiagBBox.x-diagBBox.x);
		translateBy.y += (topleftCoordOfTranslatedDiagBBox.y-diagBBox.y);
		
		ArrayList<Ellipse> containedEllipses = containedDiagram.getEllipses();
		for (Ellipse containedEllipse : containedEllipses){
			containedEllipse.setXc(containedEllipse.getXc()+translateBy.x);
			containedEllipse.setYc(containedEllipse.getYc()+translateBy.y);
		}

		return containedDiagram;
	}
	
	
	private static double[] computeScaleToFitValues(EllipseDiagram diag, Rectangle2D bbox, Point2D.Double innerPadding, Point2D.Double centreOfBox){
		//this is defined to avoid repetitive computation of these values since different structures would need to be scaled
		
		Point2D.Double[] minmax = computeScaleToFitValues_findMinMaxXY(diag, centreOfBox);
		Point2D.Double min = minmax[0];
		Point2D.Double max = minmax[1];
	    
	    // Stretch the bounding box of the ellipses so that its dimensions match the specified bounding box.
	    double scalex = (bbox.getWidth() - innerPadding.x)/(max.x-min.x);
	    double scaley = (bbox.getHeight() - innerPadding.y)/(max.y-min.y);
	    
	    // Make the scale uniform so that the specified bounding box is not exceeded.
	    double scale = scalex < scaley ? scalex : scaley;

		Point2D.Double delta = computeScaleToFitValues_findDeltaXY(minmax[0], minmax[1], scale);
	    
		return new double[]{scale, delta.x, delta.y};
	}
	
	private static double[] computeScaleToFitValues(EllipseDiagram diag, double scaleForEachPixel, Point2D.Double centreOfBox){
		//this is defined to avoid repetitive computation of these values since different structures would need to be scaled
		
		Point2D.Double[] minmax = computeScaleToFitValues_findMinMaxXY(diag, centreOfBox);
		
		// Scale for ellipse axes
		Ellipse ell0= diag.getEllipses().get(0);
		double scale = Math.sqrt((scaleForEachPixel * ell0.getArea())/(Math.PI * ell0.getA() * ell0.getB()));
		
		Point2D.Double delta = computeScaleToFitValues_findDeltaXY(minmax[0], minmax[1], scale);
	    
		return new double[]{scale, delta.x, delta.y};
	}
	private static Point2D.Double[] computeScaleToFitValues_findMinMaxXY (EllipseDiagram diag, Point2D.Double centreOfBox){
		
	    // Compute the bounding box of the ellipses.
		Rectangle2D.Double diagBBox = getDiagramBoundingBox (diag, centreOfBox, true);
		
	    double minx = diagBBox.x;
	    double miny = diagBBox.y - diagBBox.getHeight();
	    double maxx = diagBBox.x + diagBBox.getWidth();
	    double maxy = diagBBox.y;
	       
		return new Point2D.Double[]{new Point2D.Double(minx, miny), new Point2D.Double(maxx, maxy)};
	}
	
	private static Point2D.Double computeScaleToFitValues_findDeltaXY (Point2D.Double min, Point2D.Double max, double scale){

		//scale = 1;
	    // Translate the diagram such that the centre of its bounding box is equal to the origin (0,0) (the centre of the system) 
	    double deltax = - ((min.x + max.x) * scale) /2;
	    double deltay = - ((min.y + max.y) * scale) /2;
	    
		return new Point2D.Double(deltax, deltay);
	}
	
	public static void scaleToFit_scaleEllipse (Ellipse ellipse, double scale){ 
		ellipse.setA(ellipse.getA()*scale);
		ellipse.setB(ellipse.getB()*scale);
	}
	
	public static void scaleToFit_moveEllipse (Ellipse ellipse, double scale, Point2D.Double delta){ 
		ellipse.setXc( (ellipse.getXc() * scale) + delta.x);
		ellipse.setYc( (ellipse.getYc() * scale) + delta.y);
	}
	
	public static void scaleToFit_scaleCentreEllipse (Ellipse ellipse, double scale, Point2D.Double delta){ 
		scaleToFit_scaleEllipse(ellipse, scale);
		scaleToFit_moveEllipse(ellipse, scale, delta);
	}
	
	public static void scaleToFit_scaleCentreEllipses (EllipseDiagram diag, double[] scalingValues){ 
		double scale = scalingValues[0];
		Point2D.Double delta = new Point2D.Double (scalingValues[1], scalingValues[2]);
	    
	    // Scale each ellipse and translate its centre
	    ArrayList<Ellipse> ellipses = diag.getEllipses();
	    for (Ellipse e : ellipses) {
	    	scaleToFit_scaleCentreEllipse(e, scale, delta);
	    }
	} 
	public static void scaleToFit (Rectangle2D.Double rectangle, double[] scalingValues){ 
	    double scale = scalingValues[0];
	    Point2D.Double delta = new Point2D.Double(scalingValues[1], scalingValues[2]);
	    
	    scaleToFit_scaleRect(rectangle, scale);
	    scaleToFit_moveRect(rectangle, scale, delta);
	} 
	public static void scaleToFit_scaleRect (Rectangle2D.Double rectangle, double scale){ 
		rectangle.width *= scale;
		rectangle.height *= scale;
	} 
	public static void scaleToFit_moveRect (Rectangle2D.Double rectangle, double scale, Point2D.Double delta){     	
		rectangle.x = (rectangle.x * scale) + delta.x;
		rectangle.y = (rectangle.y * scale) + delta.y;
	} 
	
	public static void scaleToFit(EllipseDiagram diag, double[] scaleForEachPixelForEachEllipse, Point2D.Double centreOfBox){ 
		
		Point2D.Double[] minmax;
		ArrayList<Ellipse> ellipses = diag.getEllipses();
		double scale;
		Point2D.Double delta;
		int i = 0;
		for (Ellipse e : ellipses){
			if (i>0){return;}
			minmax = computeScaleToFitValues_findMinMaxXY(diag, centreOfBox);
			scale = Math.sqrt((scaleForEachPixelForEachEllipse[i] * e.getArea())/(Math.PI * e.getA() * e.getB()));
			delta = computeScaleToFitValues_findDeltaXY(minmax[0], minmax[1], scale);
			scaleToFit_scaleEllipse(e, scale);
			i++;
		}
	}
	
	public static void scaleToFit(EllipseDiagram diag, double scaleForEachPixel, Point2D.Double centreOfBox){ 
		double[] scalingValues = computeScaleToFitValues(diag, scaleForEachPixel, centreOfBox);
	    scaleToFit_scaleCentreEllipses(diag,scalingValues);
	}
	
	public static void scaleToFit(EllipseDiagram diag, Rectangle2D bbox, Point2D.Double innerPadding, Point2D.Double centreOfBox){ 
		double[] scalingValues = computeScaleToFitValues(diag, bbox, innerPadding, centreOfBox);
	    scaleToFit_scaleCentreEllipses(diag,scalingValues);
	}
	
    public static void scaleToFit(EllipseDiagram diagToScale, EllipseDiagram diagForScale, Rectangle2D bbox, Point2D.Double innerPadding, Point2D.Double centreOfBox){
		
		double[] scalingValues = computeScaleToFitValues(diagForScale, bbox, innerPadding, centreOfBox);
		scaleToFit_scaleCentreEllipses(diagToScale,scalingValues);
	}
	
	public static void scaleToFit(Rectangle2D.Double rectangle, EllipseDiagram diag, Rectangle2D bbox, Point2D.Double innerPadding, Point2D.Double centreOfBox){ 
	    double[] scalingValues = computeScaleToFitValues(diag, bbox, innerPadding, centreOfBox);
	    scaleToFit(rectangle, scalingValues);
	}
	
	public static void scaleToFit(Rectangle2D.Double[][] rectsInRegions, EllipseDiagram diag, double[] scaleForEachPixelForEachEllipse, Point2D.Double centreOfBox){
		
		Point2D.Double[] minmax = computeScaleToFitValues_findMinMaxXY(diag, centreOfBox);
		
		Point2D.Double delta = computeScaleToFitValues_findDeltaXY(minmax[0], minmax[1], scaleForEachPixelForEachEllipse[0]);
		scaleToFit_scaleRect(rectsInRegions[0][0], scaleForEachPixelForEachEllipse[0]);
		
		delta = computeScaleToFitValues_findDeltaXY(minmax[0], minmax[1], scaleForEachPixelForEachEllipse[1]);
		scaleToFit_scaleRect(rectsInRegions[3][0], scaleForEachPixelForEachEllipse[1]);
		
		delta = computeScaleToFitValues_findDeltaXY(minmax[0], minmax[1], scaleForEachPixelForEachEllipse[2]);
		scaleToFit_scaleRect(rectsInRegions[1][0], scaleForEachPixelForEachEllipse[2]);
		scaleToFit_scaleRect(rectsInRegions[2][0], scaleForEachPixelForEachEllipse[2]);
	}
	
	public static void scaleToFit(Rectangle2D.Double[][] rectsInRegions, EllipseDiagram diag, double scale, Point2D.Double centreOfBox){ 
		
		double[] scalingValues = computeScaleToFitValues(diag, scale, centreOfBox);
		
	    // Scale each rectange and translate its centre
	    for (Rectangle2D.Double[] rectsInRegion : rectsInRegions) {
	    	for (Rectangle2D.Double rect : rectsInRegion) {
		    	scaleToFit(rect, scalingValues);
	    	}
	    }
	}
	
	public static void scaleToFit(Rectangle2D.Double[][] rectsInRegions, EllipseDiagram diag, Rectangle2D bbox, Point2D.Double innerPadding, Point2D.Double centreOfBox){
		
		double[] scalingValues = computeScaleToFitValues(diag, bbox, innerPadding, centreOfBox);
		
	    // Scale each rectange and translate its centre
	    for (Rectangle2D.Double[] rectsInRegion : rectsInRegions) {
	    	for (Rectangle2D.Double rect : rectsInRegion) {
		    	scaleToFit(rect, scalingValues);
	    	}
	    }
	}
	
	public static double scaleAreaForScaledDiag(double area, EllipseDiagram diag, Rectangle2D bbox, Point2D.Double innerPadding, Point2D.Double centreOfBox){
		double[] scalingValues = computeScaleToFitValues(diag, bbox, innerPadding, centreOfBox);
	    double scale = scalingValues[0];
	    
	    return (scale*area);
	}
	
	public static Area getSetAsArea (Ellipse ellipse, Point2D.Double centreOfBox) {
		return (ellipse.getShapeAreaForDisplay(centreOfBox));
	}
	public static Area[] getSetsAsAreas (EllipseDiagram diagram, Point2D.Double centreOfBox) {
		// empty zone is included (index=0) but is left empty/null
		ArrayList<Ellipse> diagEllipses = diagram.getEllipses();  
		Area[] setsAsAreas = new Area[diagEllipses.size()]; 
		
		int i =0;
		for (Ellipse e : diagEllipses){
			setsAsAreas[i] = e.getShapeAreaForDisplay(centreOfBox);
			i++;
		}
		return setsAsAreas;
	}
	
	
	public static Area[] getNonEmptyRegionsAsAreasFor3SetDiag (EllipseDiagram diagram, Point2D.Double centreOfBox) { 
		ArrayList<Area> nonEmptyRegionsAsAreas = new ArrayList<Area>();
		Area[] allRegionsAsAreas = getAllRegionsAsAreasFor3SetDiag(diagram, centreOfBox);
		
		for(Area area : allRegionsAsAreas){
			if ((area!=null)&&(!area.isEmpty())){
				nonEmptyRegionsAsAreas.add(area);
			}
		}
		return nonEmptyRegionsAsAreas.toArray(new Area[]{});
	}
	
	public static Area[] getAllRegionsAsAreasFor3SetDiag (EllipseDiagram diagram, Point2D.Double centreOfBox) {
		if (diagram.getEllipses().size() != 3){
			System.out.println(ReflectionUtils.getClassName(1)+"."+ ReflectionUtils.getMethodName(1)+": the diagram is not made up of 3-ellipses");
			return null;
		}
		
		ArrayList<Ellipse> diagEllipses = diagram.getEllipses();  
		
		Area areaA = null;
		Area areaB = null;
		Area areaC = null;
		
		for (Ellipse e : diagEllipses){
			if (e.getLabel().toLowerCase().equals("a")){
				areaA = e.getShapeAreaForDisplay(centreOfBox);
			} else if (e.getLabel().toLowerCase().equals("b")){
				areaB = e.getShapeAreaForDisplay(centreOfBox);
			} else if (e.getLabel().toLowerCase().equals("c")){
				areaC = e.getShapeAreaForDisplay(centreOfBox);
			}  
		}
	
		Area regions[] = new Area[8];

		// zone a
		regions[1] = new Area(areaA);
		regions[1].subtract(areaB);
		regions[1].subtract(areaC);

		// zone b
		regions[2] = new Area(areaB);
		regions[2].subtract(areaA);
		regions[2].subtract(areaC);

		// zone c
		regions[3] = new Area(areaC);
		regions[3].subtract(areaA);
		regions[3].subtract(areaB);
		
		// zone ab
		regions[4] = new Area(areaA);
		regions[4].intersect(areaB);
		regions[4].subtract(areaC);

		// zone ac
		regions[5] = new Area(areaA);
		regions[5].intersect(areaC);
		regions[5].subtract(areaB);

		// zone bc
		regions[6] = new Area(areaB);
		regions[6].intersect(areaC);
		regions[6].subtract(areaA);
			
		// zone abc
		regions[7] = new Area(areaA);
		regions[7].intersect(areaB);
		regions[7].intersect(areaC);
		
		return regions;
	}
	

	
	
}
