/* 
 * 
 * eulerGlyphs v0.9.0
 * 
 * 2013-12-18
 *
 * 
 * 
 * eulerGlyphs -- Drawing Area-Proportional Euler Diagrams With Glyphs	    
 * 		http://www.eulerdiagrams.org/eulerGlyphs
 * 
 * 
 * 		Copyright (C) 2012-2013, Luana Micallef. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerGlyphs.
 * 			
 * 		eulerGlyphs is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerGlyphs is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerGlyphs (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */

 


package pictograph;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.Area;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import uicomponent.vispanel.VisLabelling;
import uicomponent.vispanel.VisUtils.Alignment;
import uicomponent.vispanel.VisUtils.Orientation;
import uicomponent.vispanel.VisUtils.Placement;
import utils.DimensionDouble;
import utils.MathUtils;


/**
 * To instantiate, define and handle a display of icons
 * 
 */


public class IconDisplay {
	
	public static enum Direction_Icons_IconRows {
		LEFT_TO_RIGHT__TOP_TO_BOTTOM { public String toString() {return "Left to Right, Top to Bottom";}
		   							   public String toStringShort() {return "l-r, t-b";}
									   public ArrayList<IconSet> getIconSetsToDrawIcons(ArrayList<IconSet> iconSets, Dimension displayDimensionAsIconCount, int totalFreq){
											ArrayList<IconSet> iconSetsToDrawIcons = new ArrayList<IconSet>();								
											int iconSetFreq = 0;
											int i = 0;
											for (IconSet iconSet : iconSets){
												iconSetFreq = (int)iconSet.getFrequency();
												for (i = 0; i < iconSetFreq; i++){
													iconSetsToDrawIcons.add(iconSet);
												}
											}
											return iconSetsToDrawIcons;
									   } 
		},
		LEFT_TO_RIGHT__BOTTOM_TO_TOP { public String toString() {return "left to right, bottom to top";}
		   							   public String toStringShort() {return "l-r, b-t";}
									   public ArrayList<IconSet> getIconSetsToDrawIcons(ArrayList<IconSet> iconSets, Dimension displayDimensionAsIconCount, int totalFreq){
											ArrayList<IconSet> iconSetsToDrawIcons = RIGHT_TO_LEFT__BOTTOM_TO_TOP.getIconSetsToDrawIcons(iconSets, displayDimensionAsIconCount, totalFreq);
											return getIconSetsToDrawIcons_RightToLeft(iconSetsToDrawIcons, displayDimensionAsIconCount, totalFreq);
									  }
		},	
		RIGHT_TO_LEFT__TOP_TO_BOTTOM { public String toString() {return "right to left, top_to_bottom";}
									   public String toStringShort() {return "r-l, t-b";}
									   public ArrayList<IconSet> getIconSetsToDrawIcons(ArrayList<IconSet> iconSets, Dimension displayDimensionAsIconCount, int totalFreq){
											ArrayList<IconSet> iconSetsToDrawIcons = LEFT_TO_RIGHT__TOP_TO_BOTTOM.getIconSetsToDrawIcons(iconSets, displayDimensionAsIconCount, totalFreq);
											return getIconSetsToDrawIcons_RightToLeft(iconSetsToDrawIcons, displayDimensionAsIconCount, totalFreq);
								       } 
		}, 
		RIGHT_TO_LEFT__BOTTOM_TO_TOP { public String toString() {return "right to left, bottom to top";}
									   public String toStringShort() {return "r-l, b-t";}
									   public ArrayList<IconSet> getIconSetsToDrawIcons(ArrayList<IconSet> iconSets, Dimension displayDimensionAsIconCount, int totalFreq){
											ArrayList<IconSet> iconSetsToDrawIcons = new ArrayList<IconSet>();
											int iconSetFreq = 0;
											int i = 0;
											IconSet fillerIconSet = new IconSet("","", 0, iconSets.get(0).getIconShape());
											int maxNoOfIconsWithDimension = (int)(displayDimensionAsIconCount.getWidth()*displayDimensionAsIconCount.getHeight());
											int noOfFillerIcons = maxNoOfIconsWithDimension - totalFreq;
											int noOfIconSets = iconSets.size();
											// insert any fillers
											for (i = 0; i < noOfFillerIcons; i++){
												iconSetsToDrawIcons.add(fillerIconSet);
											}
											// reverse iconsets
											IconSet iconSet;
											for (i = (noOfIconSets-1); i >= 0; i--){
												iconSet = iconSets.get(i);
												iconSetFreq = (int)iconSet.getFrequency();
												for (int j = 0; j < iconSetFreq; j++){
													iconSetsToDrawIcons.add(iconSet);
												}	
											}
								   	   		return iconSetsToDrawIcons;
									   } 
		},
		TOP_TO_BOTTOM__LEFT_TO_RIGHT{ public String toString() {return "top_to_bottom, left to right";}
									  public String toStringShort() {return "t-b, l-r";}
                                      public ArrayList<IconSet> getIconSetsToDrawIcons(ArrayList<IconSet> iconSets, Dimension displayDimensionAsIconCount, int totalFreq){
											ArrayList<IconSet> iconSetsToDrawIcons_LR_TB = LEFT_TO_RIGHT__TOP_TO_BOTTOM.getIconSetsToDrawIcons(iconSets, displayDimensionAsIconCount, totalFreq);
											
											int noOfRows = (int)displayDimensionAsIconCount.getHeight();
											int noOfCols = (int)displayDimensionAsIconCount.getWidth();
											IconSet[][] matrixIconSetsToDrawIcons = new IconSet[noOfRows][noOfCols];
											for (int c=0; c<noOfCols; c++){
												for (int r=0; r<noOfRows; r++){
													matrixIconSetsToDrawIcons[r][c]=iconSetsToDrawIcons_LR_TB.get((c*noOfRows)+r);
												}
											}
											return matrixToIconSetArrayList(matrixIconSetsToDrawIcons);
								      }
		}, 
		TOP_TO_BOTTOM__RIGHT_TO_LEFT{ public String toString() {return "top_to_bottom, right to left";}
									  public String toStringShort() {return "t-b, r-l";}
							          public ArrayList<IconSet> getIconSetsToDrawIcons(ArrayList<IconSet> iconSets, Dimension displayDimensionAsIconCount, int totalFreq){
											ArrayList<IconSet> iconSetsToDrawIcons = TOP_TO_BOTTOM__LEFT_TO_RIGHT.getIconSetsToDrawIcons(iconSets, displayDimensionAsIconCount, totalFreq);
											return getIconSetsToDrawIcons_RightToLeft(iconSetsToDrawIcons, displayDimensionAsIconCount, totalFreq);
								      }
		},
		BOTTOM_TO_TOP__LEFT_TO_RIGHT { public String toString() {return "bottom to top, left to right";}
									   public String toStringShort() {return "b-t, l-r";}
							           public ArrayList<IconSet> getIconSetsToDrawIcons(ArrayList<IconSet> iconSets, Dimension displayDimensionAsIconCount, int totalFreq){
											ArrayList<IconSet> iconSetsToDrawIcons = BOTTOM_TO_TOP__RIGHT_TO_LEFT.getIconSetsToDrawIcons(iconSets, displayDimensionAsIconCount, totalFreq);
											return getIconSetsToDrawIcons_RightToLeft(iconSetsToDrawIcons, displayDimensionAsIconCount, totalFreq);
								      }
		},
		BOTTOM_TO_TOP__RIGHT_TO_LEFT {public String toString() {return "bottom to top, right to left";}
									  public String toStringShort() {return "b-t, r-l";}
							          public ArrayList<IconSet> getIconSetsToDrawIcons(ArrayList<IconSet> iconSets, Dimension displayDimensionAsIconCount, int totalFreq){
											ArrayList<IconSet> iconSetsToDrawIcons_LR_BT = RIGHT_TO_LEFT__BOTTOM_TO_TOP.getIconSetsToDrawIcons(iconSets, displayDimensionAsIconCount, totalFreq);
											
											int noOfRows = (int)displayDimensionAsIconCount.getHeight();
											int noOfCols = (int)displayDimensionAsIconCount.getWidth();
											IconSet[][] matrixIconSetsToDrawIcons = new IconSet[noOfRows][noOfCols];
											for (int c=0; c<noOfCols; c++){
												for (int r=0; r<noOfRows; r++){
													matrixIconSetsToDrawIcons[r][c]=iconSetsToDrawIcons_LR_BT.get((c*noOfRows)+r);
												}
											}
											return matrixToIconSetArrayList(matrixIconSetsToDrawIcons);
								     }
		};


		
		public abstract String toString();
		public abstract String toStringShort();
		public abstract ArrayList<IconSet> getIconSetsToDrawIcons(ArrayList<IconSet> iconSets, Dimension displayDimensionAsIconCount, int totalFreq);
		
		protected ArrayList<IconSet> getIconSetsToDrawIcons_RightToLeft (ArrayList<IconSet> iconSets, Dimension displayDimensionAsIconCount, int totalFreq){
			ArrayList<IconSet> iconSetsToDrawIcons = new ArrayList<IconSet>(iconSets.size());

			int noOfRows = (int)displayDimensionAsIconCount.getHeight();
			int noOfCols = (int)displayDimensionAsIconCount.getWidth();
			int r = 0;
			int c = 0;
			// reverse the elements in all the rows
			for(r=0; r < noOfRows; r++){
				for(c=(noOfCols-1); c>=0; c--){
					iconSetsToDrawIcons.add(iconSets.get((r*noOfCols)+c));
				}
			}
			return iconSetsToDrawIcons;
       } 
	  
	};
	
	public static enum IconsPlacementStyle {
		CONTINUOUS_LINEARLY {public String toString() {return "linearly";} 
							 public ArrayList<IconSet> getIconSetsToDrawIcons(ArrayList<IconSet> iconSets, Dimension displayDimensionAsIconCount, Direction_Icons_IconRows direction) {
								 return iconSets;
							 }
		}, 
		CONTINUOUS_ZIGZAG { public String toString() {return "zigzag";} // (left to right, right to left, ...)";} 
		                    public ArrayList<IconSet> getIconSetsToDrawIcons(ArrayList<IconSet> iconSets, Dimension displayDimensionAsIconCount, Direction_Icons_IconRows direction) {
								ArrayList<IconSet> iconSetsToDrawIcons = new ArrayList<IconSet>();
								int noOfRows = (int)displayDimensionAsIconCount.getHeight();
								int noOfCols = (int)displayDimensionAsIconCount.getWidth();
								int r = 0;
								int c = 0;
								switch (direction){
									case LEFT_TO_RIGHT__TOP_TO_BOTTOM:
									case RIGHT_TO_LEFT__TOP_TO_BOTTOM:{
										for(r=0; r<noOfRows; r++){
											if (MathUtils.isEven(r)){
												for(c=0; c<noOfCols; c++){
													iconSetsToDrawIcons.add(iconSets.get((r*noOfCols)+c));
												}
											} else {
												for(c=noOfCols-1; c>=0; c--){
													iconSetsToDrawIcons.add(iconSets.get((r*noOfCols)+c));
												}
											}
										}
										break;
									}
									case LEFT_TO_RIGHT__BOTTOM_TO_TOP:
									case RIGHT_TO_LEFT__BOTTOM_TO_TOP:{
										for(r=0; r<noOfRows; r++){
											if (MathUtils.isEven((noOfRows-1)-r)){
												for(c=0; c<noOfCols; c++){
													iconSetsToDrawIcons.add(iconSets.get((r*noOfCols)+c));
												}
											} else {
												for(c=noOfCols-1; c>=0; c--){
													iconSetsToDrawIcons.add(iconSets.get((r*noOfCols)+c));
												}
											}
										}
										break;
									}
									case TOP_TO_BOTTOM__LEFT_TO_RIGHT:
									case BOTTOM_TO_TOP__LEFT_TO_RIGHT:{
										ArrayList<IconSet> iconSetsTransposed = matrixToIconSetArrayList(transposeIconSetMatrix(iconSetArrayToMatrixGivenRowCount(iconSets, noOfRows)));
										iconSetsTransposed  = IconsPlacementStyle.CONTINUOUS_ZIGZAG.getIconSetsToDrawIcons(iconSetsTransposed, new Dimension(noOfRows,noOfCols), Direction_Icons_IconRows.LEFT_TO_RIGHT__TOP_TO_BOTTOM);
										//could use RIGHT_TO_LEFT__TOP_TO_BOTTOM instead of LEFT_TO_RIGHT__TOP_TO_BOTTOM 
										iconSetsToDrawIcons = matrixToIconSetArrayList(transposeIconSetMatrix(iconSetArrayToMatrixGivenRowCount(iconSetsTransposed, noOfCols)));
										break;
									}
									
									case BOTTOM_TO_TOP__RIGHT_TO_LEFT:
									case TOP_TO_BOTTOM__RIGHT_TO_LEFT:{
										ArrayList<IconSet> iconSetsTransposed = matrixToIconSetArrayList(transposeIconSetMatrix(iconSetArrayToMatrixGivenRowCount(iconSets, noOfRows)));
										iconSetsTransposed  = IconsPlacementStyle.CONTINUOUS_ZIGZAG.getIconSetsToDrawIcons(iconSetsTransposed, new Dimension(noOfRows,noOfCols), Direction_Icons_IconRows.LEFT_TO_RIGHT__BOTTOM_TO_TOP);
										//could use RIGHT_TO_LEFT__BOTTOM_TO_TOP instead of LEFT_TO_RIGHT__BOTTOM_TO_TOP 
										iconSetsToDrawIcons = matrixToIconSetArrayList(transposeIconSetMatrix(iconSetArrayToMatrixGivenRowCount(iconSetsTransposed, noOfCols)));
										break;
									}
								}
								return iconSetsToDrawIcons;
					        }
		};
		
		public abstract String toString();
		public abstract ArrayList<IconSet> getIconSetsToDrawIcons(ArrayList<IconSet> iconSets, Dimension displayDimensionAsIconCount, Direction_Icons_IconRows direction);
		
	};
	

	
	public static final DimensionDouble ICON_DIMENSION_DEFAULT = new DimensionDouble(11,11);
	public static final double ICON_HORIZONTAL_SPACING_DEFAULT = 4;
	public static final double ICON_VERTICAL_SPACING_DEFAULT = 4; 
	public static final double ICONSIDE_T0_SPACING = 2.5; 
	
	public static final Orientation BEST_RECT_FIT_ORIENTATION_DEFAULT = Orientation.HORIZONTAL;
	public static final Direction_Icons_IconRows DIRECTION_ICONS_ICONROWS_DEFAULT = Direction_Icons_IconRows.LEFT_TO_RIGHT__TOP_TO_BOTTOM;
	public static final IconsPlacementStyle ICONS_PLACEMENT_STYLE_DEFAULT = IconsPlacementStyle.CONTINUOUS_ZIGZAG;
	
	
	// Data Fields
	protected ArrayList<IconSet> iconSets;
	protected Dimension dimensionAsIconCount;
	
	protected DimensionDouble iconDimension = ICON_DIMENSION_DEFAULT;
	protected double iconHorizontalSpacing = ICON_HORIZONTAL_SPACING_DEFAULT;
	protected double iconVerticalSpacing = ICON_VERTICAL_SPACING_DEFAULT;
	
	protected Orientation bestRectFitOrientation = BEST_RECT_FIT_ORIENTATION_DEFAULT;
	protected Direction_Icons_IconRows direction_Icons_IconRows = DIRECTION_ICONS_ICONROWS_DEFAULT;
	protected IconsPlacementStyle iconsPlacementStyle = ICONS_PLACEMENT_STYLE_DEFAULT;
	

	
	// Constructors
	public IconDisplay (ArrayList<IconSet> iconSets){
		setIconSets(iconSets);
		// if the other properties are not provided, the defaults are used 
		setBestDimensionAsIconCount();
	}
	public IconDisplay (ArrayList<IconSet> iconSets, Dimension dimensionAsIconCount, DimensionDouble iconDimension, double iconHorizontalSpacing, double iconVerticalSpacing, 
					    Orientation bestRectFitOrientation, Direction_Icons_IconRows direction_Icons_IconRows, IconsPlacementStyle iconsPlacementStyle){
		setIconSets(iconSets);
		setDimensionAsIconCount(dimensionAsIconCount);
		setIconDimension(iconDimension);
		setIconHorizontalSpacing(iconHorizontalSpacing);
		setIconVerticalSpacing(iconVerticalSpacing);
		setBestRectFitOrientation(bestRectFitOrientation);
		setDirection_Icons_IconRows(direction_Icons_IconRows);
		setIconsPlacementStyle(iconsPlacementStyle);
	}
	
	
	
	// Getters
	public ArrayList<IconSet> getIconSets(){
		return this.iconSets;
	}
	public Dimension getDimensionAsIconCount(){
		return this.dimensionAsIconCount;
	}
	public DimensionDouble getIconDimension(){
		return this.iconDimension;
	}
	public double getIconHorizontalSpacing(){
		return this.iconHorizontalSpacing;
	}
	public double getIconVerticalSpacing(){
		return this.iconVerticalSpacing;
	}
	public Orientation getBestRectFitOrientation(){
		return this.bestRectFitOrientation;
	}
	public Direction_Icons_IconRows getDirection_Icons_IconRows(){
		return this.direction_Icons_IconRows;
	}
	public IconsPlacementStyle getIconsPlacementStyle(){
		return this.iconsPlacementStyle;
	}
	
	
	// Setters
	public void setIconSets(ArrayList<IconSet> iconSets){
		this.iconSets = iconSets;
	}
	public void setDimensionAsIconCount(Dimension dimensionAsIconCount){
		if (isDimensionValidForTotalFreq (dimensionAsIconCount)){
			this.dimensionAsIconCount = dimensionAsIconCount;
		}
	}
	public void setDimensionAsIconCountGivenWidth(int noOfIconsPerRow){
		this.dimensionAsIconCount = computeDimensionAsIconCountGivenWidth(noOfIconsPerRow);
	}
	public void setDimensionAsIconCountGivenHeight(int noOfRows){
		this.dimensionAsIconCount = computeDimensionAsIconCountGivenHeight(noOfRows);
	}
	public void setBestDimensionAsIconCount(){
		this.dimensionAsIconCount = computeBestDimensionAsIconCount();
	}
	public void setIconDimension(DimensionDouble iconDimension){
		this.iconDimension = iconDimension;
	}
	public void setIconHorizontalSpacing(double iconHorizontalSpacing){
		this.iconHorizontalSpacing = iconHorizontalSpacing;
	}
	public void setIconVerticalSpacing(double iconVerticalSpacing){
		this.iconVerticalSpacing = iconVerticalSpacing;
	}
	public void setBestRectFitOrientation(Orientation bestRectFitOrientation){
		this.bestRectFitOrientation = bestRectFitOrientation;
		if (bestRectFitOrientation != null){
			setBestDimensionAsIconCount();
		}
	}
	public void setDirection_Icons_IconRows (Direction_Icons_IconRows direction_Icons_IconRows){
		this.direction_Icons_IconRows = direction_Icons_IconRows;
	}
	public void setIconsPlacementStyle (IconsPlacementStyle iconsPlacementStyle){
		this.iconsPlacementStyle = iconsPlacementStyle;
	}
	
	
	
	// Private Methods

	private static IconSet[][] iconSetArrayToMatrixGivenRowCount(ArrayList<IconSet> iconSets, int noOfRows){
	    int noOfCols = MathUtils.roundup(iconSets.size() / noOfRows);
	    IconSet[][] matrix  = new IconSet[noOfRows][noOfCols];
		
		for(int r=0; r < noOfRows; r++){
			for(int c=0; c<noOfCols; c++){
				matrix[r][c] = iconSets.get((r*noOfCols)+c);
			}
		}
		return matrix;
    }
    public static ArrayList<IconSet> matrixToIconSetArrayList (IconSet[][] matrix){
		int noOfRows = matrix.length;
		int noOfCols = matrix[0].length;

		ArrayList<IconSet> list = new ArrayList<IconSet>();
		for(int r=0; r < noOfRows; r++){
			for(int c=0; c<noOfCols; c++){
				list.add(matrix[r][c]);
			}
		}
		return list;
	 }
	 private static IconSet[][] transposeIconSetMatrix (IconSet[][] matrix){
		int noOfRows_inputMatrix = matrix.length;
		int noOfCols_inputMatrix = matrix[0].length;
		
		IconSet[][] transposeMatrix  = new IconSet[noOfCols_inputMatrix][noOfRows_inputMatrix];
		for (int c =0; c<noOfCols_inputMatrix; c++){
			for (int r=0; r<noOfRows_inputMatrix; r++){
				transposeMatrix[c][r]= matrix[r][c];
			}
		}
		return transposeMatrix;
	}
	
	 
	// Other Methods
	 
	public IconDisplay clone(){
		ArrayList<IconSet> iconSetsClone = cloneIconSets();
		IconDisplay iconDisplayClone = new IconDisplay(iconSetsClone);
		iconDisplayClone.setDimensionAsIconCount((Dimension)getDimensionAsIconCount().clone());
		iconDisplayClone.setIconDimension((DimensionDouble)getIconDimension().clone());
		iconDisplayClone.setIconHorizontalSpacing(getIconHorizontalSpacing());
		iconDisplayClone.setIconVerticalSpacing(getIconVerticalSpacing());
		iconDisplayClone.setBestRectFitOrientation(getBestRectFitOrientation());
		iconDisplayClone.setDirection_Icons_IconRows(getDirection_Icons_IconRows());
		iconDisplayClone.setIconsPlacementStyle(getIconsPlacementStyle());
		return iconDisplayClone;
	}

	public ArrayList<IconSet> cloneIconSets(){
		ArrayList<IconSet> iconSetsClone = new ArrayList<IconSet>(iconSets.size());  
		for (IconSet iconset : iconSets){
			iconSetsClone.add(iconset.clone());
		}
		return iconSetsClone;
	}
	
	public int getTotalFrequency(){
		int totalFrequency = 0;
		for (IconSet iconSet : iconSets){
			totalFrequency += iconSet.getFrequency();
		}
		return totalFrequency;
	}	
	public DimensionDouble getDimensionAsPixels(){
		return getDimensionAsPixels(null);
	}

	public DimensionDouble getDimensionAsPixels(Integer groupIconsByValue){

		double additionPixelsForGroupingForWidth = 0;
		double additionPixelsForGroupingForHeight = 0;
		
		if ((groupIconsByValue != null) && (groupIconsByValue > 1)){
			additionPixelsForGroupingForWidth = computeAdditionPixelsDueToGrouping(groupIconsByValue, (int)dimensionAsIconCount.getWidth()) * iconHorizontalSpacing;
			additionPixelsForGroupingForHeight = computeAdditionPixelsDueToGrouping(groupIconsByValue, (int)dimensionAsIconCount.getHeight()) * iconVerticalSpacing;
		}
		Dimension dimensionAsIconCount = getDimensionAsIconCount();
		double displayWidth = (dimensionAsIconCount.getWidth() * iconDimension.getWidth()) + ((dimensionAsIconCount.getWidth()-1) * iconHorizontalSpacing) + additionPixelsForGroupingForWidth;
		double displayHeight = (dimensionAsIconCount.getHeight() * iconDimension.getHeight()) + ((dimensionAsIconCount.getHeight()-1) * iconVerticalSpacing) + additionPixelsForGroupingForHeight;
		
		return (new DimensionDouble (displayWidth, displayHeight));
	}
	private int computeAdditionPixelsDueToGrouping(int groupingValue, int noOfIconsIn1D){
		double sqrtGroupingValue = Math.sqrt(groupingValue);
		if ((sqrtGroupingValue <= 1) || (MathUtils.isDecimal(sqrtGroupingValue))){return 0;}
		int count = 0;
		int noOfTimesNumIsFactorised = 0;
		for (int i = 1; i < noOfIconsIn1D; i++){
			noOfTimesNumIsFactorised = MathUtils.noOfTimesANumberFactorizesAnother(i, (int)sqrtGroupingValue);
			count += ((noOfTimesNumIsFactorised>0)? noOfTimesNumIsFactorised : 0);
		}
		return count;
	}

	public boolean isDimensionValidForTotalFreq (Dimension dimensionAsIconCount){
		int totalFreq = getTotalFrequency();
		int maxNoOfIconsWithDimension = getMaxNoOfIconsWithDimension();
		return (totalFreq <= maxNoOfIconsWithDimension);
	}
	
	public int getMaxNoOfIconsWithDimension(){
		return (int)(dimensionAsIconCount.getWidth() * dimensionAsIconCount.getHeight());
	}
	
	public int computeOtherSideOfDimensionAsIconCount (int noOfIconsOn1D){
		int totalFreq = getTotalFrequency();
		return MathUtils.roundup(totalFreq/noOfIconsOn1D);
	}
	public Dimension computeDimensionAsIconCountGivenWidth(int noOfIconsPerRow){
		return new Dimension (noOfIconsPerRow, computeOtherSideOfDimensionAsIconCount(noOfIconsPerRow));
	}
	public Dimension computeDimensionAsIconCountGivenHeight (int noOfRows){
		return new Dimension (computeOtherSideOfDimensionAsIconCount(noOfRows), noOfRows);
	}
	
	public Dimension computeBestDimensionAsIconCount(){
		Dimension bestDimensionAsIconCount = new Dimension();
		
		int totalFreq = getTotalFrequency();
		
		// Display as a square
		double sqrtTotalFreq = Math.sqrt(totalFreq);
		
		if (!MathUtils.isDecimal(sqrtTotalFreq)){
			bestDimensionAsIconCount.setSize(sqrtTotalFreq, sqrtTotalFreq);
			return bestDimensionAsIconCount;
		}
		
		// Display as a rectangle with no. of icons per row  5*(10^(|sf of totFreq|-2)) - vertical / horizontal, based on selected orientation
		int noOfIcons1D = 0;
		int baseForNoOfIconsPerRow = 5;
		if (totalFreq >= 100){
			int totalFreqNoOfSF = MathUtils.getNoOfSignificantFiguresBeforeDecimalPoint(totalFreq);
			noOfIcons1D = (int) (baseForNoOfIconsPerRow * Math.pow(10, totalFreqNoOfSF-3));
		} else if (totalFreq >= baseForNoOfIconsPerRow){
			noOfIcons1D = baseForNoOfIconsPerRow;
		} else {
			noOfIcons1D = totalFreq;
		}
		int noOfIconsOtherD = MathUtils.roundup(totalFreq / noOfIcons1D);
		
		return bestRectFitOrientation.defineDimension(noOfIcons1D, noOfIconsOtherD);
		
		// could have tried to get the largest 2 factors of the totalfreq to ensure a structure closer to a perfect square 
		// BUT then might get rows and columns which are not so familiar to deal with
	}

	
	public ArrayList<IconSet> getIconSetsToDrawIcons(){
		ArrayList<IconSet> iconSetsToDrawIcons = new ArrayList<IconSet>();
		int totalFreq = getTotalFrequency();	

		// Direction of Icons and Rows of Icons
		iconSetsToDrawIcons = direction_Icons_IconRows.getIconSetsToDrawIcons(iconSets, dimensionAsIconCount, totalFreq);
		
		// Icons Placement Style 
		iconSetsToDrawIcons = iconsPlacementStyle.getIconSetsToDrawIcons(iconSetsToDrawIcons, dimensionAsIconCount, direction_Icons_IconRows);
		
		return iconSetsToDrawIcons;
	}

	
	
	public void scaleToFit(Rectangle2D bbox, Point2D.Double innerPadding, Integer groupingValue){
		
		DimensionDouble displayDimensions = getDimensionAsPixels(groupingValue); 
		
		// Stretch the bounding box of the ellipses so that its dimensions match the specified bounding box. 
	    double scalex = (bbox.getWidth() - innerPadding.x)/(displayDimensions.getWidth());
	    double scaley = (bbox.getHeight() - innerPadding.y)/(displayDimensions.getHeight());
	    
	    // Make the scale uniform so that the specified bounding box is not exceeded.
	    double scale = scalex < scaley ? scalex : scaley;
	    
		// carry out the actual changes 
		setIconDimension(new DimensionDouble(getIconDimension().getWidth()*scale, getIconDimension().getHeight()*scale));
		setIconHorizontalSpacing(getIconHorizontalSpacing() * scale);
		setIconVerticalSpacing(getIconVerticalSpacing() * scale);
	}	
	
	public Point2D.Double getTopLeftCoordsOfCentredDisplay (Rectangle bbox, Integer groupingValue){
		DimensionDouble displayDimensionInPixels = getDimensionAsPixels(groupingValue);
		
		return new Point2D.Double(bbox.getX()+((bbox.getWidth()-displayDimensionInPixels.getWidth())/2), bbox.getY()+((bbox.getHeight()-displayDimensionInPixels.getHeight())/2));
	}
	
	
	public void drawScale(Graphics2D g2D, Point2D.Double topRightCoordOfDisplay, Integer groupingValue, String fontName, int fontStyle, int fontSize, Color textColour){
		double spacingBetweenScaleAndDisplay = 15;
		double intervalMarkEveryNRows = 2; 
		double intervalMarkLength = 5;
		
		// Draw scale line with interval marks 
		
		DimensionDouble dimensionAsPixels = getDimensionAsPixels(groupingValue);
		
		Point2D.Double lineStart = new Point2D.Double (0.0,0.0);
		Point2D.Double lineEnd = new Point2D.Double (0.0,0.0);
		ArrayList<Line2D.Double> intervalMarks = new ArrayList<Line2D.Double>();
		int noOfIconsPerInterval = 0;
		switch (direction_Icons_IconRows){
			case LEFT_TO_RIGHT__TOP_TO_BOTTOM:
			case RIGHT_TO_LEFT__TOP_TO_BOTTOM:
			case LEFT_TO_RIGHT__BOTTOM_TO_TOP:
			case RIGHT_TO_LEFT__BOTTOM_TO_TOP:{
				
				lineStart = new Point2D.Double (topRightCoordOfDisplay.x - spacingBetweenScaleAndDisplay, topRightCoordOfDisplay.y);
				lineEnd = new Point2D.Double (lineStart.x, lineStart.y + dimensionAsPixels.getHeight());
				
				int noOfRows = (int)dimensionAsIconCount.getHeight();
				int noOfCols = (int)dimensionAsIconCount.getWidth();
				noOfIconsPerInterval = (int)(intervalMarkEveryNRows * noOfCols);
				
				double rowHeight = dimensionAsPixels.getHeight()/noOfRows;
				double markLoc_x_start = lineStart.x - intervalMarkLength;
				double markLoc_x_end = lineStart.x;
				double markLoc_y = lineStart.y;
				for (int r=0; r <= noOfRows; r+=intervalMarkEveryNRows){
					intervalMarks.add(new Line2D.Double(markLoc_x_start,markLoc_y,markLoc_x_end,markLoc_y));
					markLoc_y += (intervalMarkEveryNRows*rowHeight); 
				}
				
				break;
			}
			case TOP_TO_BOTTOM__LEFT_TO_RIGHT:
			case TOP_TO_BOTTOM__RIGHT_TO_LEFT:
			case BOTTOM_TO_TOP__LEFT_TO_RIGHT:
			case BOTTOM_TO_TOP__RIGHT_TO_LEFT:{
				lineStart = new Point2D.Double (topRightCoordOfDisplay.x, topRightCoordOfDisplay.y + dimensionAsPixels.getHeight() + spacingBetweenScaleAndDisplay);
				lineEnd   = new Point2D.Double (lineStart.x + dimensionAsPixels.getWidth(), lineStart.y);

				int noOfCols = (int)dimensionAsIconCount.getWidth();
				int noOfRows = (int)dimensionAsIconCount.getHeight();
				noOfIconsPerInterval = (int)(intervalMarkEveryNRows * noOfRows);
				
				double colWidth = dimensionAsPixels.getWidth()/noOfCols;
				double markLoc_y_start = lineStart.y;
				double markLoc_y_end = markLoc_y_start + intervalMarkLength;
				double markLoc_x = lineStart.x;
				for (int c=0; c <= noOfCols; c+=intervalMarkEveryNRows){
					intervalMarks.add(new Line2D.Double(markLoc_x,markLoc_y_start,markLoc_x,markLoc_y_end));
					markLoc_x += (intervalMarkEveryNRows*colWidth); 
				}
				
				break;
			}	
		}
		
		g2D.setColor(textColour);
		g2D.drawLine((int)lineStart.x, (int)lineStart.y, (int)lineEnd.x, (int)lineEnd.y);
		for (Line2D.Double intervalMark : intervalMarks){
			g2D.drawLine((int)intervalMark.x1, (int)intervalMark.y1, (int)intervalMark.x2, (int)intervalMark.y2);
		}
		
		
		// Draw interval mark labels
		
		int[] intervalMarkLabels = new int[intervalMarks.size()];		
		switch (direction_Icons_IconRows){
			case LEFT_TO_RIGHT__TOP_TO_BOTTOM:
			case RIGHT_TO_LEFT__TOP_TO_BOTTOM:
			case TOP_TO_BOTTOM__LEFT_TO_RIGHT:
			case BOTTOM_TO_TOP__LEFT_TO_RIGHT:{
				for (int i = 0; i<intervalMarks.size(); i++){
					intervalMarkLabels[i] = noOfIconsPerInterval*i;
				}
				break;
			}
			case LEFT_TO_RIGHT__BOTTOM_TO_TOP:
			case RIGHT_TO_LEFT__BOTTOM_TO_TOP:
			case TOP_TO_BOTTOM__RIGHT_TO_LEFT:
			case BOTTOM_TO_TOP__RIGHT_TO_LEFT:{
				for (int i = 0; i<intervalMarks.size(); i++){
					intervalMarkLabels[i] = noOfIconsPerInterval*(intervalMarks.size()-1-i);
				}
				break;
			}	
		}
		
		switch (direction_Icons_IconRows){
			case LEFT_TO_RIGHT__TOP_TO_BOTTOM:
			case RIGHT_TO_LEFT__TOP_TO_BOTTOM:
			case LEFT_TO_RIGHT__BOTTOM_TO_TOP:
			case RIGHT_TO_LEFT__BOTTOM_TO_TOP:{	
				Rectangle rect = new Rectangle();
				Area rectArea;
				for (int i = 0; i<intervalMarks.size(); i++){
					rect.x = (int) intervalMarks.get(i).x1;
					rect.y = (int) intervalMarks.get(i).y1-50;
					rect.width = (int) Math.abs(intervalMarks.get(i).x1-intervalMarks.get(i).x2)+100;
					rect.height = (int) Math.abs(intervalMarks.get(i).y1-intervalMarks.get(i).y2)+100;
					rectArea = new Area(rect.getFrame());
					VisLabelling.placeLabelAdjacentToArea(g2D, rectArea, Integer.toString(intervalMarkLabels[i]), Placement.WEST, Alignment.LEFT, fontName, Font.PLAIN, 10, textColour, 0);
				}
				break;
			}
			case TOP_TO_BOTTOM__LEFT_TO_RIGHT:
			case TOP_TO_BOTTOM__RIGHT_TO_LEFT:
			case BOTTOM_TO_TOP__LEFT_TO_RIGHT:
			case BOTTOM_TO_TOP__RIGHT_TO_LEFT:{
				Rectangle rect = new Rectangle();
				Area rectArea;
				for (int i = 0; i<intervalMarks.size(); i++){
					rect.x = (int) intervalMarks.get(i).x1-50;
					rect.y = (int) intervalMarks.get(i).y1-100;
					rect.width = (int) Math.abs(intervalMarks.get(i).x1-intervalMarks.get(i).x2)+100;
					rect.height = (int) Math.abs(intervalMarks.get(i).y1-intervalMarks.get(i).y2)+100;
					rectArea = new Area(rect.getFrame());
					VisLabelling.placeLabelAdjacentToArea(g2D, rectArea, Integer.toString(intervalMarkLabels[i]), Placement.SOUTH, Alignment.LEFT, fontName, Font.PLAIN, 10, textColour, 0);
				}
				break;
			}
		}
		
	}
}



















