/* 
 * 
 * eulerGlyphs v0.9.0
 * 
 * 2013-12-18
 *
 * 
 * 
 * eulerGlyphs -- Drawing Area-Proportional Euler Diagrams With Glyphs	    
 * 		http://www.eulerdiagrams.org/eulerGlyphs
 * 
 * 
 * 		Copyright (C) 2012-2013, Luana Micallef. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerGlyphs.
 * 			
 * 		eulerGlyphs is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerGlyphs is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerGlyphs (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */

 


package pictograph;

import java.awt.AlphaComposite;
import java.awt.Composite;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.TexturePaint;
import java.awt.geom.AffineTransform;
import java.awt.geom.Area;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.IOException;
import javax.imageio.ImageIO;
import utils.ReflectionUtils;
import elliptic.areaproptool.Ellipse;


/**
 * To instantiate, define and handle an IconSet (i.e., a set or group of icons)
 * 
 */


public class IconSet {

	public static enum IconShape {
		RECTANGLE { public String toString(){return "rectangle";}
				    public String getImageFilePath(){return null;}
					public Area getIconArea (Point2D.Double topLeftCoord, Dimension iconDimension) {
						Rectangle rectIcon = new Rectangle();
						rectIcon.setRect(topLeftCoord.x, topLeftCoord.y, iconDimension.getWidth(), iconDimension.getHeight());
						return (new Area(rectIcon.getFrame()));
				    } 
		},
		/*
		CIRCLE { public String toString(){return "circle";}
			     public String getImageFilePath(){return null;}
		         public Area getIconArea (Point2D.Double topLeftCoord, Dimension iconDimension, Graphics2D graphics2D){
					return ELLIPSE.getIconArea(topLeftCoord, iconDimension, graphics2D);
				 }
		},
		*/
		ELLIPSE { public String toString(){return "circle";} //"ellipse";}
		 		  public String getImageFilePath(){return null;}
				  public Area getIconArea (Point2D.Double topLeftCoord, Dimension iconDimension) {
						Ellipse ellIcon = new Ellipse("", iconDimension.getWidth()/2, iconDimension.getHeight()/2, topLeftCoord.x + (iconDimension.getWidth()/2), topLeftCoord.y + (iconDimension.getHeight()/2), 0.0);
						return (new Area(ellIcon.getShapeAreaForDisplay(new Point2D.Double(0,2*(topLeftCoord.y + (iconDimension.getHeight()/2))))));
			      }
		},
		FIGURE { public String toString(){return "figure";}
				 public String getImageFilePath(){return "/pictograph/figure2.png";}
		  		 public Area getIconArea(Point2D.Double topLeftCoord, Dimension iconDimension) {
		  			
		  			// this draws a figure icon 
		  			// -> code written by, obtained from and used with the permission of Pierre Dragicevic at AVIZ, INRIA, France in 2011
		  			 
		  			Rectangle2D.Double bounds = new Rectangle2D.Double(topLeftCoord.getX(), topLeftCoord.getY(), iconDimension.getWidth(), iconDimension.getHeight());
		  			
		  			Area shape = new Area();
		  			
		  			// Head
		  			double hs = 4.0;
		  			Shape head = new Ellipse2D.Double(-hs/2, 0, hs, hs);
		  			shape.add(new Area(head)); 
		  			
		  			// Body (actually half of it)
		  			GeneralPath body = new GeneralPath();
		  			double y0 = 4.5;
		  			double xs = 3.0;
		  			body.moveTo(0, y0);
		  			body.lineTo(-xs, y0);
		  			double sr = 2.5;
		  			body.quadTo(-xs - sr, y0, -xs - sr, y0 + sr);
		  			double al = 7.0;
		  			body.lineTo(-xs - sr, y0 + sr + al);
		  			double aw = 1.9;
		  			body.quadTo(-xs - sr, y0 + sr + al + aw/2, -xs - sr + aw/2, y0 + sr + al + aw/2);
		  			body.quadTo(-xs - sr + aw, y0 + sr + al + aw/2, -xs - sr + aw, y0 + sr + al);
		  			double dd = 1.0;
		  			body.lineTo(-xs - sr + aw, y0 + sr + dd);
		  			double d = sr - aw;
		  			body.quadTo(-xs - d, y0 + sr - d/2 + dd, -xs - d/2, y0 + sr - d/2 + dd);  
		  			body.quadTo(-xs, y0 + sr - d/2 + dd, -xs, y0 + sr + dd);
		  			double ll = 15.0;
		  			double lw = 2.7;
		  			body.lineTo(-xs, y0 + al + ll - lw/2);
		  			body.quadTo(-xs, y0 + al + ll, -xs + lw/2, y0 + al + ll);
		  			body.quadTo(-xs + lw, y0 + al + ll, -xs + lw, y0 + al + ll - lw/2);
		  			double d2 = (xs - lw)*2;
		  			double dd2 = 1.0;
		  			body.lineTo(-xs + lw, y0 + sr + al + d2/2 + dd2);
		  			body.quadTo(-xs + lw, y0 + sr + al + dd2, -xs + lw + d2/2, y0 + sr + al + dd2);
		  			body.closePath();
		  			shape.add(new Area(body));
		  			
		  			// Also add the other body's half
		  			AffineTransform mirror = AffineTransform.getScaleInstance(-1, 1);
		  			Shape body2 = body.createTransformedShape(mirror);
		  			shape.add(new Area(body2));
		  			
		  			// Transform to the requested bounds
		  			Rectangle2D currentBounds = shape.getBounds2D();
		  			AffineTransform transform = new AffineTransform();
		  			transform.translate(bounds.getX(), bounds.getY());
		  			transform.scale(bounds.getWidth() / currentBounds.getWidth(), bounds.getHeight() / currentBounds.getHeight());
		  			transform.translate(-currentBounds.getX(), -currentBounds.getY());
		  			shape = shape.createTransformedArea(transform);
		  			
		  			return shape;
		  		}

		};
		
		public abstract String toString();
		public abstract String getImageFilePath();
		public abstract Area getIconArea (Point2D.Double topLeftCoord, Dimension iconDimension);
		
		public BufferedImage loadImageFromFile(){
			String imageFilePath = getImageFilePath();
			if (imageFilePath == null){return null;}
			try {
				return (ImageIO.read(getClass().getResourceAsStream(imageFilePath)));
			} catch (IOException e) {
				System.out.println(ReflectionUtils.getClassName(1)+"."+ ReflectionUtils.getMethodName(1)+": "+e);
		    }
			return null;
		}
 		public void paintImage (Point2D.Double topLeftCoord, Dimension iconDimension, Graphics2D graphics2D) {
  			paintImage(topLeftCoord, iconDimension, graphics2D, loadImageFromFile());
  		}
  		public void paintImage (Point2D.Double topLeftCoord, Dimension iconDimension, Graphics2D graphics2D, BufferedImage imageIcon) {
  			if (imageIcon == null){return;}
  			 
			Rectangle2D rectIcon = new Rectangle2D.Double(topLeftCoord.x, topLeftCoord.y, iconDimension.getWidth(), iconDimension.getHeight());
			TexturePaint texturePaintIcon = new TexturePaint(imageIcon, rectIcon);
			Area areaIcon = new Area(rectIcon.getFrame());
			
			Composite prevComposite = graphics2D.getComposite();
			Composite c = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1f); //SRC_OVER the smaller the value .4f the more transparent
			graphics2D.setComposite(c);
			graphics2D.setPaint(texturePaintIcon);
			graphics2D.fill(areaIcon);
			graphics2D.setComposite(prevComposite);
			graphics2D.draw(areaIcon);
	   }
	};
	
	
	// Data fields 
	protected String ref;
	protected String label;
	protected int frequency;
	protected IconShape iconShape;
	protected Dimension iconDimension;
	protected Point2D.Double[] topLeftCoordOfIcons;  //using a cartesian coordinate system (whereby x incr to the right and y increases upwards)
	
	
	// Constructors
	public IconSet (String ref, String label, double frequency, IconShape iconShape){
		setRef(ref);
		setLabel(label);
		setFrequency(frequency);
		setIconShape(iconShape);
	}
	public IconSet (String ref, String label, int frequency, IconShape iconShape){
		setRef(ref);
		setLabel(label);
		setFrequency(frequency);
		setIconShape(iconShape);
	}
	
	
	// Getters
	public String getRef(){
		return this.ref;
	}
	public String getLabel(){
		return this.label;
	}
	public int getFrequency(){
		return this.frequency;
	}
	public IconShape getIconShape(){
		return this.iconShape;
	}
	public Dimension getIconDimension(){
		return this.iconDimension;
	}
	public Point2D.Double[] getTopLeftCoordOfIcons(){
		return this.topLeftCoordOfIcons;
	}
	
	
	// Setters
	public void setRef(String ref){
		this.ref = ref;
	}
	public void setLabel(String label){
		this.label = label;
	}
	public void setFrequency(double frequency){
		this.frequency = (int)Math.round(frequency); 
	}
	public void setIconShape (IconShape iconShape){
		this.iconShape = iconShape;
	}
	public void setIconDimension (Dimension iconDimension){
		this.iconDimension = iconDimension;
	}
	public void setIconDimension (double iconWidth, double iconHeight){
		Dimension dimension = new Dimension();
		dimension.setSize(iconWidth, iconHeight);
		setIconDimension(dimension);
	}
	public void setTopLeftCoordOfIcons (Point2D.Double[] topLeftCoordOfIcons){
		if (topLeftCoordOfIcons.length != getFrequency()){return;} 
		this.topLeftCoordOfIcons = topLeftCoordOfIcons;
	}
	
	
	// Methods 
	
	public IconSet clone(){
		return (new IconSet(new String(ref), new String(label), frequency, iconShape));
	}
	
}
