/* 
 * 
 * eulerGlyphs v0.9.0
 * 
 * 2013-12-18
 *
 * 
 * 
 * eulerGlyphs -- Drawing Area-Proportional Euler Diagrams With Glyphs	    
 * 		http://www.eulerdiagrams.org/eulerGlyphs
 * 
 * 
 * 		Copyright (C) 2012-2013, Luana Micallef. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerGlyphs.
 * 			
 * 		eulerGlyphs is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerGlyphs is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerGlyphs (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */

 


package uicomponent.vispanel;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.geom.AffineTransform;
import java.awt.geom.Area;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Random;
import javax.swing.JOptionPane;

import pictograph.IconDisplay;
import pictograph.IconSet;

import uicomponent.utils.panel.JPanelUtils;
import uicomponent.vispanel.VisUtils.*;
import utils.DimensionDouble;
import utils.GeometryUtils;
import utils.MathUtils;
import utils.StringUtils;
import visforfallacy.EulerPop2SetsIconsForNeglectBaseRate.IconsPositioning;
import visstyle.HeterogeneousChannelStyle;
import visstyle.VisStyle;

import elliptic.areaproptool.Ellipse;
import elliptic.areaproptool.EllipseDiagram;
import elliptic.areaproptool.EllipseDiagramOps;
import eulerellipse.*;


/**
 * 
 * To instantiate, define and handle panels for Euler diagrams with ellipses
 *
 */


public class EulerEllipsePanel extends VisPanel {
	
	// Class Constants
	private static final long serialVersionUID = 1L;
	public static final boolean SHOW_ICONS_DEFAULT = false;
	
	public static final String FONT_NAME_DEFAULT = "Arial";
	public static final int FONT_STYLE_DEFAULT = Font.PLAIN;
	public static final int FONT_SIZE_DEFAULT = 14;
	public static final Color COLOUR_BACKGROUND_DEFAULT = Color.white;
	public static final Color COLOUR_TEXT_DEFAULT = Color.black;//darkGray;
	
	// Constants 
	public boolean showBorderOfFilledCurves = false;
	
	// Data fields
	protected EllipseDiagram eulerEllipseDiagram;
	protected HeterogeneousChannelStyle[] ellipsesVisStyle;
	protected String[] ellipsesRefLabel;
	protected String[] regionsInDiagram;
	protected boolean showSetsLabel;
	protected String[] setsLabel;
	protected Placement[] setsLabelPlacement;
	protected Alignment[] setsLabelTextAlignment;
	protected boolean showRegionsLabel;
	protected String[] regionsLabel;
	protected boolean showLegend;
	protected boolean showIcons = SHOW_ICONS_DEFAULT;
	protected DimensionDouble iconDimension;
	protected IconsPositioning iconsPositioning;
	protected ArrayList<IconSet> regionsIconSet;
	protected Rectangle2D.Double[][] rectsInRegions;
	protected Rectangle2D.Double containingRectangle = null;
	protected Boolean needLargerPopSizeToDisplayIcons=false;
	protected int popSize;
	protected EllipseDiagram areaPropEulerDiag;
	public double[] regionsArea;
	protected Integer groupingValue;
	
	protected String fontName_sets = FONT_NAME_DEFAULT;
	protected int fontStyle_sets = Font.BOLD;
	protected int fontSize_sets =  FONT_SIZE_DEFAULT;
	protected String fontName_regions = FONT_NAME_DEFAULT;
	protected int fontStyle_regions = FONT_STYLE_DEFAULT;
	protected int fontSize_regions =  FONT_SIZE_DEFAULT;
	protected Color colourBackground = COLOUR_BACKGROUND_DEFAULT;
	protected Color colourText = COLOUR_TEXT_DEFAULT;
	protected int multiLineSpacing = 6;  // the number of pixels between lines in a multi-line label
	protected boolean setBackground = true;
	

	
	// Constructor 
	public EulerEllipsePanel(){
		super();
	}


	// Getters
	public EllipseDiagram getEulerEllipseDiagram (){
		return this.eulerEllipseDiagram;
	}
	public HeterogeneousChannelStyle[] getEllipsesVisStyle(){
		return this.ellipsesVisStyle;
	}
	public String[] getRegionsInDiagram(){
		return this.regionsInDiagram;
	}
	public boolean getShowSetsLabel(){
		return this.showSetsLabel;
	}
	public String[] getSetsLabel(){
		return this.setsLabel;
	}
	public Placement[] getSetsLabelPlacement(){
		return this.setsLabelPlacement;
	}
	public Alignment[] getSetsLabelTextAlignment(){
		return this.setsLabelTextAlignment;
	}
	public boolean getShowRegionsLabel(){
		return this.showRegionsLabel;
	}
	public String[] getRegionsLabel(){
		return this.regionsLabel;
	}
	public boolean getShowLegend(){
		return this.showLegend;
	}
	public boolean getShowIcons(){
		return this.showIcons;
	}
	public DimensionDouble getIconDimension(){
		return this.iconDimension;
	}
	public IconsPositioning getIconsPositioning(){
		return this.iconsPositioning;
	}
	public ArrayList<IconSet> getRegionsIconSet(){
		return this.regionsIconSet;
	}
	public Rectangle2D.Double[][] getRectsInRegions(){
		return this.rectsInRegions;
	}
	public Rectangle2D.Double getContainingRecantangle(){
		return this.containingRectangle;
	}
	
	public String getFontNameSets(){
		return this.fontName_sets;
	}
	public int getFontStyleSets(){
		return this.fontStyle_sets;
	}
	public int getFontSizeSets(){
		return this.fontSize_sets;
	}
	public String getFontNameRegions(){
		return this.fontName_regions;
	}
	public int getFontStyleRegions(){
		return this.fontStyle_regions;
	}
	public int getFontSizeRegions(){
		return this.fontSize_regions;
	}
	public Color getColourBackground(){
		return this.colourBackground;
	}
	public Color getColourText(){
		return this.colourText;
	}
	public boolean getSetBackground(){
		return this.setBackground;
	}

	public int getGroupingValue(){
		return this.groupingValue;
	}

	
	
	// Setters
	public void setEulerEllipseDiagram (EllipseDiagram eulerEllipseDiagram){
		this.eulerEllipseDiagram = eulerEllipseDiagram;
	}
	public void setEllipsesVisStyleEuler(HeterogeneousChannelStyle[] ellipsesVisStyle){
		this.ellipsesVisStyle = ellipsesVisStyle;
	} 
	
	public void setEllipsesRefLabel(String[] ellipsesRefLabel){
		this.ellipsesRefLabel = ellipsesRefLabel;
	} 
	public void setRegionsInDiagram (String[] regionsInDiagram){
		this.regionsInDiagram = regionsInDiagram;
	}
	public void setShowSetsLabel(boolean showSetsLabel){
		this.showSetsLabel = showSetsLabel;
	}
	public void setSetsLabel(String[] setsLabel){
		this.setsLabel = setsLabel;
	}
	public void setSetsLabelPlacement(Placement[] setsLabelPlacement){
		this.setsLabelPlacement = setsLabelPlacement;
	}
	public void setSetsLabelTextAlignment(Alignment[] setsLabelTextAlignment){
		this.setsLabelTextAlignment = setsLabelTextAlignment;
	}
	public void setShowRegionsLabel(boolean showRegionsLabel){
		this.showRegionsLabel = showRegionsLabel;
	}
	public void setRegionsLabel(String[] regionsLabel){
		this.regionsLabel = regionsLabel;
	}
	public void setShowLegend(boolean showLegend){
		this.showLegend = showLegend;
	}
	public void setShowIcons(boolean showIcons){
		this.showIcons = showIcons;
	}
	public void setIconDimension(DimensionDouble iconDimension){
		this.iconDimension = iconDimension;
	}
	public void setIconsPositioning(IconsPositioning iconsPositioning){
		this.iconsPositioning = iconsPositioning;
	}
	public void setRegionsIconSet(ArrayList<IconSet> regionsIconSet){
		this.regionsIconSet = regionsIconSet;
	}
	public void setRectsInRegions(Rectangle2D.Double[][] rectsInRegions){
		this.rectsInRegions = rectsInRegions;
	}
	public void setContainingRecantangle(Rectangle2D.Double containingRecantangle){
		this.containingRectangle = containingRecantangle;
	}
	
	public void setFontNameSets (String fontName_sets){
		this.fontName_sets = fontName_sets;
	}
	public void setFontStyleSets (int fontStyle_sets){
		this.fontStyle_sets = fontStyle_sets;
	}
	public void setFontSizeSets (int fontSize_sets){
		this.fontSize_sets = fontSize_sets;
	}
	public void setFontNameRegions (String fontName_regions){
		this.fontName_regions = fontName_regions;
	}
	public void setFontStyleRegions (int fontStyle_regions){
		this.fontStyle_regions = fontStyle_regions;
	}
	public void setFontSizeRegions (int fontSize_regions){
		this.fontSize_regions = fontSize_regions;
	}
	public void setColourBackground (Color colourBackground){
		this.colourBackground = colourBackground;
	}
	public void setColourText (Color colourText){
		this.colourText = colourText;
	}
	public void setSetBackground (boolean setBackground){
		this.setBackground = setBackground;
	}

	public void setGroupingValue(int groupingValue){
		this.groupingValue = groupingValue;
	}
	public void setNeedLargerPopSizeToDisplayIcons(Boolean needLargerPopSizeToDisplayIcons){
		this.needLargerPopSizeToDisplayIcons = needLargerPopSizeToDisplayIcons;
	}	
	public void setPopSize(int popSize){
		this.popSize = popSize;
	}
	public void setAreaPropEulerDiag (EllipseDiagram areaPropEulerDiag){
		this.areaPropEulerDiag = areaPropEulerDiag;
	}
	
	
	// Inherited/Overriden Methods 
	
	public void paint(Graphics g) {
		paintComponent(g);
	}
	
	public void paintComponent (Graphics g) {

		// Graphics 2D
		Graphics2D g2D = (Graphics2D) g;
		JPanelUtils.setGraphics2DAntiAliasing(g2D, true);
		
		
		// Paint background
		if (setBackground){
			setBackground(colourBackground);
			super.paintComponent(g2D);
		}
		
		// When a diagram is not available 
		if (eulerEllipseDiagram == null){return;}
		
		
		// if cannot display icons with such population size
		if (showIcons && needLargerPopSizeToDisplayIcons){
			return;
		}
	
		
		// Scale the displayed diagram not the actual one -> this avoids confusion with requiredAreaSpecs and currZoneAreas
		EllipseDiagram eulerEllipseDiagram_scaledToFit = eulerEllipseDiagram.clone();
		EulerEllipseVisUtils.scaleToFit(eulerEllipseDiagram_scaledToFit, areaPropEulerDiag, getBoundingBox(), getInnerPadding(), getCentreOfPanel()); 
		
		// Draw the outer rectangle containing the euler diagram 
		Area containingRectArea = null;
		if (this.containingRectangle != null){
			Rectangle2D.Double rectToDraw = new Rectangle2D.Double(containingRectangle.x, containingRectangle.y, containingRectangle.getWidth(), containingRectangle.getHeight()); 
			EulerEllipseVisUtils.scaleToFit(rectToDraw, areaPropEulerDiag, getBoundingBox(), getInnerPadding(), getCentreOfPanel());
			Point2D.Double rectTopLeftCoord = GeometryUtils.changeCoorSys_originMid_to_originTopLeft(new Point2D.Double(rectToDraw.x, rectToDraw.y), getCentreOfPanel());
			rectToDraw.x = rectTopLeftCoord.x;
			rectToDraw.y = rectTopLeftCoord.y;
			containingRectArea = GeometryUtils.getAreaForRectangle(rectToDraw);
			if (ellipsesVisStyle.length == 3){
				ellipsesVisStyle[0].paintArea(g2D, containingRectArea, true, false, false);
			} else if (ellipsesVisStyle.length > 3){
				ellipsesVisStyle[0].paintArea(g2D, containingRectArea, false, false, true);
			}
		}
		
		// Set areas
		Area setArea[] = EulerEllipseVisUtils.getSetsAsAreas(eulerEllipseDiagram_scaledToFit, getCentreOfPanel());

		// Get regions as areas
		Ellipse tempEllipseRemoval = null;
		if (eulerEllipseDiagram_scaledToFit.getEllipses().size()>3){
			//this is the case where all the +ve pop and +ve test are surrounded by the -ve pop set and thus there is an extra ellipse to form the donut shape of -ve pop
			tempEllipseRemoval = eulerEllipseDiagram_scaledToFit.getEllipses().get(1);
			eulerEllipseDiagram_scaledToFit.getEllipses().remove(1);
		}
		
		Area[] regionArea = EulerEllipseVisUtils.getAllRegionsAsAreasFor3SetDiag(eulerEllipseDiagram_scaledToFit, getCentreOfPanel());
		ArrayList<String> allPossibleRegionsRef = new ArrayList<String>(Arrays.asList(EllipseDiagramOps.zoneLabels_Venn3));
		
		
		// Show icons 
		if (showIcons){
			
			Rectangle2D.Double icon_scaled = new Rectangle2D.Double(0, 0, iconDimension.getWidth(), iconDimension.getHeight());
			EulerEllipseVisUtils.scaleToFit(icon_scaled, areaPropEulerDiag, getBoundingBox(), getInnerPadding(), getCentreOfPanel());
			DimensionDouble iconDimension_scaled = new DimensionDouble(MathUtils.rounddown(icon_scaled.getWidth()),MathUtils.rounddown(icon_scaled.getHeight()));
			
			switch (iconsPositioning){
			case RANDOM:{

				// Place the icons randomly but not colliding with the rest 
				double totalIconSetFreq;
				Area areaIcon;
				Rectangle2D rectBBoxRegion;
				Boolean iconCollidesWithOthers = null;
				Point2D.Double[] iconsPos;
				Point2D.Double otherIconPos;
				HeterogeneousChannelStyle currVisStyle;
				double x = 0; 
				double y = 0;
				int r = 0;
				int i =0;
				Random random = new Random();
				int randomInt=0;
				int attempts=0;
				int maxAttempts=50;
				for (IconSet iconSet : regionsIconSet){
					
					totalIconSetFreq = iconSet.getFrequency();
					iconsPos = new Point2D.Double[(int)totalIconSetFreq];
					r = allPossibleRegionsRef.indexOf(iconSet.getRef())+1; //+1 since first element returned by getAllRegionsAsAreasFor3SetDiag, is the outer empty set
					
					rectBBoxRegion = regionArea[r].getBounds2D();
					
					
					for (int f=0; f<totalIconSetFreq; f++){

						iconCollidesWithOthers = null;
						while ((iconCollidesWithOthers==null)||(iconCollidesWithOthers==true)){
							
							attempts=0;
							do{
								if (attempts>maxAttempts){
									Toolkit.getDefaultToolkit().beep();
									JOptionPane.showMessageDialog(this, "Cannot draw the glyphs in the diagram.\nThere might be too many glyphs for the diagram.", "Generating an Area-Proportional Euler Diagram with Randomly Positioned Glyphs", JOptionPane.ERROR_MESSAGE);
									return;
								}
								/*
								 * naive method 
								 * -> based on the idea of placing random icons in a rectangle 
								 *    => it will ensure that all the region is covered including areas what are in the bounding rectangle but not in the curve 
								 */
								randomInt = random.nextInt((int)(rectBBoxRegion.getWidth()-iconDimension_scaled.getWidth()));
								x = rectBBoxRegion.getX() + randomInt;
								randomInt = random.nextInt((int)(rectBBoxRegion.getHeight()-iconDimension_scaled.getHeight()));
								y = rectBBoxRegion.getY() + randomInt;
							
								attempts++;
							} while ((!regionArea[r].contains(new Point2D.Double(x,y))) || 
									 (!regionArea[r].contains(new Point2D.Double(x+iconDimension_scaled.getWidth(),y))) || 
									 (!regionArea[r].contains(new Point2D.Double(x,y+iconDimension_scaled.getHeight()))) || 
									 (!regionArea[r].contains(new Point2D.Double(x+iconDimension_scaled.getWidth(),y+iconDimension_scaled.getHeight()))) );
							
							if (iconCollidesWithOthers==null){
								iconCollidesWithOthers = false;								
							}
							
							for(int fcheck=0; fcheck < f; fcheck++){
								/* 
								 * without any extra padding
								 */
								otherIconPos = iconsPos[fcheck];
								otherIconPos = new Point2D.Double(iconsPos[fcheck].x, iconsPos[fcheck].y);
								
								if (((x >= otherIconPos.x) && (x <= (otherIconPos.x+iconDimension_scaled.getWidth())) || ((x + iconDimension_scaled.getWidth()) >= otherIconPos.x) && ((x + iconDimension_scaled.getWidth()) <= (otherIconPos.x+iconDimension_scaled.getWidth()))) &&
									((y >= otherIconPos.y) && (y <= (otherIconPos.y+iconDimension_scaled.getHeight())) || ((y + iconDimension_scaled.getHeight()) >= otherIconPos.y) && ((y + iconDimension_scaled.getHeight()) <= (otherIconPos.y+iconDimension_scaled.getHeight())))){
									iconCollidesWithOthers = true;
									break;
								}else {
									iconCollidesWithOthers = false;
								}
							}
						}
						iconsPos[f] = new Point2D.Double (x,y);
						areaIcon = iconSet.getIconShape().getIconArea(new Point2D.Double(x,y), iconDimension_scaled);
						if ((iconSet.getRef()!=null) && (iconSet.getRef().length()>0)){
							for (char setLabel : iconSet.getRef().toCharArray()){ 
								currVisStyle = (HeterogeneousChannelStyle)VisStyle.findVisStyleWithRef(ellipsesVisStyle, Character.toString(setLabel));					
								if (currVisStyle == null){continue;}
								currVisStyle.paintArea(g2D, areaIcon, showBorderOfFilledCurves, false, true);
							}
						}
					}
					i++;
				}	
				break;
			}
			case UNIFORM:{
				
				Rectangle2D.Double[][] rectsInRegionsScaled = new Rectangle2D.Double[rectsInRegions.length][];
				int regIndex =0;
				for(Rectangle2D.Double[] rectsInRegion : rectsInRegions){
					Rectangle2D.Double[] rectsInRegionScaled = new Rectangle2D.Double[rectsInRegion.length];
					int regionRectIndex = 0;
					for(Rectangle2D.Double rect : rectsInRegion){
						rectsInRegionScaled[regionRectIndex] = new  Rectangle2D.Double(rect.x, rect.y, rect.getWidth(), rect.getHeight());
						regionRectIndex++;
					}
					rectsInRegionsScaled[regIndex] = rectsInRegionScaled;
					regIndex++;
				}
				EulerEllipseVisUtils.scaleToFit(rectsInRegionsScaled, areaPropEulerDiag, getBoundingBox(), getInnerPadding(), getCentreOfPanel());
				
				// Fill up the entire regions with icons 
				int totalIconSetFreq;
				Area areaIcon;
				Rectangle2D rectBBoxRegion;
				Point2D.Double[] iconsPos;
				HeterogeneousChannelStyle currVisStyle;
				int r = 0;
				int i =0;
				
			
				Point2D.Double currRectTopLeftCoord;
				Point2D.Double initTopLeftCoord;
				Rectangle2D.Double currRect = null;
				Rectangle2D.Double displayRect;
				Point2D.Double currRectCentre = new Point2D.Double(0.0,0.0);
				
				
				for (IconSet iconSet : regionsIconSet){
					totalIconSetFreq = iconSet.getFrequency();
					iconsPos = new Point2D.Double[(int)totalIconSetFreq];
					r = allPossibleRegionsRef.indexOf(iconSet.getRef())+1; //+1 since first element returned by getAllRegionsAsAreasFor3SetDiag, is the outer empty set
					
					rectBBoxRegion = regionArea[r].getBounds2D();
					
					double spacing = iconDimension.getWidth()/IconDisplay.ICONSIDE_T0_SPACING;
					int maxNoOfIconsInMaxRect = (int)((GeometryUtils.computeAreaOfRect(rectsInRegionsScaled[i][0]))/(((iconDimension_scaled.getHeight()+spacing)*(iconDimension_scaled.getWidth()))));
					int noOfIconsInMaxRect = Math.min(totalIconSetFreq,maxNoOfIconsInMaxRect);
					int noOfIconsOutOfMaxRect = totalIconSetFreq - noOfIconsInMaxRect;
					if (noOfIconsOutOfMaxRect<0){noOfIconsOutOfMaxRect=0;}
					
					int j = 0;
					for(Rectangle2D.Double rect: rectsInRegionsScaled[i]){
						
						currRect = rectsInRegionsScaled[i][j];
						currRectCentre.x = currRect.x + (currRect.getWidth()/2);
						currRectCentre.y = currRect.y - (currRect.getHeight()/2);
						
						initTopLeftCoord = GeometryUtils.changeCoorSys_originMid_to_originTopLeft(new Point2D.Double(currRect.x, currRect.y), getCentreOfPanel());
						displayRect = currRect;

						currRectTopLeftCoord = new Point2D.Double(initTopLeftCoord.x, initTopLeftCoord.y);
						int noOfIconsToBePlacedOutOfMaxRect = noOfIconsOutOfMaxRect; //Math.min(totalIconSetFreq,noOfIconsOutOfMaxRect);
						
						double leftIconInMaxRect_x = initTopLeftCoord.x;  // x of leftmost edge of the leftmost icons
						double topIconInMaxRect_y = initTopLeftCoord.y;   // y of topmost edge of the topmost icons
						Double rightIconInMaxRect_x = null;		           // x of rightmost edge of the rightmost icons
						Double bottomIconInMaxRect_y = null;			 // y of bottommost edge of the bottommost icons
						
						int f=0;
						
						noOfIconsToBePlacedOutOfMaxRect += (noOfIconsInMaxRect-f);
						
						// fill up the max rect to the max 
						while ((noOfIconsToBePlacedOutOfMaxRect>0) && 
							   (currRectTopLeftCoord.x + iconDimension_scaled.getWidth() + spacing) <= (initTopLeftCoord.x + displayRect.getWidth()) && 
							   (currRectTopLeftCoord.y + iconDimension_scaled.getHeight() + spacing) <= (initTopLeftCoord.y + displayRect.getHeight()) && 
							   ((regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y))) && 
							   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y))) &&
							   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y+iconDimension_scaled.getHeight()))) &&
							   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y+iconDimension_scaled.getHeight()))))){ 
						
							areaIcon = iconSet.getIconShape().getIconArea(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y), iconDimension_scaled);
							
							if ((iconSet.getRef()!=null) && (iconSet.getRef().length()>0)){
								for (char setLabel : iconSet.getRef().toCharArray()){ 
									currVisStyle = (HeterogeneousChannelStyle)VisStyle.findVisStyleWithRef(ellipsesVisStyle, Character.toString(setLabel));					
									if (currVisStyle == null){continue;}
									currVisStyle.paintArea(g2D, areaIcon, showBorderOfFilledCurves, false, true);
								}
							}
							
							noOfIconsToBePlacedOutOfMaxRect--;
							
							currRectTopLeftCoord.x += iconDimension_scaled.getWidth() + spacing;
							if ((currRectTopLeftCoord.x + iconDimension_scaled.getWidth()) > (initTopLeftCoord.x + displayRect.getWidth())){
								if (rightIconInMaxRect_x == null){
									rightIconInMaxRect_x = currRectTopLeftCoord.x;
								}
								currRectTopLeftCoord.x = initTopLeftCoord.x;
								currRectTopLeftCoord.y += iconDimension_scaled.getHeight() + spacing;
							}
						}
						
						if (bottomIconInMaxRect_y == null){
							bottomIconInMaxRect_y = currRectTopLeftCoord.y;
						}
						
				
						// icons to the right of the max rect 
						rightIconInMaxRect_x = (rightIconInMaxRect_x == null) ? rect.getX(): rightIconInMaxRect_x; 
						currRectTopLeftCoord.x = rightIconInMaxRect_x; 
						currRectTopLeftCoord.y = topIconInMaxRect_y;
						
						while ((noOfIconsToBePlacedOutOfMaxRect>0) &&  
							   ((currRectTopLeftCoord.y + iconDimension_scaled.getHeight()) <= (initTopLeftCoord.y + displayRect.getHeight())) ) {
							
							   if ((regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y))) && 
									   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y))) &&
									   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y+iconDimension_scaled.getHeight()))) &&
									   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y+iconDimension_scaled.getHeight())))){

									areaIcon = iconSet.getIconShape().getIconArea(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y), iconDimension_scaled);
									
									if ((iconSet.getRef()!=null) && (iconSet.getRef().length()>0)){
										for (char setLabel : iconSet.getRef().toCharArray()){ 
											currVisStyle = (HeterogeneousChannelStyle)VisStyle.findVisStyleWithRef(ellipsesVisStyle, Character.toString(setLabel));					
											if (currVisStyle == null){continue;}
											currVisStyle.paintArea(g2D, areaIcon, showBorderOfFilledCurves, false, true);
										}
									}
								
									noOfIconsToBePlacedOutOfMaxRect--;
								}
								
								currRectTopLeftCoord.x += iconDimension_scaled.getWidth() + spacing;
								if ( !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y))) ||
								     !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y))) ||
									 !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y+iconDimension_scaled.getHeight()))) ||
									 !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y+iconDimension_scaled.getHeight())))){

									currRectTopLeftCoord.x = rightIconInMaxRect_x;
									currRectTopLeftCoord.y += iconDimension_scaled.getHeight() + spacing;
								}
						}
						
						// icons to the bottom of the max rect
						currRectTopLeftCoord.x = leftIconInMaxRect_x; 
						currRectTopLeftCoord.y = bottomIconInMaxRect_y;
						
						while ((noOfIconsToBePlacedOutOfMaxRect>0) &&  
							   (currRectTopLeftCoord.y + iconDimension_scaled.getHeight()) <= (rectBBoxRegion.getY() + rectBBoxRegion.getHeight()) ) {
							
							   if ((regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y))) && 
									   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y))) &&
									   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y+iconDimension_scaled.getHeight()))) &&
									   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y+iconDimension_scaled.getHeight())))){

									areaIcon = iconSet.getIconShape().getIconArea(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y), iconDimension_scaled);
									
									if ((iconSet.getRef()!=null) && (iconSet.getRef().length()>0)){
										for (char setLabel : iconSet.getRef().toCharArray()){ 
											currVisStyle = (HeterogeneousChannelStyle)VisStyle.findVisStyleWithRef(ellipsesVisStyle, Character.toString(setLabel));					
											if (currVisStyle == null){continue;}
											currVisStyle.paintArea(g2D, areaIcon, showBorderOfFilledCurves, false, true);
										}
									}
								
									noOfIconsToBePlacedOutOfMaxRect--;
								}
								
								currRectTopLeftCoord.x += iconDimension_scaled.getWidth() + spacing;
								if ( !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y))) ||
								     !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y))) ||
									 !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y+iconDimension_scaled.getHeight()))) ||
									 !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y+iconDimension_scaled.getHeight())))){

									if ((currRectTopLeftCoord.x + iconDimension_scaled.getWidth()) > (initTopLeftCoord.x + displayRect.getWidth())){
										currRectTopLeftCoord.x = leftIconInMaxRect_x;
										currRectTopLeftCoord.y += iconDimension_scaled.getHeight() + spacing;
									}
								}
						}
						
						// icons to the top of the max rect
						currRectTopLeftCoord.x = leftIconInMaxRect_x; 
						currRectTopLeftCoord.y = topIconInMaxRect_y - spacing - iconDimension_scaled.getHeight();
						
						while ((noOfIconsToBePlacedOutOfMaxRect>0) &&  
							   ((currRectTopLeftCoord.y - iconDimension_scaled.getHeight()) > rectBBoxRegion.getY()) ) {
							
							   if ((regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y))) && 
									   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y))) &&
									   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y+iconDimension_scaled.getHeight()))) &&
									   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y+iconDimension_scaled.getHeight())))){

									areaIcon = iconSet.getIconShape().getIconArea(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y), iconDimension_scaled);
									
									if ((iconSet.getRef()!=null) && (iconSet.getRef().length()>0)){
										for (char setLabel : iconSet.getRef().toCharArray()){ 
											currVisStyle = (HeterogeneousChannelStyle)VisStyle.findVisStyleWithRef(ellipsesVisStyle, Character.toString(setLabel));					
											if (currVisStyle == null){continue;}
											currVisStyle.paintArea(g2D, areaIcon, showBorderOfFilledCurves, false, true);
										}
									}
								
									noOfIconsToBePlacedOutOfMaxRect--;
								}
								
								currRectTopLeftCoord.x += iconDimension_scaled.getWidth() + spacing;
								if ( !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y))) ||
								     !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y))) ||
									 !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y+iconDimension_scaled.getHeight()))) ||
									 !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y+iconDimension_scaled.getHeight())))){

									if ((currRectTopLeftCoord.x + iconDimension_scaled.getWidth()) > (initTopLeftCoord.x + displayRect.getWidth())){
										currRectTopLeftCoord.x = leftIconInMaxRect_x;
										currRectTopLeftCoord.y -= iconDimension_scaled.getHeight() + spacing;
									}

								}

						}
						
						// icons to the left of the max rect 
						currRectTopLeftCoord.x = leftIconInMaxRect_x - iconDimension_scaled.getWidth() - spacing; 
						currRectTopLeftCoord.y = topIconInMaxRect_y;
						
						while ((noOfIconsToBePlacedOutOfMaxRect>0) &&  
							   ((currRectTopLeftCoord.y + iconDimension_scaled.getHeight()) <= (initTopLeftCoord.y + displayRect.getHeight())) ) {
							
							   if ((regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y))) && 
									   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y))) &&
									   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y+iconDimension_scaled.getHeight()))) &&
									   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y+iconDimension_scaled.getHeight())))){

									areaIcon = iconSet.getIconShape().getIconArea(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y), iconDimension_scaled);
									
									if ((iconSet.getRef()!=null) && (iconSet.getRef().length()>0)){
										for (char setLabel : iconSet.getRef().toCharArray()){ 
											currVisStyle = (HeterogeneousChannelStyle)VisStyle.findVisStyleWithRef(ellipsesVisStyle, Character.toString(setLabel));					
											if (currVisStyle == null){continue;}
											currVisStyle.paintArea(g2D, areaIcon, showBorderOfFilledCurves, false, true);
										}
									}
								
									noOfIconsToBePlacedOutOfMaxRect--;
								}
								
								currRectTopLeftCoord.x -= iconDimension_scaled.getWidth() + spacing;
								if ( !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y))) ||
								     !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y))) ||
									 !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y+iconDimension_scaled.getHeight()))) ||
									 !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y+iconDimension_scaled.getHeight())))){

									currRectTopLeftCoord.x = leftIconInMaxRect_x - iconDimension_scaled.getWidth() - spacing;
									currRectTopLeftCoord.y += iconDimension_scaled.getHeight() + spacing;
								}
						}
						
						// icons at the top left corner of the max rect
						currRectTopLeftCoord.x = leftIconInMaxRect_x - iconDimension_scaled.getWidth() - spacing; 
						currRectTopLeftCoord.y = topIconInMaxRect_y;
						
						while ((noOfIconsToBePlacedOutOfMaxRect>0) &&  
								((currRectTopLeftCoord.y - iconDimension_scaled.getHeight()) > rectBBoxRegion.getY()) ) {
							
							   if ((regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y))) && 
									   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y))) &&
									   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y+iconDimension_scaled.getHeight()))) &&
									   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y+iconDimension_scaled.getHeight())))){

									areaIcon = iconSet.getIconShape().getIconArea(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y), iconDimension_scaled);
									
									if ((iconSet.getRef()!=null) && (iconSet.getRef().length()>0)){
										for (char setLabel : iconSet.getRef().toCharArray()){ 
											currVisStyle = (HeterogeneousChannelStyle)VisStyle.findVisStyleWithRef(ellipsesVisStyle, Character.toString(setLabel));					
											if (currVisStyle == null){continue;}
											currVisStyle.paintArea(g2D, areaIcon, showBorderOfFilledCurves, false, true);
										}
									}
								
									noOfIconsToBePlacedOutOfMaxRect--;
								}
								
								currRectTopLeftCoord.x -= iconDimension_scaled.getWidth() + spacing;
								if ( !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y))) ||
								     !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y))) ||
									 !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y+iconDimension_scaled.getHeight()))) ||
									 !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y+iconDimension_scaled.getHeight())))){

									currRectTopLeftCoord.x = leftIconInMaxRect_x - iconDimension_scaled.getWidth() - spacing;
									currRectTopLeftCoord.y -= iconDimension_scaled.getHeight() + spacing;
								}
						}
						
						// icons at the bottom left corner of the max rect
						currRectTopLeftCoord.x = leftIconInMaxRect_x - iconDimension_scaled.getWidth() - spacing; 
						currRectTopLeftCoord.y = bottomIconInMaxRect_y;
						
						while ((noOfIconsToBePlacedOutOfMaxRect>0) &&  
								((currRectTopLeftCoord.y + iconDimension_scaled.getHeight()) < (rectBBoxRegion.getY()+rectBBoxRegion.getHeight())) ) {
							
							   if ((regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y))) && 
									   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y))) &&
									   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y+iconDimension_scaled.getHeight()))) &&
									   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y+iconDimension_scaled.getHeight())))){

									areaIcon = iconSet.getIconShape().getIconArea(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y), iconDimension_scaled);
									
									if ((iconSet.getRef()!=null) && (iconSet.getRef().length()>0)){
										for (char setLabel : iconSet.getRef().toCharArray()){ 
											currVisStyle = (HeterogeneousChannelStyle)VisStyle.findVisStyleWithRef(ellipsesVisStyle, Character.toString(setLabel));					
											if (currVisStyle == null){continue;}
											currVisStyle.paintArea(g2D, areaIcon, showBorderOfFilledCurves, false, true);
										}
									}
								
									noOfIconsToBePlacedOutOfMaxRect--;
								}
								
								currRectTopLeftCoord.x -= iconDimension_scaled.getWidth() + spacing;
								if ( !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y))) ||
								     !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y))) ||
									 !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y+iconDimension_scaled.getHeight()))) ||
									 !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y+iconDimension_scaled.getHeight())))){

									currRectTopLeftCoord.x = leftIconInMaxRect_x - iconDimension_scaled.getWidth() - spacing;
									currRectTopLeftCoord.y += iconDimension_scaled.getHeight() + spacing;
								}
						}
						
						
						
						// icons at the top right corner of the max rect
						currRectTopLeftCoord.x = rightIconInMaxRect_x; 
						currRectTopLeftCoord.y = topIconInMaxRect_y;
						
						while ((noOfIconsToBePlacedOutOfMaxRect>0) &&  
								((currRectTopLeftCoord.y - iconDimension_scaled.getHeight()) > rectBBoxRegion.getY()) ) {
							
							   if ((regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y))) && 
									   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y))) &&
									   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y+iconDimension_scaled.getHeight()))) &&
									   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y+iconDimension_scaled.getHeight())))){

									areaIcon = iconSet.getIconShape().getIconArea(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y), iconDimension_scaled);
									
									if ((iconSet.getRef()!=null) && (iconSet.getRef().length()>0)){
										for (char setLabel : iconSet.getRef().toCharArray()){ 
											currVisStyle = (HeterogeneousChannelStyle)VisStyle.findVisStyleWithRef(ellipsesVisStyle, Character.toString(setLabel));					
											if (currVisStyle == null){continue;}
											currVisStyle.paintArea(g2D, areaIcon, showBorderOfFilledCurves, false, true);
										}
									}
								
									noOfIconsToBePlacedOutOfMaxRect--;
								}
								
								currRectTopLeftCoord.x += iconDimension_scaled.getWidth() + spacing;
								if ( !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y))) ||
								     !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y))) ||
									 !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y+iconDimension_scaled.getHeight()))) ||
									 !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y+iconDimension_scaled.getHeight())))){

									currRectTopLeftCoord.x = rightIconInMaxRect_x;
									currRectTopLeftCoord.y -= iconDimension_scaled.getHeight() + spacing;
								}
						}
						
						// icons at the bottom right corner of the max rect
						currRectTopLeftCoord.x = rightIconInMaxRect_x; 
						currRectTopLeftCoord.y = bottomIconInMaxRect_y;
						
						while ((noOfIconsToBePlacedOutOfMaxRect>0) &&  
								((currRectTopLeftCoord.y + iconDimension_scaled.getHeight()) < (rectBBoxRegion.getY()+rectBBoxRegion.getHeight())) ) {
							
							   if ((regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y))) && 
									   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y))) &&
									   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y+iconDimension_scaled.getHeight()))) &&
									   (regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y+iconDimension_scaled.getHeight())))){

									areaIcon = iconSet.getIconShape().getIconArea(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y), iconDimension_scaled);
									
									if ((iconSet.getRef()!=null) && (iconSet.getRef().length()>0)){
										for (char setLabel : iconSet.getRef().toCharArray()){ 
											currVisStyle = (HeterogeneousChannelStyle)VisStyle.findVisStyleWithRef(ellipsesVisStyle, Character.toString(setLabel));					
											if (currVisStyle == null){continue;}
											currVisStyle.paintArea(g2D, areaIcon, showBorderOfFilledCurves, false, true);
										}
									}
									
									noOfIconsToBePlacedOutOfMaxRect--;
								}
								
								currRectTopLeftCoord.x -= iconDimension_scaled.getWidth() + spacing;
								if ( !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y))) ||
								     !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y))) ||
									 !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x,currRectTopLeftCoord.y+iconDimension_scaled.getHeight()))) ||
									 !(regionArea[r].contains(new Point2D.Double(currRectTopLeftCoord.x+iconDimension_scaled.getWidth(),currRectTopLeftCoord.y+iconDimension_scaled.getHeight())))){

									currRectTopLeftCoord.x = rightIconInMaxRect_x;
									currRectTopLeftCoord.y += iconDimension_scaled.getHeight() + spacing;
								}
						}
						j++;
					}
					i++;
				}	
		
				break;
			}
			default: break;
			}
		}

		
		// Draw the ellipses
		for (int i = 0; i < setArea.length; i++) {
			if ((this.containingRectangle != null) && (i==0)){continue;} //don't paint the outer first ellipse if a rectangle was passed for it
			ellipsesVisStyle[i].paintArea(g2D, setArea[i], showBorderOfFilledCurves || showIcons, showIcons, !showIcons);
		}
		

		// Draw set labels
		if (this.showSetsLabel){
			int i = 0;
			int noOfEmptyLabels=0;
			for (String label: this.setsLabel){
				if ((label==null)||(label.equals(""))){i++;noOfEmptyLabels++;continue;}
				Color setColour = ((HeterogeneousChannelStyle)VisStyle.findVisStyleWithRef(ellipsesVisStyle,ellipsesRefLabel[i-noOfEmptyLabels])).getOutlineColour();
				if ((i==0) && (containingRectArea != null)){		
					VisLabelling.placeLabelAdjacentToArea(g2D, containingRectArea, label, setsLabelPlacement[i], setsLabelTextAlignment[i], fontName_sets, fontStyle_sets, fontSize_sets, setColour, multiLineSpacing); //colourText);
				} else {
					VisLabelling.placeLabelAdjacentToArea(g2D, setArea[i], label, setsLabelPlacement[i], setsLabelTextAlignment[i], fontName_sets, fontStyle_sets, fontSize_sets, setColour, multiLineSpacing); //colourText);
				}
				i++;
			}
		}
		
		
		// Draw region labels 
		if (this.showRegionsLabel){
			int i = 0; 
			int r = 0;
			for (String label: this.regionsLabel){
				if (regionsInDiagram != null){
					r = allPossibleRegionsRef.indexOf(regionsInDiagram[i]) + 1; //+1 since first element returned by getAllRegionsAsAreasFor3SetDiag, is the outer empty set
				} else {
					r = i;
				}
				VisLabelling.placeLabelAtCentreOfRegionArea(g2D, regionArea[r], label, fontName_regions, fontStyle_regions, fontSize_regions, colourText);
				i++;
			}
		}
		
		
		
		// Show legend
		//    currently the legend is placed below the diagram aligned to the left side of the diagram 
		// ** this legend must be consistent and the same for all the visualizations 
		
		int spacingBetweenSetLegend = 60;
		
		Dimension setLegendIconDimension = (iconDimension==null) ? IconDisplay.ICON_DIMENSION_DEFAULT : iconDimension;
		Area setLegendArea = null;
		HeterogeneousChannelStyle currVisStyle;
		Rectangle2D.Double textBBox = new Rectangle2D.Double();
		
		Dimension legendDimension = new Dimension(0,0);
		Point2D.Double currRectTopLeftCoordForLegend = new Point2D.Double (0,0);
		if (this.showLegend){
			int i=0;
			String prevSetRefLabel = null;
			Color textLegendColour = null;
			
			ArrayList<Area> areasToPaint = new ArrayList<Area>();
			ArrayList<HeterogeneousChannelStyle> visStyleForAreasToPaint = new ArrayList<HeterogeneousChannelStyle>();
			ArrayList<String> setsLabelToPaint = new ArrayList<String>();
			String prevSetLabelText = "1 of the "; //\\n";
			
			for (String setRefLabel : ellipsesRefLabel){
				if((prevSetRefLabel!=null) && (prevSetRefLabel.equals(setRefLabel))){
					prevSetRefLabel = setRefLabel;
					i++;
					continue;
				}

				if ((setRefLabel!=null) && (setRefLabel.length()>0)){
					for (char setLabel : setRefLabel.toCharArray()){ 
						currVisStyle = (HeterogeneousChannelStyle)VisStyle.findVisStyleWithRef(ellipsesVisStyle, Character.toString(setLabel));	
						if (!showBorderOfFilledCurves && currVisStyle.canBeDrawnWithoutABorder()){  //if the border is not drawn and the style always you not to draw the border as in the Solid Fill style, then increase the width and height of the icon by the outline thickness multiplied by 2 to ensure that the size of this legend icon is the same as those which have to be drawn with an outline (like the styles which are not filled in)
							setLegendArea = IconSet.IconShape.ELLIPSE.getIconArea(currRectTopLeftCoordForLegend, new DimensionDouble(setLegendIconDimension.width+(currVisStyle.getOutlineThickness()*2)+0, setLegendIconDimension.height+(currVisStyle.getOutlineThickness()*2)+0));
						} else {
							setLegendArea = IconSet.IconShape.ELLIPSE.getIconArea(currRectTopLeftCoordForLegend, setLegendIconDimension);
						}
						if (currVisStyle == null){continue;}
						areasToPaint.add(setLegendArea);
						visStyleForAreasToPaint.add(currVisStyle);
						setsLabelToPaint.add(prevSetLabelText + setsLabel[i]);
						textLegendColour = currVisStyle.getOutlineColour();
					}
				}				
				if ((this.setsLabel[i]!=null)&&(!this.setsLabel[i].equals(""))){
					if (textLegendColour == null){textLegendColour = colourText;}
					textBBox = VisLabelling.placeLabelAdjacentToArea(g2D, setLegendArea, StringUtils.removeExtraSpaces(StringUtils.removeBreakLines(prevSetLabelText + this.setsLabel[i])), Placement.EAST, Alignment.LEFT, fontName_sets, Font.PLAIN, fontSize_sets, textLegendColour, multiLineSpacing, false); 
				}
				currRectTopLeftCoordForLegend.x += (setLegendIconDimension.getWidth() + textBBox.getWidth() + spacingBetweenSetLegend);
				legendDimension.width = (int) (textBBox.getBounds().x + textBBox.getWidth());
				if (legendDimension.height < textBBox.getHeight()){
					legendDimension.height = (int) (textBBox.getBounds().y+textBBox.getHeight());
				}
				prevSetRefLabel = setRefLabel;
				i++;
			}
			Point2D.Double displaceBy = new Point2D.Double ((getWidth()-legendDimension.width)/2,(getHeight()-legendDimension.height-10));
			AffineTransform affinetransform = new AffineTransform();
			int areasToPaintCount = areasToPaint.size();
			Rectangle areaBounds;
			for (int a=0; a<areasToPaintCount; a++){
				areaBounds = areasToPaint.get(a).getBounds();
				affinetransform = new AffineTransform();
				affinetransform.translate(displaceBy.x, displaceBy.y);
				areasToPaint.get(a).transform(affinetransform);
				visStyleForAreasToPaint.get(a).paintArea(g2D, areasToPaint.get(a), showBorderOfFilledCurves, false, true);
				
				textLegendColour = visStyleForAreasToPaint.get(a).getOutlineColour();
				if ((setsLabelToPaint.get(a)!=null)&&(!setsLabelToPaint.get(a).equals(""))){
					if (textLegendColour == null){textLegendColour = colourText;}
					textBBox = VisLabelling.placeLabelAdjacentToArea(g2D, areasToPaint.get(a), StringUtils.removeExtraSpaces(StringUtils.removeBreakLines(setsLabelToPaint.get(a))), Placement.EAST, Alignment.LEFT, fontName_sets, Font.PLAIN, fontSize_sets, textLegendColour, multiLineSpacing); // colourText, multiLineSpacing);
				}
			}
		}

	}
	
	
	// Methods
	public void refresh(){
		paintImmediately(getBoundingBox());
	}


	@Override
	public boolean isEmpty() {
		return (this.eulerEllipseDiagram == null);
	}
}
