/* 
 * 
 * eulerGlyphs v0.9.0
 * 
 * 2013-12-18
 *
 * 
 * 
 * eulerGlyphs -- Drawing Area-Proportional Euler Diagrams With Glyphs	    
 * 		http://www.eulerdiagrams.org/eulerGlyphs
 * 
 * 
 * 		Copyright (C) 2012-2013, Luana Micallef. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerGlyphs.
 * 			
 * 		eulerGlyphs is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerGlyphs is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerGlyphs (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */

 


package uicomponent.vispanel;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.awt.geom.Area;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import pictograph.IconDisplay;
import pictograph.IconSet;
import pictograph.IconSet.IconShape;
import uicomponent.utils.panel.JPanelUtils;
import uicomponent.vispanel.VisUtils.*;
import utils.DimensionDouble;
import utils.MathUtils;
import utils.StringUtils;
import visforfallacy.IconDisplayForNeglectBaseRate;
import visstyle.HeterogeneousChannelStyle;
import visstyle.VisStyle;


/**
 * 
 * To instantiate,define and handle panels for icon-based displays (or frequency grids)
 *
 */



public class IconDisplayPanel extends VisPanel  {
	
	// Class Constants
	private static final long serialVersionUID = 1L;
	
	public static final String FONT_NAME_DEFAULT = "Arial";
	public static final int FONT_STYLE_DEFAULT = Font.PLAIN;
	public static final int FONT_SIZE_DEFAULT = 14; //12;
	public static final Color COLOUR_BACKGROUND_DEFAULT = Color.white;
	public static final Color COLOUR_TEXT_DEFAULT = Color.black;//darkGray;
	public static final boolean SHOW_SCALE_DEFAULT = true;
	public static final Integer GROUP_BY_VALUE_DEFAULT = null;
	
	// Constants 
	public boolean showBorderOfFilledCurves = false;
	
	
	// Data fields
	protected DimensionDouble iconDimension;
	protected IconDisplay iconDisplay;
	protected HeterogeneousChannelStyle[] setsVisStyle;
	protected boolean showSetsLabel;
	protected String[] setsLabel;
	protected Placement[] setsLabelPlacement;
	protected Alignment[] setsLabelTextAlignment;
	protected boolean showRegionsLabel;
	protected String[] regionsLabel;
	protected boolean showLegend;
	protected boolean showScale = SHOW_SCALE_DEFAULT;
	protected Integer groupingValue = GROUP_BY_VALUE_DEFAULT;
	protected Boolean needLargerPopSizeToDisplayIcons=false;
	protected int popSize;
	
	protected String fontName_sets = FONT_NAME_DEFAULT;
	protected int fontStyle_sets = Font.BOLD;
	protected int fontSize_sets =  FONT_SIZE_DEFAULT;
	protected String fontName_regions = FONT_NAME_DEFAULT;
	protected int fontStyle_regions = FONT_STYLE_DEFAULT;
	protected int fontSize_regions =  FONT_SIZE_DEFAULT;
	protected Color colourBackground = COLOUR_BACKGROUND_DEFAULT;
	protected Color colourText = COLOUR_TEXT_DEFAULT;
	protected int multiLineSpacing = 6;  // the number of pixels between lines in a multi-line label
	protected boolean setBackground = true;

	
	// Constructor 
	public IconDisplayPanel(){
		super();
	}


	// Getters
	public IconDisplay getIconDisplay (){
		return this.iconDisplay;
	}
	public HeterogeneousChannelStyle[] getIconSetsVisStyle(){
		return this.setsVisStyle;
	}
	public boolean getShowSetsLabel(){
		return this.showSetsLabel;
	}
	public String[] getSetsLabel(){
		return this.setsLabel;
	}
	public Placement[] getSetsLabelPlacement(){
		return this.setsLabelPlacement;
	}
	public Alignment[] getSetsLabelTextAlignment(){
		return this.setsLabelTextAlignment;
	}
	public boolean getShowRegionsLabel(){
		return this.showRegionsLabel;
	}
	public String[] getRegionsLabel(){
		return this.regionsLabel;
	}
	public boolean getShowLegend(){
		return this.showLegend;
	}
	public boolean getShowScale(){
		return this.showScale;
	}
	public Integer getGroupingValue(){
		return this.groupingValue;
	}
	
	public String getFontNameSets(){
		return this.fontName_sets;
	}
	public int getFontStyleSets(){
		return this.fontStyle_sets;
	}
	public int getFontSizeSets(){
		return this.fontSize_sets;
	}
	public String getFontNameRegions(){
		return this.fontName_regions;
	}
	public int getFontStyleRegions(){
		return this.fontStyle_regions;
	}
	public int getFontSizeRegions(){
		return this.fontSize_regions;
	}
	public Color getColourBackground(){
		return this.colourBackground;
	}
	public Color getColourText(){
		return this.colourText;
	}
	public boolean getSetBackground(){
		return this.setBackground;
	}
	
	
	// Setters
	public void setIconDimension(DimensionDouble iconDimension){
		this.iconDimension = iconDimension;
	}
	public void setIconDisplay (IconDisplay iconDisplay){
		this.iconDisplay = iconDisplay;
	}
	public void setIconSetsVisStyleEuler(HeterogeneousChannelStyle[] iconSetsVisStyle){
		this.setsVisStyle = iconSetsVisStyle;
	} 
	public void setShowSetsLabel(boolean showSetsLabel){
		this.showSetsLabel = showSetsLabel;
	}
	public void setSetsLabel(String[] setsLabel){
		this.setsLabel = setsLabel;
	}
	public void setSetsLabelPlacement(Placement[] setsLabelPlacement){
		this.setsLabelPlacement = setsLabelPlacement;
	}
	public void setSetsLabelTextAlignment(Alignment[] setsLabelTextAlignment){
		this.setsLabelTextAlignment = setsLabelTextAlignment;
	}
	public void setShowRegionsLabel(boolean showRegionsLabel){
		this.showRegionsLabel = showRegionsLabel;
	}
	public void setRegionsLabel(String[] regionsLabel){
		this.regionsLabel = regionsLabel;
	}
	public void setShowLegend(boolean showLegend){
		this.showLegend = showLegend;
	}
	public void setShowScale(boolean showScale){
		this.showScale = showScale;
	}
	public void setGroupingValue (Integer groupingValue){
		if (MathUtils.isDecimal(Math.sqrt(groupingValue))){
			this.groupingValue = null;
		} else {
			this.groupingValue = groupingValue;
		}
	}
	public void setNeedLargerPopSizeToDisplayIcons(Boolean needLargerPopSizeToDisplayIcons){
		this.needLargerPopSizeToDisplayIcons = needLargerPopSizeToDisplayIcons;
	}	
	public void setPopSize(int popSize){
		this.popSize = popSize;
	}
	
	
	public void setFontNameSets (String fontName_sets){
		this.fontName_sets = fontName_sets;
	}
	public void setFontStyleSets (int fontStyle_sets){
		this.fontStyle_sets = fontStyle_sets;
	}
	public void setFontSizeSets (int fontSize_sets){
		this.fontSize_sets = fontSize_sets;
	}
	public void setFontNameRegions (String fontName_regions){
		this.fontName_regions = fontName_regions;
	}
	public void setFontStyleRegions (int fontStyle_regions){
		this.fontStyle_regions = fontStyle_regions;
	}
	public void setFontSizeRegions (int fontSize_regions){
		this.fontSize_regions = fontSize_regions;
	} 
	public void setColourBackground (Color colourBackground){
		this.colourBackground = colourBackground;
	}
	public void setColourText (Color colourText){
		this.colourText = colourText;
	}
	public void setSetBackground (boolean setBackground){
		this.setBackground = setBackground;
	}

	
	
	// Inherited/Overriden Methods 
	
	public void paint(Graphics g) {
		paintComponent(g);
	}
	
	public void paintComponent (Graphics g) {
		// scales diagram to fit in panel

		// Graphics 2D
		Graphics2D g2D = (Graphics2D) g;
		JPanelUtils.setGraphics2DAntiAliasing(g2D, true);
		
		// Paint background
		if (setBackground){
			setBackground(colourBackground);
			super.paintComponent(g2D);
		}
		
		// When a diagram is not available 
		if (iconDisplay == null){return;}
		
		// if cannot display icons with such population size
		if (needLargerPopSizeToDisplayIcons){
			return;
		}
		
		
		// Scale the displayed diagram not the actual one -> this avoid confusion with requiredAreaSpecs and currZoneAreas
		IconDisplay iconDisplay_scaledToFit = iconDisplay.clone();
		
		
		// Start the display 
		Point2D.Double displayTopLeftCoord = iconDisplay_scaledToFit.getTopLeftCoordsOfCentredDisplay(getBoundingBox(), groupingValue);
		DimensionDouble displayDimensionAsPixels = iconDisplay_scaledToFit.getDimensionAsPixels(groupingValue); 
		
		DimensionDouble iconDimension = iconDisplay_scaledToFit.getIconDimension();
		double iconWidth = iconDimension.getWidth();
		double iconHeight = iconDimension.getHeight();
		double iconHorizSpacing = iconDisplay_scaledToFit.getIconHorizontalSpacing();
		double iconVertSpacing = iconDisplay_scaledToFit.getIconVerticalSpacing();
		
		Point2D.Double currRectTopLeftCoord = new Point2D.Double (displayTopLeftCoord.x, displayTopLeftCoord.y);
		IconShape iconSetShape;
		Area areaIcon;
		BufferedImage imageForIcon;
		
		HeterogeneousChannelStyle currVisStyle;
		
		ArrayList<IconSet> iconSetsToDrawIcons = iconDisplay_scaledToFit.getIconSetsToDrawIcons();
		// this will display row by row, top to bottom, left to right => the required ordering has to be done by getIconSetsToDrawIcons()
		int sqrtGroupByValue = ((groupingValue!=null) && (groupingValue>1)) ? (int) Math.sqrt((groupingValue)) : 0;
		int c = 0;
		int r = 0;
		for (IconSet iconSet : iconSetsToDrawIcons){
			iconSetShape = iconSet.getIconShape();
			
			if ((iconSet.getRef()!=null) && (iconSet.getRef().length()>0)){
				for (char setLabel : iconSet.getRef().toCharArray()){ 
					currVisStyle = (HeterogeneousChannelStyle)VisStyle.findVisStyleWithRef(setsVisStyle, Character.toString(setLabel));		
					if (!showBorderOfFilledCurves && currVisStyle.canBeDrawnWithoutABorder()){  //if the border is not drawn and the style always you not to draw the border as in the Solid Fill style, then increase the width and height of the icon by the outline thickness multiplied by 2 to ensure that the size of this legend icon is the same as those which have to be drawn with an outline (like the styles which are not filled in)
						areaIcon = iconSetShape.getIconArea(currRectTopLeftCoord, new DimensionDouble(iconDimension.width+(currVisStyle.getOutlineThickness()*2)-1, iconDimension.height+(currVisStyle.getOutlineThickness()*2)-1));
					} else {
						areaIcon = iconSetShape.getIconArea(currRectTopLeftCoord, iconDimension);
					}	
					if (currVisStyle == null){continue;}
					currVisStyle.paintArea(g2D, areaIcon, showBorderOfFilledCurves, false, true);
				}
			}
	
			c++;
			currRectTopLeftCoord.x += iconWidth + (( ((groupingValue!=null)&& (groupingValue>1) && ((c % sqrtGroupByValue)==0)) ? MathUtils.noOfTimesANumberFactorizesAnother(c, sqrtGroupByValue)+1 : 1) * iconHorizSpacing);
			if (currRectTopLeftCoord.x > (displayTopLeftCoord.x + displayDimensionAsPixels.getWidth())){
				currRectTopLeftCoord.x = displayTopLeftCoord.x;
				r++;
				currRectTopLeftCoord.y += iconHeight + (( ((groupingValue!=null)&& (groupingValue>1) && ((r % sqrtGroupByValue)==0)) ? MathUtils.noOfTimesANumberFactorizesAnother(r, sqrtGroupByValue)+1 : 1) * iconVertSpacing);
				c=0;
			}
		}

		// Draw scale
		if (showScale){
			iconDisplay_scaledToFit.drawScale(g2D, displayTopLeftCoord, groupingValue, fontName_sets, fontStyle_sets, fontSize_sets, colourText);

		}
		
		
		int spacingBetweenSetLegend = 60;
		
		Dimension setLegendIconDimension = iconDimension;
		Area setLegendArea = null;
		
		Rectangle2D.Double textBBox = new Rectangle2D.Double();
		Color textLegendColour = null;
		Dimension legendDimension = new Dimension(0,0);
		Point2D.Double currRectTopLeftCoordForLegend = new Point2D.Double (0,0);
		
		if (this.showSetsLabel || this.showLegend){
			int i=0;
		
			ArrayList<Area> areasToPaint = new ArrayList<Area>();
			ArrayList<HeterogeneousChannelStyle> visStyleForAreasToPaint = new ArrayList<HeterogeneousChannelStyle>();
			ArrayList<String> setsLabelToPaint = new ArrayList<String>();
			String prevSetLabelText = "1 of the ";
			
			for (String setRefLabel : IconDisplayForNeglectBaseRate.setsRefLabel){
				if ((setRefLabel!=null) && (setRefLabel.length()>0)){
					for (char setLabel : setRefLabel.toCharArray()){ 
						currVisStyle = (HeterogeneousChannelStyle)VisStyle.findVisStyleWithRef(setsVisStyle, Character.toString(setLabel));	
						if (!showBorderOfFilledCurves && currVisStyle.canBeDrawnWithoutABorder()){  //if the border is not drawn and the style always you not to draw the border as in the Solid Fill style, then increase the width and height of the icon by the outline thickness multiplied by 2 to ensure that the size of this legend icon is the same as those which have to be drawn with an outline (like the styles which are not filled in)
							setLegendArea = IconSet.IconShape.ELLIPSE.getIconArea(currRectTopLeftCoordForLegend, new DimensionDouble(setLegendIconDimension.width+(currVisStyle.getOutlineThickness()*2)-1, setLegendIconDimension.height+(currVisStyle.getOutlineThickness()*2)-1));
						} else {
							setLegendArea = IconSet.IconShape.ELLIPSE.getIconArea(currRectTopLeftCoordForLegend, setLegendIconDimension);
						}
						if (currVisStyle == null){continue;}
						areasToPaint.add(setLegendArea);
						visStyleForAreasToPaint.add(currVisStyle);
						setsLabelToPaint.add(prevSetLabelText+setsLabel[i]);
						textLegendColour = currVisStyle.getOutlineColour();
					}
				}				
				if ((this.setsLabel[i]!=null)&&(!this.setsLabel[i].equals(""))){
					if (textLegendColour == null){textLegendColour = colourText;}
					textBBox = VisLabelling.placeLabelAdjacentToArea(g2D, setLegendArea, StringUtils.removeExtraSpaces(StringUtils.removeBreakLines(prevSetLabelText+this.setsLabel[i])), Placement.EAST, Alignment.LEFT, fontName_sets, Font.PLAIN, fontSize_sets, textLegendColour, multiLineSpacing, false); 
				}
				
				legendDimension.width = (int) (textBBox.getBounds().x + textBBox.getWidth());
				if (legendDimension.height < textBBox.getHeight()){
					legendDimension.height = (int) (textBBox.getHeight());
				}
				
				currRectTopLeftCoordForLegend.x += (setLegendIconDimension.getWidth() + textBBox.getWidth() + spacingBetweenSetLegend); //(iconWidth + (this.setsLabel[i].length()*10) + 20);
				i++;
				
			}
			
		
			Point2D.Double displaceBy = new Point2D.Double ((getWidth()-legendDimension.width)/2,(getHeight()-legendDimension.height-50));
			AffineTransform affinetransform = new AffineTransform();
			int areasToPaintCount = areasToPaint.size();
			Rectangle areaBounds;
			for (int a=0; a<areasToPaintCount; a++){
				areaBounds = areasToPaint.get(a).getBounds();
				affinetransform = new AffineTransform();
				affinetransform.translate(displaceBy.x, displaceBy.y);
				areasToPaint.get(a).transform(affinetransform);
				visStyleForAreasToPaint.get(a).paintArea(g2D, areasToPaint.get(a), showBorderOfFilledCurves, false, true);
				
				textLegendColour = visStyleForAreasToPaint.get(a).getOutlineColour();
				if ((setsLabelToPaint.get(a)!=null)&&(!setsLabelToPaint.get(a).equals(""))){
					if (textLegendColour == null){textLegendColour = colourText;}
					textBBox = VisLabelling.placeLabelAdjacentToArea(g2D, areasToPaint.get(a), StringUtils.removeExtraSpaces(StringUtils.removeBreakLines(setsLabelToPaint.get(a))), Placement.EAST, Alignment.LEFT, fontName_sets, Font.PLAIN, fontSize_sets, textLegendColour, multiLineSpacing); // colourText, multiLineSpacing);
				}
			}
		}
		
		
		
		// Draw region labels 
		if (this.showRegionsLabel){
			int i=0;
			textLegendColour = null;
			currRectTopLeftCoordForLegend = new Point2D.Double (0,0);
			legendDimension = new Dimension(0,0);
			
			ArrayList<Area> areasToPaint = new ArrayList<Area>();
			ArrayList<HeterogeneousChannelStyle> visStyleForAreasToPaint = new ArrayList<HeterogeneousChannelStyle>();
			ArrayList<String> setsLabelToPaint = new ArrayList<String>();
			
			for (String iconSetRefLabel: IconDisplayForNeglectBaseRate.iconSetsRefLabel){
					
				if ((iconSetRefLabel!=null) && (iconSetRefLabel.length()>0)){
					for (char iconSetRefLabelChar : iconSetRefLabel.toCharArray()){ 
						currVisStyle = (HeterogeneousChannelStyle)VisStyle.findVisStyleWithRef(setsVisStyle, Character.toString(iconSetRefLabelChar));
						if (!showBorderOfFilledCurves && currVisStyle.canBeDrawnWithoutABorder()){  //if the border is not drawn and the style always you not to draw the border as in the Solid Fill style, then increase the width and height of the icon by the outline thickness multiplied by 2 to ensure that the size of this legend icon is the same as those which have to be drawn with an outline (like the styles which are not filled in)
							setLegendArea = IconSet.IconShape.ELLIPSE.getIconArea(currRectTopLeftCoordForLegend, new DimensionDouble(setLegendIconDimension.width+(currVisStyle.getOutlineThickness()*2-1), setLegendIconDimension.height+(currVisStyle.getOutlineThickness()*2-1)));
						} else {
							setLegendArea = IconSet.IconShape.ELLIPSE.getIconArea(currRectTopLeftCoordForLegend, setLegendIconDimension);
						}
						if (currVisStyle == null){continue;}
						areasToPaint.add(setLegendArea);
						visStyleForAreasToPaint.add(currVisStyle);
						setsLabelToPaint.add(regionsLabel[i]);
						textLegendColour = currVisStyle.getOutlineColour();
					}
				}				
				if ((this.regionsLabel[i]!=null)&&(!this.regionsLabel[i].equals(""))){
					if (textLegendColour == null){textLegendColour = colourText;}
					textBBox = VisLabelling.placeLabelAdjacentToArea(g2D,setLegendArea, this.regionsLabel[i], Placement.EAST, Alignment.LEFT, fontName_sets, Font.PLAIN, fontSize_sets, textLegendColour, multiLineSpacing, false); // colourText, multiLineSpacing);
				}

				legendDimension.width = (int) (textBBox.getBounds().x + textBBox.getWidth());
				if (legendDimension.height < textBBox.getHeight()){
					legendDimension.height = (int) (textBBox.getHeight());
				}
				
				currRectTopLeftCoordForLegend.x += (setLegendIconDimension.getWidth() + textBBox.getWidth() + spacingBetweenSetLegend); //(iconWidth + (this.setsLabel[i].length()*10) + 20);
				
				i++;
			
			}
			

			Point2D.Double displaceBy = new Point2D.Double ((getWidth()-legendDimension.width)/2,(getHeight()-legendDimension.height-20));
			AffineTransform affinetransform = new AffineTransform();
			int areasToPaintCount = areasToPaint.size();
			Rectangle areaBounds;
			for (int a=0; a<areasToPaintCount; a++){
				areaBounds = areasToPaint.get(a).getBounds();
				affinetransform = new AffineTransform();
				affinetransform.translate(displaceBy.x, displaceBy.y);
				areasToPaint.get(a).transform(affinetransform);
				visStyleForAreasToPaint.get(a).paintArea(g2D, areasToPaint.get(a), showBorderOfFilledCurves, false, true);
				
				textLegendColour = visStyleForAreasToPaint.get(a).getOutlineColour();
				if ((setsLabelToPaint.get(a)!=null)&&(!setsLabelToPaint.get(a).equals(""))){
					if (textLegendColour == null){textLegendColour = colourText;}
					textBBox = VisLabelling.placeLabelAdjacentToArea(g2D, areasToPaint.get(a), StringUtils.removeExtraSpaces(StringUtils.removeBreakLines(setsLabelToPaint.get(a))), Placement.EAST, Alignment.LEFT, fontName_sets, Font.PLAIN, fontSize_sets, textLegendColour, multiLineSpacing); // colourText, multiLineSpacing);
				}
			}
		}

		
	}

	
	// Methods
	public void refresh(){
		paintImmediately(getBoundingBox());
	}


	@Override
	public boolean isEmpty() {
		return (this.iconDisplay==null);
	}
}
