/* 
 * 
 * eulerGlyphs v0.9.0
 * 
 * 2013-12-18
 *
 * 
 * 
 * eulerGlyphs -- Drawing Area-Proportional Euler Diagrams With Glyphs	    
 * 		http://www.eulerdiagrams.org/eulerGlyphs
 * 
 * 
 * 		Copyright (C) 2012-2013, Luana Micallef. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerGlyphs.
 * 			
 * 		eulerGlyphs is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerGlyphs is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerGlyphs (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */

 


package uicomponent.vispanel;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.awt.geom.Area;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import uicomponent.vispanel.VisUtils.Alignment;
import uicomponent.vispanel.VisUtils.Placement;
import utils.GeometryUtils;
import utils.StringUtils;


/**
 * 
 * To display and position labels appropriately on a visualization.
 *
 */



public class VisLabelling {
	
	
	public static Rectangle2D.Double placeLabelAdjacentToArea (Graphics2D g2D, Area area, String label, Placement labelPlacement, Alignment alignText, String fontName, int fontStyle, int fontSize, Color textColour, int multiLineSpacing) {
		return placeLabelAdjacentToArea (g2D, area, label, labelPlacement, alignText, fontName, fontStyle, fontSize, textColour, multiLineSpacing, true); 
	}	
	
	public static Rectangle2D.Double placeLabelAdjacentToArea (Graphics2D g2D, Area area, String label, Placement labelPlacement, Alignment alignText, String fontName, int fontStyle, int fontSize, Color textColour, int multiLineSpacing, boolean draw) {
		// returns a rectangle bounding the entire label 
		
		int spaceBetweenLabelAndSet = 10;
		
		Rectangle setBBox = area.getBounds();
		
		Font font = new Font(fontName,fontStyle, fontSize);
		FontRenderContext frc = g2D.getFontRenderContext();
		
		String[] labelLines = StringUtils.splitStringByNewLine(label);
		
		TextLayout[] labelLinesLayout = new TextLayout[labelLines.length];
		Point2D.Double[] labelLinesTopLeftCoord = new Point2D.Double[labelLines.length];
		Rectangle2D[] labelLinesTextBBox = new Rectangle2D[labelLines.length];
		
		double labelWidth = 0;
		double labelHeight = 0;
		
		for (int i=0; i < labelLines.length; i++){ 
			labelLinesLayout[i] = new TextLayout(labelLines[i], font, frc);
			labelLinesTextBBox[i] = labelLinesLayout[i].getBounds();
			
			labelWidth = (labelWidth < labelLinesTextBBox[i].getWidth()) ? labelLinesTextBBox[i].getWidth() : labelWidth;
			labelHeight += labelLinesTextBBox[i].getHeight() + ((i < (labelLines.length-1)) ? multiLineSpacing : 0);
		}
	
		Point2D.Double labelTopLeftCoord = new Point2D.Double(0,0); 		
		switch (labelPlacement){
			case WEST:{
				labelTopLeftCoord.x = (int)(setBBox.x - labelWidth - spaceBetweenLabelAndSet);  
				labelTopLeftCoord.y = (int)(setBBox.y + (setBBox.getHeight()/2)) - (labelHeight/2); 				
				break; 
			}
			case EAST:{
				labelTopLeftCoord.x = (int)(setBBox.x + setBBox.getWidth() + spaceBetweenLabelAndSet); 
				labelTopLeftCoord.y = (int)(setBBox.y + (setBBox.getHeight()/2) - (labelHeight/2)); 
				break;
			}
			case NORTH:{
				labelTopLeftCoord.x = (int)(setBBox.x + (setBBox.getWidth()/2) - (labelWidth/2));
				labelTopLeftCoord.y = setBBox.y - spaceBetweenLabelAndSet - labelHeight;
				break;
			}
			case SOUTH:{
				labelTopLeftCoord.x = (int)(setBBox.x + (setBBox.getWidth()/2) - (labelWidth/2));  
				labelTopLeftCoord.y = (int)(setBBox.y + setBBox.getHeight() + spaceBetweenLabelAndSet); 
				break;
			}
		}
		
		g2D.setFont(font);
		g2D.setColor(textColour);
		for (int i=0; i<labelLines.length; i++){
			labelLinesTopLeftCoord[i] = new Point2D.Double(0,0); 
			switch (alignText){
				case LEFT:{
					labelLinesTopLeftCoord[i].x = labelTopLeftCoord.x;
					break;
				}
				case RIGHT:{
					labelLinesTopLeftCoord[i].x = labelTopLeftCoord.x + (labelWidth-labelLinesTextBBox[i].getWidth()); 
					break;
				}
				case CENTRE:{
					labelLinesTopLeftCoord[i].x = labelTopLeftCoord.x + ((labelWidth-labelLinesTextBBox[i].getWidth())/2); 
					break;
				}
			}
			if (i==0){
				labelLinesTopLeftCoord[i].y = labelTopLeftCoord.y + labelLinesTextBBox[i].getHeight();
			} else {
				labelLinesTopLeftCoord[i].y = labelLinesTopLeftCoord[i-1].y + (((labelLines.length>1)&&(i>0)) ? multiLineSpacing : 0) + labelLinesTextBBox[i].getHeight();
			}
			if (draw){
				labelLinesLayout[i].draw(g2D, (float)labelLinesTopLeftCoord[i].x, (float)labelLinesTopLeftCoord[i].y);
			}
		}
		
		return new Rectangle2D.Double(labelLinesTopLeftCoord[0].x, labelLinesTopLeftCoord[0].y, labelWidth, labelHeight);
		
	}

	public static void placeLabelAtCentreOfRegionArea (Graphics2D g2D, Area regionArea, String label, String fontName, int fontStyle, int fontSize, Color textColour) {
		
		if(label.length() == 0) {
			return;
		}
			
		int x = 0;
		int y = 0;
		Point centre = GeometryUtils.getCentreOfMaxRectInArea(regionArea);
		if(centre != null) {
			x = centre.x;
			y = centre.y;
		} else {
			return;
		}
		

		// display the labels
		
		Font font = new Font(fontName,fontStyle, fontSize);
		FontRenderContext frc = g2D.getFontRenderContext();	
		TextLayout labelLayout = new TextLayout(label, font, frc);
	
		Rectangle2D textBBox = labelLayout.getBounds();
				
		x = x - (int)textBBox.getWidth()/2;
		y = y + (int)textBBox.getHeight()/2;
			
		g2D.setColor(textColour);
		g2D.setFont(font);
		labelLayout.draw(g2D,x,y); 	
	}
	
}
	