/* 
 * 
 * eulerGlyphs v0.9.0
 * 
 * 2013-12-18
 *
 * 
 * 
 * eulerGlyphs -- Drawing Area-Proportional Euler Diagrams With Glyphs	    
 * 		http://www.eulerdiagrams.org/eulerGlyphs
 * 
 * 
 * 		Copyright (C) 2012-2013, Luana Micallef. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerGlyphs.
 * 			
 * 		eulerGlyphs is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerGlyphs is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerGlyphs (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */

 


package utils;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Random;


/**
 * 
 * Mathematical Operations and Utilities
 *
 */



public class MathUtils {
		
	public static boolean isEven(double value){
		return (!isDecimal(value/2));
	}
	public static boolean isOdd(double value){
		return (isDecimal(value/2));
	}
	
	public static boolean isMultipleOf (double value, double multiplier){
		return ((value%multiplier)==0);
	}
	
	public static boolean isDecimal (double value){
		return (Math.floor(value) != value); 
		// Math.floor returns the whole number part as a double
	}

	public static int roundup(double value){
		// instead of rounding up to the closest that is 24.3->24 and 24.5->25
		// this rounds up to the next integer that is 24.0-> 24, 24.1->25, 24.5->24
		return (int)(Math.floor(value) + (MathUtils.isDecimal(value) ? 1: 0));
	}
	
	public static int rounddown(double value){
		// rounds down so 25.4 -> 25 and similarly 25.5 -> 25 and 25.7 -> 25
		return (int)(Math.floor(value));
	}
	
	public static double roundToDps (double value, int digitsBeforeDP, int digitsAfterDP) {
		String formatAsStr = "";
		for (int i=0; i < digitsBeforeDP; i++){ formatAsStr += "#";}
		formatAsStr = ".";
		for (int i=0; i < digitsAfterDP; i++){  formatAsStr += "#";}
		
    	DecimalFormat dpForm = new DecimalFormat(formatAsStr);
	    return Double.valueOf(dpForm.format(value));
	}
	
	
	
	public static int noOfTimesANumberFactorizesAnother (double value, int factor){
		int factorCount = 0;
		while (value > 1){
			if ((value % factor) != 0){
				return factorCount;
			}
			value = value / factor;  //just checked that remainder is 0 => value must be an integer after division
			factorCount++;
		}
		return factorCount;
	}
	
	
	public static Integer factorial(int value) {
		// iterative version 
		
		if (value < 0){ return null;}
	
		if ((value == 0) || (value == 1)){
			return 1;
		} else {
			int result = value;
			for (int i = (value-1); i > 1; i--){
				result *= i;
			}
			return result;
		}
	}
	
	public static int noOfCombinations (int n, int r){
		// nCr = n!/(((n-r)!r!))
		// different ways how to choose r out of the n 		
		return (factorial(n) / (factorial(n-r)*factorial(r)));
	}
	
	
	 public static double getMinInArray(double[] numArray){  
		 //even though double, even int should be handled
		 //cannot have a generic type or else would have to do explicit checks on type to ensure that it is a numeric type
		 
		 double min = numArray[0];
		 for (int i =1; i < numArray.length; i++){
			 if (numArray[i] < min){
				 min = numArray[i];
			 }
		 }
		 return min;
	 }
	 
	 public static int getMutliplyFactorToConvertDecimalToWholeNumber(double value){
		 int noOfDpMovesToRight = 0;
		 
		 String valueAsString = Double.toString(value);
		 int dpPosIndex = valueAsString.indexOf('.');
		 
		 if (dpPosIndex > -1){
			 int noOfZerosAfterDP = 0;
			 int i = dpPosIndex+1; 
			 while ( (valueAsString.charAt(i)=='0') && (i < valueAsString.length())){
				 noOfZerosAfterDP++;
				 i++;
			 }
			 noOfDpMovesToRight = noOfZerosAfterDP + 1;
	     }
		 return (int)Math.pow(10, noOfDpMovesToRight);
	 }
	 
	 public static int getNoOfSignificantFiguresBeforeDecimalPoint(double value){
		 
		 String valueAsString = Double.toString(value);
		 int dpPosIndex = valueAsString.indexOf('.');
		 if (dpPosIndex < 0){
			 return valueAsString.length();
		 } else {
			 String sfBeforeDP = valueAsString.substring(0, dpPosIndex);
			 return sfBeforeDP.length();
		 }
	 }
	 
	 
	 public static Object[] reverseElements (Object[] elems){
		 Object[] reversedElems = new Object[elems.length];
		 for (int i = (elems.length - 1); i >=0 ; i--){
			 reversedElems[(elems.length-1)-i] = elems;
		 }
		 return reversedElems;
	 }
	 
	 public static ArrayList<Object> reverseElements (ArrayList<Object> elems){
		 Object[] reversedElems = new Object[elems.size()];
		 for (int i = (elems.size() - 1); i >=0 ; i--){
			 reversedElems[(elems.size()-1)-i] = elems;
		 }
		 return new ArrayList<Object>(Arrays.asList(reverseElements(elems.toArray())));
	 }
	 
	 
	public static Double randomNumberInRange (double range_first, double range_last){
		
		// check the given range
	    if (range_first > range_last) {
		      //System.out.println("Utilities.randomNoInRange:  inappropriate range - the first number in the range is greater than the last one");
		      return null;
	    }
	    
	    Random random = new Random();
	    
	    double range = range_last - range_first + 1;
	    
	    // compute a fraction of the range, 0 <= frac < range
	    double fraction = (range * random.nextDouble());
	    double randomNo =  (fraction + range_first);  
	    
	    return (new Double (randomNo));
	}
	 
}








