/* 
 * 
 * eulerGlyphs v0.9.0
 * 
 * 2013-12-18
 *
 * 
 * 
 * eulerGlyphs -- Drawing Area-Proportional Euler Diagrams With Glyphs	    
 * 		http://www.eulerdiagrams.org/eulerGlyphs
 * 
 * 
 * 		Copyright (C) 2012-2013, Luana Micallef. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerGlyphs.
 * 			
 * 		eulerGlyphs is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerGlyphs is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerGlyphs (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */

 


package utils;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map.Entry;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


/**
 * 
 * String Manipulation Operations and Utilities
 *
 */


public class StringUtils {
	
	/*
	 * Splits up a string into multiple lines 
	 * 
	 * If a string contains '\n' to indicate new line, then the string will be split up into the required lines and an array of these line strings is returned
	 * 
	 * @param string the string to be split up by new line 
	 * @return an array of all of the lines in input string 
	 */
	public static String[] splitStringByNewLine(String string){
		String[] lines = string.split(Pattern.quote("\\\\n") + '+'); 
		
		if (lines.length == 1){
			lines = string.split(Pattern.quote("\\n") + '+');
		}
		if (lines.length == 1){
			lines = string.split(Pattern.quote("\n") + '+'); 
		}
		return lines;
	}
	
	/*
	 * Remove extra white-spaces from a string
	 * 
	 * Returns a version of the input string where all contiguous white-spaces are replaced with a single space. 
	 * Line terminators are treated like whitespace.
	 * 
	 * @param inputStr the string that needs to be filtered out from extra white-spaces
	 * @return a string same as the input string but with all duplicate white-spaces removed
	 */
	public static String removeExtraSpaces(String inputStr) {
	    String patternStr = "\\s+";
	    String replaceStr = " ";
	    Pattern pattern = Pattern.compile(patternStr);
	    Matcher matcher = pattern.matcher(inputStr);
	    return matcher.replaceAll(replaceStr);
	}
	
	/*
	 * Remove break lines (\n) from a string
	 * 
	 * @param inputStr the string that needs to be filtered out from break liness
	 * @return a string same as the input string but without any break lines
	 */
	public static String removeBreakLines(String inputStr) {
		return inputStr.replace("\\\\n", " ");
	}
	
	
	
	/*
	 * Returns the first Object value in a hashmap with an equal String key 
	 * The actual value of the String object (i.e. the characters of the String) are compared and checked rather than the pointer to the String object
	 * 
	 * @param hashmap the hashmap to be searched having String as key and Object as value
	 * @param str the String to the searched in the keys of the hashmap
	 * @return the object value in the hashmap having the required string characters as key 
	 */
	public static Object getHashMapObjectWithEqualStringKeyValue(HashMap<String,Object> hashmap, String str){
		Iterator itr = hashmap.entrySet().iterator();
		Entry<String, Object> entry;
		
		while (itr.hasNext()){
			entry = (Entry<String, Object>) itr.next();
			if (entry.getKey().equals(str)){
				return entry.getValue();
			}
		}
		return null;
	}
	
}
