/* 
 * 
 * eulerGlyphs v0.9.0
 * 
 * 2013-12-18
 *
 * 
 * 
 * eulerGlyphs -- Drawing Area-Proportional Euler Diagrams With Glyphs	    
 * 		http://www.eulerdiagrams.org/eulerGlyphs
 * 
 * 
 * 		Copyright (C) 2012-2013, Luana Micallef. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerGlyphs.
 * 			
 * 		eulerGlyphs is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerGlyphs is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerGlyphs (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */

 


package visforfallacy;


import java.util.ArrayList;

import pictograph.IconSet;
import pictograph.IconSet.IconShape;
import uicomponent.vispanel.EulerEllipsePanel;


/**
 * To instantiate, define and handle a 1-set population Euler diagram with icons (or glyphs) to prevent the neglect base rate fallacy
 * 
 */


public class EulerPop1SetIconsForNeglectBaseRate extends EulerPop1SetForNeglectBaseRate{

	public static final String[] iconSetsTitle = new String[]{"+vePop and -veTest","+vePop and +veTest","-vePop and +veTest","-vePop and -veTest"};
	public static final String[] iconSetsRefLabel = new String[]{"ab","abc","ac", "a"};  // these must be coherent with above setsTitle etc 
	
	// Data fields
	protected boolean showAreaProportionalEulerDiagram = true;
	protected boolean centreIconsInRectRegion = true;
	protected boolean scaleIconsInRegions = false;
	
	
	// Constructor
	public EulerPop1SetIconsForNeglectBaseRate(EulerEllipsePanel displayPanel, boolean showAreaProportionalEulerDiagram){
		super(displayPanel); 
		setShowAreaProportionalEulerDiagram(showAreaProportionalEulerDiagram);
	}
	public EulerPop1SetIconsForNeglectBaseRate(double baseRate, double testSensitivity, double testFalsePositive, EulerEllipsePanel displayPanel, boolean showAreaProportionalEulerDiagram){
		super(baseRate, testSensitivity, testFalsePositive, displayPanel); 
		setShowAreaProportionalEulerDiagram(showAreaProportionalEulerDiagram);
	}
	public EulerPop1SetIconsForNeglectBaseRate(double[] inputProbabs, EulerEllipsePanel displayPanel, boolean showAreaProportionalEulerDiagram){
		super(inputProbabs, displayPanel); 
		setShowAreaProportionalEulerDiagram(showAreaProportionalEulerDiagram);
	}
	
	
	// Getters and Setters
	public boolean getShowAreaProportionalEulerDiagram(){
		return this.showAreaProportionalEulerDiagram;
	}
	public boolean getCentreIconsInRectRegion(){
		return this.centreIconsInRectRegion;
	}
	public boolean getScaleIconsInRegions(){
		return this.scaleIconsInRegions;
	}
	
	public void setShowAreaProportionalEulerDiagram(boolean showAreaProportionalEulerDiagram){
		this.showAreaProportionalEulerDiagram = showAreaProportionalEulerDiagram;
	}
	public void setCentreIconsInRectRegion(boolean centreIconsInRectRegion){
		this.centreIconsInRectRegion = centreIconsInRectRegion;
	}
	public void setScaleIconsInRegions(boolean scaleIconsInRegions){
		this.scaleIconsInRegions = scaleIconsInRegions;
	}
	
	
	
	/**
	 * Generates the visualization for the fallacy problem 
	 * without checking whether all the probabilities that should be known for the problem are available or not 
	 * 
	 * {@link #generate()} carries out this check. {@link #generate()} is actually the one which is publicly accessible.
	 */
	protected void generateSafely(boolean showSetLabels, ShowRegionSizeMode showRegionSizeMode, boolean showLegend, int totalPopFreq) {

		
		// Get all the probabilistic values according to Bayes' rule
		double[] bayesProbabs = computeBayesRuleProbabs();
		
		
		// Obtain the details of the icon diagram 
		
		// ... areas as probabilistic values (i.e. as values between 0 and 1)
		double[] regionsProbab = new double[]{bayesProbabs[TEST_N_GIVEN_POP_Y], bayesProbabs[TEST_Y_GIVEN_POP_Y], bayesProbabs[TEST_Y_GIVEN_POP_N], bayesProbabs[TEST_N_GIVEN_POP_N]};
		double[] setsFreq = translateProbabsToFreqs(bayesProbabs, regionsProbab);		
		
		// ... define an icon display
		IconShape displayIconShape = IconShape.ELLIPSE;
		ArrayList<IconSet> iconSets = new ArrayList<IconSet>(3);
		iconSets.add(new IconSet(iconSetsRefLabel[IconDisplayForNeglectBaseRate.POP_N_TEST_N_ICONSET], iconSetsTitle[IconDisplayForNeglectBaseRate.POP_N_TEST_N_ICONSET], setsFreq[TEST_N_GIVEN_POP_N], displayIconShape));
		iconSets.add(new IconSet(iconSetsRefLabel[IconDisplayForNeglectBaseRate.POP_Y_TEST_N_ICONSET], iconSetsTitle[IconDisplayForNeglectBaseRate.POP_Y_TEST_N_ICONSET], setsFreq[TEST_N_GIVEN_POP_Y], displayIconShape));
		iconSets.add(new IconSet(iconSetsRefLabel[IconDisplayForNeglectBaseRate.POP_N_TEST_Y_ICONSET], iconSetsTitle[IconDisplayForNeglectBaseRate.POP_N_TEST_Y_ICONSET], setsFreq[TEST_Y_GIVEN_POP_N], displayIconShape));
		iconSets.add(new IconSet(iconSetsRefLabel[IconDisplayForNeglectBaseRate.POP_Y_TEST_Y_ICONSET], iconSetsTitle[IconDisplayForNeglectBaseRate.POP_Y_TEST_Y_ICONSET], setsFreq[TEST_Y_GIVEN_POP_Y], displayIconShape));
		
		getDisplayPanel().setShowIcons(true);
		getDisplayPanel().setRegionsIconSet(iconSets);
		
		super.generateSafely(showSetLabels, showRegionSizeMode, showLegend, totalPopFreq);
	}	
	
}
