/* 
 * 
 * eulerGlyphs v0.9.0
 * 
 * 2013-12-18
 *
 * 
 * 
 * eulerGlyphs -- Drawing Area-Proportional Euler Diagrams With Glyphs	    
 * 		http://www.eulerdiagrams.org/eulerGlyphs
 * 
 * 
 * 		Copyright (C) 2012-2013, Luana Micallef. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerGlyphs.
 * 			
 * 		eulerGlyphs is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerGlyphs is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerGlyphs (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */

 


package visforfallacy;

import java.awt.Toolkit;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import javax.swing.JOptionPane;

import elliptic.areaproptool.Ellipse;
import elliptic.areaproptool.EllipseDiagram;
import eulerellipse.EulerEllipseVisUtils;

import pictograph.IconDisplay;
import uicomponent.vispanel.EulerEllipsePanel;
import utils.GeometryUtils;
import utils.TypingUtils;
import visstyle.HeterogeneousChannelStyle;


/**
 * To instantiate, define and handle a 2-set population Euler diagram with icons (or glyphs) to prevent the neglect base rate fallacy
 * 
 */


public class EulerPop2SetsIconsForNeglectBaseRate extends EulerPop2SetsForNeglectBaseRate{
	
	
	public static enum IconsPositioning {
		RANDOM {public String toString() {return "random";} },
		UNIFORM {public String toString() {return "uniform";} };
		public abstract String toString();
	}
	
	// Static class fields
	public static final Point2D.Double INNER_ICON_BOX_PADDING_DEFAULT = new Point2D.Double(1,1);
	public static final Point2D.Double ICON_SPACING_DEFAULT = new Point2D.Double(2,2);
	
	// Data fields
	protected boolean showAreaProportionalEulerDiagram = true;
	protected boolean centreIconsInRectRegion = true;
	protected boolean scaleIconsInRegions = false;
	protected Point2D.Double innerIconBoxPadding = INNER_ICON_BOX_PADDING_DEFAULT;
	protected Point2D.Double iconSpacing = ICON_SPACING_DEFAULT;
	protected IconsPositioning iconsPositioning;

	
	// Constructor
	public EulerPop2SetsIconsForNeglectBaseRate(EulerEllipsePanel displayPanel, boolean showAreaProportionalEulerDiagram){
		super(displayPanel); 
		setShowAreaProportionalEulerDiagram(showAreaProportionalEulerDiagram);
	}
	public EulerPop2SetsIconsForNeglectBaseRate(double baseRate, double testSensitivity, double testFalsePositive, EulerEllipsePanel displayPanel, boolean showAreaProportionalEulerDiagram){
		super(baseRate, testSensitivity, testFalsePositive, displayPanel); 
		setShowAreaProportionalEulerDiagram(showAreaProportionalEulerDiagram);
	}
	public EulerPop2SetsIconsForNeglectBaseRate(double[] inputProbabs, EulerEllipsePanel displayPanel, boolean showAreaProportionalEulerDiagram){
		super(inputProbabs, displayPanel); 
		setShowAreaProportionalEulerDiagram(showAreaProportionalEulerDiagram);
	}
	
	
	// Getters
	public boolean getShowAreaProportionalEulerDiagram(){
		return this.showAreaProportionalEulerDiagram;
	}
	public boolean getCentreIconsInRectRegion(){
		return this.centreIconsInRectRegion;
	}
	public boolean getScaleIconsInRegions(){
		return this.scaleIconsInRegions;
	}
	public Point2D.Double getInnerIconBoxPadding(){
		return this.innerIconBoxPadding;
	}
	public Point2D.Double getIconSpacing(){
		return this.iconSpacing;
	}
	public IconsPositioning getIconsPositioning(){
		return this.iconsPositioning;
	}
	
	// Setters
	public void setShowAreaProportionalEulerDiagram(boolean showAreaProportionalEulerDiagram){
		this.showAreaProportionalEulerDiagram = showAreaProportionalEulerDiagram;
	}
	public void setCentreIconsInRectRegion(boolean centreIconsInRectRegion){
		this.centreIconsInRectRegion = centreIconsInRectRegion;
	}
	public void setScaleIconsInRegions(boolean scaleIconsInRegions){
		this.scaleIconsInRegions = scaleIconsInRegions;
	}
	public void setInnerIconBoxPadding(Point2D.Double innerIconBoxPadding){
		this.innerIconBoxPadding = innerIconBoxPadding;
	}
	public void setIconSpacing(Point2D.Double iconSpacing){
		this.iconSpacing = iconSpacing;
	}
	public void setIconsPositioning(IconsPositioning iconsPositioning){
		this.iconsPositioning = iconsPositioning;
	}

	
	
	/**
	 * Generates the visualization for the fallacy problem 
	 * without checking whether all the probabilities that should be known for the problem are available or not 
	 * 
	 * {@link #generate()} carries out this check. {@link #generate()} is actually the one which is publicly accessible.
	 */
	protected void generateSafely(boolean showSetLabels, ShowRegionSizeMode showRegionSizeMode, boolean showLegend, int totalPopFreq) {

		// Compute the actual details/settings to visualize the icons sets in the Euler diagram
		initBasicValues();
		needLargerPopSizeToDisplayIcons = needLargerPopSizeToDisplayIcons();
		if (needLargerPopSizeToDisplayIcons){
			getDisplayPanel().setShowIcons(true);
			getDisplayPanel().setPopSize(popSize);
			getDisplayPanel().setNeedLargerPopSizeToDisplayIcons(needLargerPopSizeToDisplayIcons);
			Toolkit.getDefaultToolkit().beep();
			JOptionPane.showMessageDialog(getDisplayPanel(), "A larger population size is required for glyphs to be added to the Euler diagram.\nPlease enter a value that is greater than "+popSize+".", "Generating an Euler Diagram with Glyphs", JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		// Get exact area-proportional Euler diagram to represent the probabilities
		EllipseDiagram areapropEulerDiagramForInputs = super.computeAreaProportionalEulerDiagram(bayesProbabs);
		EulerEllipseVisUtils.scaleToFit(areapropEulerDiagramForInputs, getDisplayPanel().getBoundingBox(), getDisplayPanel().getInnerPadding(), getDisplayPanel().getCentreOfPanel()); 
		
		iconDimension = computeIconDimensionToFitInAPED (areapropEulerDiagramForInputs, getDisplayPanel());

		getDisplayPanel().setIconDimension(iconDimension);
		getDisplayPanel().setShowIcons(true);
		getDisplayPanel().setPopSize(popSize);
		getDisplayPanel().setNeedLargerPopSizeToDisplayIcons(needLargerPopSizeToDisplayIcons);
		getDisplayPanel().setIconsPositioning(iconsPositioning);
		getDisplayPanel().setRegionsIconSet(regionsIconSet);  
		getDisplayPanel().setAreaPropEulerDiag(areapropEulerDiagramForInputs);
		
		switch (iconsPositioning){
			case RANDOM:{
				super.generateSafely(showSetLabels, showRegionSizeMode, showLegend, totalPopFreq);
				break;
			}
		
			case UNIFORM:{
				
				double[] ellipsesAreaFreq = computeEllipsesAreaAsFreqs(bayesProbabs, popSize);
				double[] ellipsesAreaBasedOnIconDim = new double[3];
				double spacing = iconDimension.getWidth()/IconDisplay.ICONSIDE_T0_SPACING;
				double iconArea = (iconDimension.getWidth()+spacing) * (iconDimension.getHeight()+spacing);
				for (int i=0; i<ellipsesAreaBasedOnIconDim.length; i++){
					ellipsesAreaBasedOnIconDim[i] = ellipsesAreaFreq[i] * iconArea;
				}
				
				Rectangle2D.Double[][] rectsInRegionsOfAPdiag = getLargestContainedRectsForEulerRegions(areapropEulerDiagramForInputs);
				double[] totalAreaOfRectsInEllipses = new double[3];
				totalAreaOfRectsInEllipses[0] = GeometryUtils.computeAreaOfRect(rectsInRegionsOfAPdiag[0][0]) + GeometryUtils.computeAreaOfRect(rectsInRegionsOfAPdiag[1][0]);
				totalAreaOfRectsInEllipses[1] = GeometryUtils.computeAreaOfRect(rectsInRegionsOfAPdiag[2][0]) + GeometryUtils.computeAreaOfRect(rectsInRegionsOfAPdiag[3][0]);
				totalAreaOfRectsInEllipses[2] = GeometryUtils.computeAreaOfRect(rectsInRegionsOfAPdiag[1][0]) + GeometryUtils.computeAreaOfRect(rectsInRegionsOfAPdiag[2][0]);
				
				
				double[] ellipsesScale = new double[3];
				for (int i=0; i<ellipsesScale.length; i++){
					ellipsesScale[i] = ellipsesAreaBasedOnIconDim[i] / totalAreaOfRectsInEllipses[i];
				}
				


				// scale the ellipses and move the ellipses accordingly
				
				EllipseDiagram notAreapropEulerDiagramForInputs = areapropEulerDiagramForInputs.clone();
				
				ArrayList<Ellipse> ellipses = notAreapropEulerDiagramForInputs.getEllipses();
				Ellipse ellipse;
				double scaleAxis =0;
				double prevAaxis;
				double newAaxis;
				
			   
				ellipse = ellipses.get(0);
				scaleAxis = Math.sqrt(ellipsesScale[0]);
				prevAaxis = ellipse.getA();
				ellipse.setA(ellipse.getA()*scaleAxis);
				ellipse.setB(ellipse.getB()*scaleAxis);
				newAaxis = ellipse.getA();
				ellipse.setXc( (ellipse.getXc() - ((newAaxis-prevAaxis)/2)) );
				ellipse = ellipses.get(1);
				ellipse.setXc( (ellipse.getXc() + ((newAaxis-prevAaxis)/2)) );
				ellipse = ellipses.get(2);
				ellipse.setA(ellipse.getA()+((newAaxis-prevAaxis)));
				
				
				
				// scale -ve pop set -> and modify the rest accordingly
				ellipsesScale[1] = (ellipsesAreaBasedOnIconDim[1]*1.02)/ellipses.get(1).getArea(); 
				ellipse = ellipses.get(1);
				scaleAxis = Math.sqrt(ellipsesScale[1]);
				prevAaxis = ellipse.getA();
				ellipse.setA(ellipse.getA()*scaleAxis);
				ellipse.setB(ellipse.getB()*scaleAxis);
				newAaxis = ellipse.getA();
				ellipse.setXc( (ellipse.getXc() + ((newAaxis-prevAaxis)/2)) );
				ellipse = ellipses.get(0);
				ellipse.setXc( (ellipse.getXc() - ((newAaxis-prevAaxis)/2)) );
				ellipse = ellipses.get(2);
				ellipse.setA(ellipse.getA()+((newAaxis-prevAaxis)/2));
			
				double[] currentAreaOfRegions = new double[4];
				
				
				ArrayList<Point2D.Double> intpnts = ellipses.get(0).getIntPnts_WithEllipse(ellipses.get(2));
				currentAreaOfRegions[1] = ellipses.get(0).getSharedZoneArea_ByInteg(ellipses.get(2), new Point2D.Double[]{intpnts.get(0), intpnts.get(1)} , 0);
				currentAreaOfRegions[0] = ellipses.get(0).getArea() - currentAreaOfRegions[1];
				
				intpnts = ellipses.get(1).getIntPnts_WithEllipse(ellipses.get(2));
				currentAreaOfRegions[2] = ellipses.get(1).getSharedZoneArea_ByInteg(ellipses.get(2), new Point2D.Double[]{intpnts.get(0), intpnts.get(1)} , 0);
				currentAreaOfRegions[3] = ellipses.get(1).getArea() - currentAreaOfRegions[2];
				
				double[] tempArea = new double[2];
				tempArea[0] = ((iconArea + (spacing*spacing)) * regionsIconSet.get(0).getFrequency()) / currentAreaOfRegions[0]; 
				tempArea[1] = ((iconArea + (spacing*spacing)) * regionsIconSet.get(1).getFrequency()) / currentAreaOfRegions[1];
				ellipsesScale[0] = Math.max (tempArea[0],tempArea[0]);
				ellipsesScale[1] = Math.max( ((iconArea + (spacing*spacing)) * regionsIconSet.get(2).getFrequency()) / currentAreaOfRegions[2], 
	                                         ((iconArea + (spacing*spacing)) * regionsIconSet.get(3).getFrequency()) / currentAreaOfRegions[3]);
				ellipsesScale[2] = Math.max( ((iconArea + (spacing*spacing)) * regionsIconSet.get(1).getFrequency()) / currentAreaOfRegions[1], 
	                                         ((iconArea + (spacing*spacing)) * regionsIconSet.get(2).getFrequency()) / currentAreaOfRegions[2]);
				
				double maxScaling = Math.max(ellipsesScale[0], Math.max(ellipsesScale[1],ellipsesScale[2]));
				
				
				
				// scale +ve test set -> and modify the rest accordingly
				ellipse = ellipses.get(2);
				scaleAxis = (Math.sqrt(ellipsesScale[2]));
				prevAaxis = ellipse.getA();
				ellipse.setA(ellipse.getA()*1.05*scaleAxis);
				ellipse.setB(ellipse.getB()*scaleAxis);
				newAaxis = ellipse.getA();
				ellipse = ellipses.get(1);
				ellipse.setXc( (ellipse.getXc() + ((newAaxis-prevAaxis)/2)) );
				ellipse = ellipses.get(0);
				ellipse.setXc( (ellipse.getXc() - ((newAaxis-prevAaxis)/2)) );
				double enlargeEll0And1 = (Math.abs(ellipses.get(1).getXc() - ellipses.get(0).getXc()) - ellipses.get(1).getA() - ellipses.get(0).getA())/2;
				ellipse = ellipses.get(0);
				ellipse.setA(ellipse.getA()+enlargeEll0And1);
				ellipse = ellipses.get(1);
				ellipse.setA(ellipse.getA()+enlargeEll0And1);

				
				
				intpnts = ellipses.get(0).getIntPnts_WithEllipse(ellipses.get(2));
				currentAreaOfRegions[1] = ellipses.get(0).getSharedZoneArea_ByInteg(ellipses.get(2), new Point2D.Double[]{intpnts.get(0), intpnts.get(1)} , 0);
				currentAreaOfRegions[0] = ellipses.get(0).getArea() - currentAreaOfRegions[1];
				
				intpnts = ellipses.get(1).getIntPnts_WithEllipse(ellipses.get(2));
				currentAreaOfRegions[2] = ellipses.get(1).getSharedZoneArea_ByInteg(ellipses.get(2), new Point2D.Double[]{intpnts.get(0), intpnts.get(1)} , 0);
				currentAreaOfRegions[3] = ellipses.get(1).getArea() - currentAreaOfRegions[2];
				
			    tempArea = new double[2];
				tempArea[0] = ((iconArea + (spacing*spacing)) * regionsIconSet.get(0).getFrequency()) / currentAreaOfRegions[0]; 
				tempArea[1] = ((iconArea + (spacing*spacing)) * regionsIconSet.get(1).getFrequency()) / currentAreaOfRegions[1];
				ellipsesScale[0] = Math.max (tempArea[0],tempArea[0]);
				ellipsesScale[1] = Math.max( ((iconArea + (spacing*spacing)) * regionsIconSet.get(2).getFrequency()) / currentAreaOfRegions[2], 
	                                         ((iconArea + (spacing*spacing)) * regionsIconSet.get(3).getFrequency()) / currentAreaOfRegions[3]);
				ellipsesScale[2] = Math.max( ((iconArea + (spacing*spacing)) * regionsIconSet.get(1).getFrequency()) / currentAreaOfRegions[1], 
	                                         ((iconArea + (spacing*spacing)) * regionsIconSet.get(2).getFrequency()) / currentAreaOfRegions[2]);
				
				
				if (Math.round(ellipsesScale[1])<1.1){
					ellipse = ellipses.get(1);
					scaleAxis = Math.sqrt(ellipsesScale[1]);
					prevAaxis = ellipse.getA();
					ellipse.setA(ellipse.getA()*0.98*scaleAxis);
					newAaxis = ellipse.getA();
					ellipse.setXc( (ellipse.getXc() + ((newAaxis-prevAaxis))) );
				}

				
				getDisplayPanel().setEllipsesVisStyleEuler(TypingUtils.typecastObjArrayToTArray(visStyle, new HeterogeneousChannelStyle[]{}));
				getDisplayPanel().setRegionsInDiagram(requiredRegionsRef);
				getDisplayPanel().setEllipsesRefLabel(this.getEllipsesRefLabel());
				getDisplayPanel().setShowSetsLabel(showSetLabels);
				getDisplayPanel().setSetsLabel(this.getLabels());
				getDisplayPanel().setSetsLabelPlacement(setsLabelPlacement);
				getDisplayPanel().setSetsLabelTextAlignment(setsLabelTextAlignment);
				boolean showRegionSizeLabels = showRegionSizeMode != ShowRegionSizeMode.DONOT_SHOW;
				getDisplayPanel().setShowRegionsLabel(showRegionSizeLabels);
				if (showRegionSizeLabels){
					getDisplayPanel().setRegionsLabel(composeRegionSizeLabels(bayesProbabs, showRegionSizeMode, totalPopFreq));
				}
				getDisplayPanel().setShowLegend(showLegend);
				
				getDisplayPanel().setEulerEllipseDiagram(notAreapropEulerDiagramForInputs);
				getDisplayPanel().setRectsInRegions(getLargestContainedRectsForEulerRegions(notAreapropEulerDiagramForInputs));
		
				
				getDisplayPanel().refresh();
				break;
			}
			default: break;
		}
		

	}		
}
