/* 
 * 
 * eulerGlyphs v0.9.0
 * 
 * 2013-12-18
 *
 * 
 * 
 * eulerGlyphs -- Drawing Area-Proportional Euler Diagrams With Glyphs	    
 * 		http://www.eulerdiagrams.org/eulerGlyphs
 * 
 * 
 * 		Copyright (C) 2012-2013, Luana Micallef. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerGlyphs.
 * 			
 * 		eulerGlyphs is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerGlyphs is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerGlyphs (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */

 


package visforfallacy;

import java.awt.Toolkit;

import javax.swing.JOptionPane;
import eulerellipse.EulerEllipseVisStyle;

import pictograph.IconDisplay;
import pictograph.IconDisplay.Direction_Icons_IconRows;
import pictograph.IconDisplay.IconsPlacementStyle;

import uicomponent.vispanel.IconDisplayPanel;
import uicomponent.vispanel.VisUtils.*;
import utils.MathUtils;
import utils.ReflectionUtils;
import utils.TypingUtils;
import visstyle.HeterogeneousChannelStyle;
import visstyle.VisStyle;


/**
 * To instantiate, define and handle an icon-based display (or frequency grid) to prevent the neglect base rate fallacy
 * 
 */


public class IconDisplayForNeglectBaseRate extends VisForNeglectBaseRate{
	
	// Static Constants
	public static final int POP_Y_TEST_N_ICONSET = 0;
	public static final int POP_Y_TEST_Y_ICONSET = 1;
	public static final int POP_N_TEST_Y_ICONSET = 2;
	public static final int POP_N_TEST_N_ICONSET = 3;
	
	public static final String[] setsRefLabel = new String[]{"a","b","c"};
	private static final String[] setsTitle = new String[]{"+vePop","-vePop","+veTest"};
	private static final Placement[] setsLabelPlacement = {Placement.WEST, Placement.WEST, Placement.WEST};
	private static final Alignment[] setsLabelTextAlignment = {Alignment.RIGHT, Alignment.RIGHT, Alignment.RIGHT};

	//protected IconShape displayIconShape;
	protected Orientation displayBestRectFitOrientation;
	protected int displayIconsPerRow;
	protected Direction_Icons_IconRows displayDirection_Icons_IconRows;
	protected IconsPlacementStyle displayIconsPlacementStyle;
	protected Integer displayGroupingValue; 
	

	// Constructors
	public IconDisplayForNeglectBaseRate(IconDisplayPanel displayPanel){
		super(); 
		setDisplayPanel(displayPanel);
	}
	public IconDisplayForNeglectBaseRate(double baseRate, double testSensitivity, double testFalsePositive, IconDisplayPanel displayPanel){
		setDisplayPanel(displayPanel);
		setInputProbabs(baseRate, testSensitivity, testFalsePositive);
	}
	public IconDisplayForNeglectBaseRate(double[] inputProbabs, IconDisplayPanel displayPanel){
		setDisplayPanel(displayPanel);
		try {
			setInputProbabs(inputProbabs);
		} catch (Exception e) {
			System.out.println(ReflectionUtils.getClassName(1)+"."+ ReflectionUtils.getMethodName(1)+": "+e);
		}
	}
	
	
	// Setters 
	public void setDisplayBestRectFitOrientation(Orientation displayBestRectFitOrientation){
		this.displayBestRectFitOrientation = displayBestRectFitOrientation;
	}
	public void setDisplayIconsPerRow(int displayIconsPerRow){
		this.displayIconsPerRow = displayIconsPerRow;
	}
	public void setDisplayDirection_Icons_IconRows(Direction_Icons_IconRows displayDirection_Icons_IconRows){
		this.displayDirection_Icons_IconRows = displayDirection_Icons_IconRows;
	}
	public void setDisplayIconsPlacementStyle(IconsPlacementStyle displayIconsPlacementStyle){
		this.displayIconsPlacementStyle = displayIconsPlacementStyle;
	}
	public void setDisplayGroupingValue (Integer displayGroupingValue){
		this.displayGroupingValue = displayGroupingValue;
	}
	
	
	
	// Overridden Getter and Setter 
	public IconDisplayPanel getDisplayPanel (){
		return ((IconDisplayPanel)this.displayPanel);
	}
	public void setDisplayPanel (IconDisplayPanel displayPanel){
		this.displayPanel = displayPanel;
	}
	public String[] getSetsTitle(){
		return setsTitle;
	}
	public Integer getDisplayGroupingValue(){
		return displayGroupingValue;
	}
	
	
	
	// Methods
	
	// ... implementing abstract methods in the parent class
	
	public VisStyle[] getDefaultVisStyle(boolean colourBlindSafe){
		return ((VisStyle[])EulerEllipseVisStyle.heterogeneousChannelStyle_3Ellipses(colourBlindSafe));
	}
	

	/**
	 * Generates the visualization for the fallacy problem 
	 * without checking whether all the probabilities that should be known for the problem are available or not 
	 * 
	 * {@link #generate()} carries out this check. {@link #generate()} is actually the one which is publicly accessible.
	 */
	protected void generateSafely(boolean showSetLabels, ShowRegionSizeMode showRegionSizeMode, boolean showLegend, int totalPopFreq) {

		initBasicValues();
		needLargerPopSizeToDisplayIcons = needLargerPopSizeToDisplayIcons();
		if (needLargerPopSizeToDisplayIcons){
			getDisplayPanel().setPopSize(popSize);
			getDisplayPanel().setNeedLargerPopSizeToDisplayIcons(needLargerPopSizeToDisplayIcons);
			Toolkit.getDefaultToolkit().beep();
			JOptionPane.showMessageDialog(getDisplayPanel(), "A larger population size is required for the frequency grid to be generated.\nPlease enter a value that is greater than "+popSize+".", "Generating a Frequency Grid", JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		// Obtain the details of the icon diagram 
		
		// ... areas as probabilistic values (i.e. as values between 0 and 1)
		double[] regionsProbab = new double[]{bayesProbabs[TEST_N_GIVEN_POP_Y], bayesProbabs[TEST_Y_GIVEN_POP_Y], bayesProbabs[TEST_Y_GIVEN_POP_N], bayesProbabs[TEST_N_GIVEN_POP_N]};

		
		IconDisplay iconDisplay = new IconDisplay(regionsIconSet);
		
		iconDimension = computeIconDimensionToFitInAPED(this.getDisplayPanel());
		iconDisplay.setIconDimension(iconDimension);
		
		iconDisplay.setBestRectFitOrientation(displayBestRectFitOrientation);
		if (displayBestRectFitOrientation == null){
			iconDisplay.setDimensionAsIconCountGivenWidth(displayIconsPerRow);
		}
		iconDisplay.setDirection_Icons_IconRows(displayDirection_Icons_IconRows);
		iconDisplay.setIconsPlacementStyle(displayIconsPlacementStyle);
		
		
		// Display the diagram in the panel
		getDisplayPanel().setIconDimension(iconDimension);
		getDisplayPanel().setPopSize(popSize);
		getDisplayPanel().setIconDisplay(iconDisplay);
		getDisplayPanel().setIconSetsVisStyleEuler(TypingUtils.typecastObjArrayToTArray(visStyle, new HeterogeneousChannelStyle[]{}));
		getDisplayPanel().setShowSetsLabel(showSetLabels);
		getDisplayPanel().setSetsLabel(this.getLabels());
		getDisplayPanel().setSetsLabelPlacement(setsLabelPlacement);
		getDisplayPanel().setSetsLabelTextAlignment(setsLabelTextAlignment);
		boolean showRegionSizeLabels = showRegionSizeMode != ShowRegionSizeMode.DONOT_SHOW; 
		getDisplayPanel().setShowRegionsLabel(showRegionSizeLabels);
		if (showRegionSizeLabels){
			double[] regionsFreq = new double[]{setsFreq[TEST_N_GIVEN_POP_Y], setsFreq[TEST_Y_GIVEN_POP_Y], setsFreq[TEST_Y_GIVEN_POP_N], setsFreq[TEST_N_GIVEN_POP_N]};
			getDisplayPanel().setRegionsLabel(composeRegionSizeLabels(regionsProbab, regionsFreq, showRegionSizeMode, iconDisplay));
		}
		getDisplayPanel().setShowLegend(showLegend);
		
		getDisplayPanel().refresh();
	}
	
	protected String[] composeRegionSizeLabels (double[] setsProbab, double[] setsFreq, ShowRegionSizeMode mode, IconDisplay iconDisplay){
		final int DP = 2;
		final int MAX_SF_PRIOR_DP = 15;
			
		double[] regionSizes = (mode == ShowRegionSizeMode.SHOW_AS_FREQ) ? setsFreq : mode.translateProbabsAsMode(setsProbab,-1);
		double[] regionSizesRnded = new double[regionSizes.length];
		
		int i = 0;
		for (double rs : regionSizes){
			regionSizesRnded[i] = MathUtils.roundToDps(rs, MAX_SF_PRIOR_DP, DP);
			i++;
		}
		return translateValuesToStringForMode(regionSizesRnded, mode);
	}
	
}
