/* 
 * 
 * eulerGlyphs v0.9.0
 * 
 * 2013-12-18
 *
 * 
 * 
 * eulerGlyphs -- Drawing Area-Proportional Euler Diagrams With Glyphs	    
 * 		http://www.eulerdiagrams.org/eulerGlyphs
 * 
 * 
 * 		Copyright (C) 2012-2013, Luana Micallef. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerGlyphs.
 * 			
 * 		eulerGlyphs is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerGlyphs is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerGlyphs (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */

 


package visforfallacy;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import elliptic.areaproptool.EllipseDiagram;
import eulerellipse.EulerEllipseVisUtils;
import pictograph.IconSet;
import pictograph.IconSet.IconShape;
import uicomponent.vispanel.EulerEllipsePanel;
import uicomponent.vispanel.VisPanel;
import utils.DimensionDouble;
import utils.GeometryUtils;
import utils.MathUtils;
import visstyle.VisStyle;


/**
 * An abstract class for the proposed visualizations to avoid the neglect base rate fallacy
 * 
 */


public abstract class VisForNeglectBaseRate extends VisForFallacy{

	
	// Static Final Fields 
	// ...indexes for properties of the problem
	
	// ** ideally the following should be defined as enum but then would have to use Map<K,V> instead of a simple array
	//    which makes things most complex especially when changing the value 
	public static final int BASE_RATE = 0;
	public static final int TEST_SENSITIVITY = 1;
	public static final int TEST_FALSE_POSITIVE = 2; 
	public static final int INPUT_PROBABS_COUNT = 3;
	
	public static final int POP_SET = 0;
	public static final int POP_Y_SET = 1;
	public static final int TEST_Y_SET = 2;
	public static final int POP_N_EXCLUDING_AREA = 3;
	
	public static final int POP_Y = 0;
	public static final int POP_N = 1;
	public static final int TEST_Y_GIVEN_POP_Y = 2;
	public static final int TEST_N_GIVEN_POP_Y = 3; 
	public static final int TEST_Y_GIVEN_POP_N = 4;
	public static final int TEST_N_GIVEN_POP_N = 5;
	public static final int ALL_PROBABS_COUNT = 6;
	
	public static final int SETS_COUNT = 3;
	
	public static final String[] iconSetsTitle = new String[]{"+vePop and -veTest","+vePop and +veTest","-vePop and +veTest","-vePop and -veTest"};
	public static final String[] iconSetsRefLabel = new String[]{"a","ac","bc", "b"};  // these must be coherent with above setsTitle etc 

	
	protected VisStyle[] visStyle;
	protected double[] bayesProbabs;
	protected double[] setsFreq;
	IconShape displayIconShape;
	ArrayList<IconSet> regionsIconSet;
	DimensionDouble iconDimension;
	Integer popSize; //total population of problem
	Boolean needLargerPopSizeToDisplayIcons = false;

	

	protected VisForNeglectBaseRate(){
		this.inputProbabs = new double[INPUT_PROBABS_COUNT];
		this.labels = new String[SETS_COUNT];		
	}
	
	
	// Getters
	public double getBaseRateProbab(){
		return this.inputProbabs[BASE_RATE];
	}
	public double getTestSensitivityProbab(){
		return this.inputProbabs[TEST_SENSITIVITY];
	}
	public double getTestFalsePositiveProbab(){
		return this.inputProbabs[TEST_FALSE_POSITIVE];
	}
	public String getBaseRateLabel(){
		return this.labels[BASE_RATE];
	}
	public String getTestSensitivityLabel(){
		return this.labels[TEST_SENSITIVITY];
	}
	public String getTestFalsePositiveLabel(){
		return this.labels[TEST_FALSE_POSITIVE];
	}
	public VisStyle[] getVisStyle(){
		return this.visStyle;
	}
	
	// Setters
	public void setBaseRateProbab(double baseRateProbab){
		this.inputProbabs[BASE_RATE] = baseRateProbab;
	}
	public void setTestSensitivityProbab(double testSensitivityProbab){
		this.inputProbabs[TEST_SENSITIVITY] = testSensitivityProbab;
	}
	public void setTestFalsePositiveProbab(double testFalsePositiveProbab){
		this.inputProbabs[TEST_FALSE_POSITIVE] = testFalsePositiveProbab;
	}
	public void setInputProbabs(double baseRateProbab, double testSensitivityProbab, double testFalsePositiveProbab){
		try{
			setInputProbabs(new double[]{baseRateProbab, testSensitivityProbab, testFalsePositiveProbab});
		} catch (Exception e){
			// do nothing because an exception will never be thrown since all the required params are provided
		}
	}
	public void setInputProbabs(double[] inputProbabs) throws Exception{
		if (inputProbabs == null){
			throw new Exception ("The probabilities that should be known for the problem are not available");
		} else if (inputProbabs.length != INPUT_PROBABS_COUNT) {
			throw new Exception ("Not all the " + INPUT_PROBABS_COUNT + " probabilities that should be known for the problem are available");
		} else {
			this.inputProbabs = inputProbabs;
		}
	}
	public void setPopSize(Integer popSize){
		this.popSize = popSize;
	}
	
	
	public void initBasicValues(){	
		bayesProbabs = computeBayesRuleProbabs();
		setsFreq = translateProbabsToFreqs(bayesProbabs, popSize); 
		
		regionsIconSet = new ArrayList<IconSet>(4);
		regionsIconSet.add(new IconSet(iconSetsRefLabel[IconDisplayForNeglectBaseRate.POP_Y_TEST_N_ICONSET], iconSetsTitle[IconDisplayForNeglectBaseRate.POP_Y_TEST_N_ICONSET], setsFreq[TEST_N_GIVEN_POP_Y], displayIconShape));
		regionsIconSet.add(new IconSet(iconSetsRefLabel[IconDisplayForNeglectBaseRate.POP_Y_TEST_Y_ICONSET], iconSetsTitle[IconDisplayForNeglectBaseRate.POP_Y_TEST_Y_ICONSET], setsFreq[TEST_Y_GIVEN_POP_Y], displayIconShape));
		regionsIconSet.add(new IconSet(iconSetsRefLabel[IconDisplayForNeglectBaseRate.POP_N_TEST_Y_ICONSET], iconSetsTitle[IconDisplayForNeglectBaseRate.POP_N_TEST_Y_ICONSET], setsFreq[TEST_Y_GIVEN_POP_N], displayIconShape));
		regionsIconSet.add(new IconSet(iconSetsRefLabel[IconDisplayForNeglectBaseRate.POP_N_TEST_N_ICONSET], iconSetsTitle[IconDisplayForNeglectBaseRate.POP_N_TEST_N_ICONSET], setsFreq[TEST_N_GIVEN_POP_N], displayIconShape));
	}
	
	public void setBaseRateLabel(String baseRateLabel){
		this.labels[BASE_RATE] = baseRateLabel;		
	}
	public void setTestSensitivityLabel(String testSensitivityLabel){
		this.labels[TEST_SENSITIVITY] = testSensitivityLabel;	
	}
	public void setTestFalsePositiveLabel(String testFalsePositiveLabel){
		this.labels[TEST_FALSE_POSITIVE] = testFalsePositiveLabel;
		
	}
	public void setLabels(String[] labels) {
		if (labels == null){
		} else if (labels.length != SETS_COUNT) {
		} else {
			this.labels = labels;
		}
	}
	// not sure whether exceptions should be generated for labels
	public void setVisStyle(VisStyle[] visStyle){
		this.visStyle = visStyle;
	}
	public void setDisplayIconShape(IconShape iconShape){
		this.displayIconShape = iconShape;
	}
	
	
	// Abstract
	public abstract VisStyle[] getDefaultVisStyle(boolean colourBlindSafe);
	
	// Abstract getters
	public abstract String[] getSetsTitle();
	
	
	
	// Overloaded Methods
	/**
	 * Generates the visualization for the fallacy problem 
	 * 
	 * @throws Exception An exception is throw when not all or none of the probabilities that should be known for the problem are available
	 */
	public void generate(boolean showSetLabels, boolean showLegend) throws Exception{
		generate (showSetLabels, ShowRegionSizeMode.DONOT_SHOW, showLegend, 0);
	}
	public void generate(boolean showSetLabels, ShowRegionSizeMode showRegionSizeMode, boolean showLegend, int totalPopFreq) throws Exception{
		if (inputProbabs == null){
			throw new Exception ("The probabilities that should be known for the problem are not available");
		} else if (inputProbabs.length != INPUT_PROBABS_COUNT) {
			throw new Exception ("Not all the " + INPUT_PROBABS_COUNT + " probabilities that should be known for the problem are available");
		} else {
			generateSafely(showSetLabels, showRegionSizeMode, showLegend, totalPopFreq);
		}
	}
	
	
	// Implemented Methods
    // ... utilities 
	/**
	 * Apply Bayes' Rule using the known probabilities for the problem and compute the remain probabilistic values 
	 * 
	 * @return an array containing all the probabilistic values for the problem
	 */
	protected double[] computeBayesRuleProbabs (){
		double[] bayesProbabs = new double[]{0.0,0.0,0.0,0.0,0.0,0.0};

		// as probabs2w
		bayesProbabs[POP_Y] = getBaseRateProbab();
		bayesProbabs[POP_N] = 1 - bayesProbabs[POP_Y];
		
		bayesProbabs[TEST_Y_GIVEN_POP_Y] = getTestSensitivityProbab() * bayesProbabs[POP_Y]; 
		bayesProbabs[TEST_N_GIVEN_POP_Y] = (1-getTestSensitivityProbab()) * bayesProbabs[POP_Y];
		
		bayesProbabs[TEST_Y_GIVEN_POP_N] = getTestFalsePositiveProbab() * bayesProbabs[POP_N]; 
		bayesProbabs[TEST_N_GIVEN_POP_N] = (1-getTestFalsePositiveProbab())* bayesProbabs[POP_N];  
	    
		return bayesProbabs;
	}
		
	protected DimensionDouble computeIconDimensionToFitInAPED(VisPanel visPanel){
		EulerEllipsePanel eulerEllipsePanel = new EulerEllipsePanel();
		eulerEllipsePanel.setSize(visPanel.getSize());
		EulerPop2SetsForNeglectBaseRate eulerPop2SetsForNeglectBaseRate = new EulerPop2SetsForNeglectBaseRate(inputProbabs, eulerEllipsePanel);
		EllipseDiagram  areapropEulerDiagramForInputs = eulerPop2SetsForNeglectBaseRate.computeAreaProportionalEulerDiagram(bayesProbabs);
		EulerEllipseVisUtils.scaleToFit(areapropEulerDiagramForInputs, eulerEllipsePanel.getBoundingBox(), eulerEllipsePanel.getInnerPadding(), eulerEllipsePanel.getCentreOfPanel());
		return computeIconDimensionToFitInAPED (areapropEulerDiagramForInputs, eulerEllipsePanel);
	}
	
	protected DimensionDouble computeIconDimensionToFitInAPED(EllipseDiagram areapropEulerDiagramForInputs, VisPanel visPanel){	
		// Find the icon size based on the size of the max rectangle in the smallest region 
			
		Rectangle2D.Double[][] rectsInRegions = EulerPop2SetsForNeglectBaseRate.getLargestContainedRectsForEulerRegions(areapropEulerDiagramForInputs);
		
		Rectangle2D.Double[][] rectsInRegionsScaled = new Rectangle2D.Double[rectsInRegions.length][];
		int regIndex =0;
		for(Rectangle2D.Double[] rectsInRegion : rectsInRegions){
			Rectangle2D.Double[] rectsInRegionScaled = new Rectangle2D.Double[rectsInRegion.length];
			int regionRectIndex = 0;
			for(Rectangle2D.Double rect : rectsInRegion){
				rectsInRegionScaled[regionRectIndex] = new  Rectangle2D.Double(rect.x, rect.y, rect.getWidth(), rect.getHeight());
				regionRectIndex++;
			}
			rectsInRegionsScaled[regIndex] = rectsInRegionScaled;
			regIndex++;
		}
		EulerEllipseVisUtils.scaleToFit(rectsInRegionsScaled, areapropEulerDiagramForInputs, visPanel.getBoundingBox(), visPanel.getInnerPadding(), visPanel.getCentreOfPanel());
		
		int regionWithSmallestRect_index = 0;
		Double areaOfSmallestRectInRegion = null;
		Rectangle2D.Double rectOfSmallestRectInRegion = null;
		double currArea = 0;
		int r =0;
		for (Rectangle2D.Double[] rectsInRegion : rectsInRegions){
			//if (r==0){r++;continue;}
			for(Rectangle2D.Double rect : rectsInRegion){
				currArea = GeometryUtils.computeAreaOfRect(rect);
				if ((areaOfSmallestRectInRegion == null)||(currArea < areaOfSmallestRectInRegion)){
					areaOfSmallestRectInRegion = currArea;
					regionWithSmallestRect_index = r;
					rectOfSmallestRectInRegion = rect;
				}
			}
			r++;
		}
		IconSet iconSetOfSmallestRegion = regionsIconSet.get(regionWithSmallestRect_index);
		double areaPerIcon = areaOfSmallestRectInRegion/iconSetOfSmallestRegion.getFrequency();
		double sidesForSquareIcon = Math.sqrt(areaPerIcon);
		sidesForSquareIcon = MathUtils.rounddown(sidesForSquareIcon);
		return iconDimension = new DimensionDouble(sidesForSquareIcon,sidesForSquareIcon);
	}
	
	protected Boolean needLargerPopSizeToDisplayIcons(){

		for (IconSet iconSet : regionsIconSet){
			if (iconSet.getFrequency() <= 0){
				return true;
			}
		}
		return false; 
	}
}
