/* 
 * 
 * eulerGlyphs v0.9.0
 * 
 * 2013-12-18
 *
 * 
 * 
 * eulerGlyphs -- Drawing Area-Proportional Euler Diagrams With Glyphs	    
 * 		http://www.eulerdiagrams.org/eulerGlyphs
 * 
 * 
 * 		Copyright (C) 2012-2013, Luana Micallef. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerGlyphs.
 * 			
 * 		eulerGlyphs is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerGlyphs is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerGlyphs (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */

 


package visstyle;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Area;


/**
 * To instantiate, define and handle a heterogeneous channel type visualization style
 * 
 */


public class HeterogeneousChannelStyle extends VisStyle{

	// Predefined Colours
	//    Note that Color(float r, float g, float b, float a) -> Creates an sRGB color with the specified red, green, blue, and alpha values in the range (0.0 - 1.0).
	//    => hence the values passed as parameters are not the CMYK values
	public static final Color COLOUR_PINK_DARK = new Color(231, 115, 94, 128); 
	public static final Color COLOUR_PINK_LIGHT = new Color(233, 130, 108, 128); 
	public static final Color COLOUR_GREEN = new Color(0, 147, 128, 128); 
	public static final Color COLOUR_BLUE = new Color(33, 44, 119, 128);  
	public static final Color COLOUR_NULL = new Color(0f, 0f, 0f, 0f); 
	
	// Predefined Colours for Colour-Blind (targeting mainly the most common case that is red-green blindness)
	//    colour scheme obtained from colorbrewer2.org -> colorblind safe, 4 data classes
	public static final Color COLOUR_COLOURBLINDSAFE_BLUESCHEME_1a = new Color(255,255,204,128);  //CMYK=(0,0,20,0), RGB=(255,255,204)
	public static final Color COLOUR_COLOURBLINDSAFE_BLUESCHEME_1b = new Color(161,218,180,128); //CMYK=(37,0,25,0), RGB=(161,218,180)
	public static final Color COLOUR_COLOURBLINDSAFE_BLUESCHEME_1c = new Color(65,182,196,128); //CMYK=(75,0,10,0), RGB=(65,182,196)
	public static final Color COLOUR_COLOURBLINDSAFE_BLUESCHEME_1d = new Color(34,94,168,128); //CMYK=(90,45,0,0), RGB=(34,94,168)

	// Predefined Colours for Colour-Blind (targeting mainly the most common case that is red-green blindness)
	//    -> fill colours are slightly darker; used mainly when the borders are removed 
	public static final Color COLOUR_COLOURBLINDSAFE_RED_DARK = new Color(0.5f, 0f, 0f);  
	public static final Color COLOUR_COLOURBLINDSAFE_RED_LIGHT = new Color(1f, 0.6f, 0.6f); 
	public static final Color COLOUR_COLOURBLINDSAFE_BLUE_DARK = new Color(0.2f, 0.25f, 0.95f); 
	public static final Color COLOUR_COLOURBLINDSAFE_GREEN_DARKDRAB = new Color(0.42f, 0.56f, .14f); 
	public static final Color COLOUR_COLOURBLINDSAFE_BLUE_LIGHT = new Color(0.68f, 0.68f, 1f); 
	public static final Color COLOUR_COLOURBLINDSAFE_BLACK = new Color(0f, 0f, 0f, 1f); 

	
	// Data fields
	protected Color fillColour;
	protected Color outlineColour;
	protected int outlineThickness;

	
	// Constructor
	protected HeterogeneousChannelStyle (String ref, String title, Color fillColour, Color outlineColour, int outlineThickness){
		this.ref = ref;
		this.title = title;
		this.fillColour = fillColour;
		this.outlineColour = outlineColour;
		this.outlineThickness = outlineThickness;
	}
	
	
	// Getters
	public Color getFillColour(){
		return this.fillColour;
	}
	public Color getOutlineColour(){
		return this.outlineColour;
	}
	public int getOutlineThickness(){
		return this.outlineThickness;
	}
	
	// Setters
	public void setFillColour(Color fillColour){
		this.fillColour = fillColour;
	}
	public void setOutlineColour (Color outlineColour){
		this.outlineColour = outlineColour;
	}
	public void setOutlineThickness(int outlineThickness){
		this.outlineThickness = outlineThickness;
	}
	
	
	// Methods
	
	public Boolean canBeDrawnWithoutABorder(){return null;}
	
	public void paintArea(Graphics2D g2D, Area setArea){}
	public void paintArea(Graphics2D g2D, Area setArea, boolean drawOutline, boolean makeOutlineTranslucent, boolean doFill){}
	// ... might want to have an option to decide whether to anti-alias or not
	// ... but I think it is best to have this in the general paint method (eg that of the panel) that invokes this
	
	
	public HeterogeneousChannelStyle clone(){
		return new HeterogeneousChannelStyle(this.ref, this.title, this.fillColour, this.outlineColour, this.outlineThickness);
	}
}
