/* 
 * 
 * eulerGlyphs v0.9.0
 * 
 * 2013-12-18
 *
 * 
 * 
 * eulerGlyphs -- Drawing Area-Proportional Euler Diagrams With Glyphs	    
 * 		http://www.eulerdiagrams.org/eulerGlyphs
 * 
 * 
 * 		Copyright (C) 2012-2013, Luana Micallef. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerGlyphs.
 * 			
 * 		eulerGlyphs is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerGlyphs is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerGlyphs (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */

 


package visstyle;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Graphics2D;
import java.awt.geom.Area;


/**
 * To instantiate, define and handle an outline, heterogeneous channel type visualization style
 * 
 */


public class HeterogeneousChannelStyleOutline extends HeterogeneousChannelStyle {
	
	protected static final String TITLE = "Outline";
	
	protected static final Color FILL_COLOUR = null;
	protected static final Color OUTLINE_COLOUR = COLOUR_BLUE;
	protected static final int OUTLINE_THICKNESS = 2;

	// Constructor
	public HeterogeneousChannelStyleOutline(String ref){
		super (ref, TITLE, FILL_COLOUR, OUTLINE_COLOUR, OUTLINE_THICKNESS);
	}
	
	
	// Implementing Overridden Methods
	
	public Boolean canBeDrawnWithoutABorder(){return false;}
	
	public void paintArea(Graphics2D g2D, Area setArea, boolean drawOutline, boolean makeOutlineTranslucent, boolean doFill){
		paintArea(g2D, setArea, makeOutlineTranslucent);
	}
	public void paintArea(Graphics2D g2D, Area setArea, boolean makeOutlineTranslucent){
		// outline
		Composite prevComposite;
	    prevComposite = g2D.getComposite();
	    Composite c = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, .35f); //the smaller the value .4f the more transparent
	    if (makeOutlineTranslucent){g2D.setComposite(c);}
		g2D.setPaint(super.outlineColour);
		g2D.setStroke(new BasicStroke(super.outlineThickness)); 
		g2D.draw(setArea);
	    if (makeOutlineTranslucent){g2D.setComposite(prevComposite);}
	}
	
	
	public String toString(){
		return title;
	}
	
	public HeterogeneousChannelStyleOutline clone(){
		HeterogeneousChannelStyleOutline clone = new HeterogeneousChannelStyleOutline(this.ref);
		clone.title = this.title;
		clone.fillColour = this.fillColour;
		clone.outlineColour = this.outlineColour;
		clone.outlineThickness = this.outlineThickness;
		return clone;
	}
}
