/* 
 * 
 * eulerGlyphs v0.9.0
 * 
 * 2013-12-18
 *
 * 
 * 
 * eulerGlyphs -- Drawing Area-Proportional Euler Diagrams With Glyphs	    
 * 		http://www.eulerdiagrams.org/eulerGlyphs
 * 
 * 
 * 		Copyright (C) 2012-2013, Luana Micallef. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerGlyphs.
 * 			
 * 		eulerGlyphs is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerGlyphs is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerGlyphs (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */

 


package visstyle;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Graphics2D;
import java.awt.TexturePaint;
import java.awt.geom.Area;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.IOException;
import javax.imageio.ImageIO;
import utils.ReflectionUtils;


/**
 * To instantiate, define and handle a texture, heterogeneous channel type visualization style
 * 
 */


public class HeterogeneousChannelStyleTexture extends HeterogeneousChannelStyle {
	
	private static final String TITLE = "Texture";
	
	protected static final Color FILL_COLOUR = null;
	protected static final Color OUTLINE_COLOUR = COLOUR_GREEN;
	protected static final int OUTLINE_THICKNESS = 1;
	
	// for paint component for the texture fill
	protected BufferedImage textureImage;
	protected Rectangle2D textureRect;
	protected TexturePaint texturePaint;
	
	
	// Constructor
	public HeterogeneousChannelStyleTexture(String ref) {
		super (ref, TITLE, FILL_COLOUR, OUTLINE_COLOUR, OUTLINE_THICKNESS);
		initTextureFillUtils();
	}
	
	
	public void initTextureFillUtils(){
		
		// for paint component -> for texture fill
		try {
			textureImage = ImageIO.read(getClass().getResourceAsStream("/visstyle/TextureStripes.jpg")); 
			textureRect = new Rectangle2D.Double(0, 0.5, 8.5, 8);
		    texturePaint = new TexturePaint(textureImage, textureRect);
		} catch (IOException e) {
			System.out.println(ReflectionUtils.getClassName(1)+"."+ ReflectionUtils.getMethodName(1)+": "+e);
		}
	}
	
	// Implementing Overridden Methods
	
	public Boolean canBeDrawnWithoutABorder(){return false;}
	
	public void paintArea(Graphics2D g2D, Area setArea, boolean drawOutline, boolean makeOutlineTranslucent, boolean doFill){
		paintArea(g2D, setArea);
	}
	public void paintArea(Graphics2D g2D, Area setArea){
		Composite prevComposite;
		g2D.setColor(super.outlineColour);
	    prevComposite = g2D.getComposite();
	    Composite c = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, .4f);
	    g2D.setComposite(c);
	    g2D.setPaint(texturePaint);
		g2D.fill(setArea);
		g2D.setComposite(prevComposite);
		// outline
		g2D.setPaint(outlineColour);
		g2D.setStroke(new BasicStroke(super.outlineThickness));
		g2D.draw(setArea);
	}
	
	
	public String toString(){
		return title;
	}
	
	public HeterogeneousChannelStyleTexture clone(){
		HeterogeneousChannelStyleTexture clone = new HeterogeneousChannelStyleTexture(this.ref);
		clone.title = this.title;
		clone.fillColour = this.fillColour;
		clone.outlineColour = this.outlineColour;
		clone.outlineThickness = this.outlineThickness;
		return clone;
	}
	
}
